# -*- coding: iso-8859-2 -*-

#	Programmer:	Piotr Borek
#	E-mail:		piotrborek@op.pl
#
#	Copyright 2006  Piotr Borek
#
#	Distributed under the terms of the GPL (GNU Public License)
#
#    pyX4 is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import lib.core
import lib.config
import lib.draw
import gui.main_gui
import conf.base
import widget.dialogs

from lib.splash import SplashWindow
SplashWindow.pulse_text_import("conf.editor.colors")

import gtk
import os

schemes = [
    'custom',
    'default',
    'bluebird',
    'darkblue',
    'mc'
]

class ColorsProperty(conf.base.PropertyBase):
    add_color_property = None
    set_color_scheme = None
    property_reinit = None
    color_scheme = 'default'
    def __init__(self):
        """Konstruktor"""
	conf.base.PropertyBase.__init__(self, _("Color preferences: All colors"))
        ColorsProperty.add_color_property = self.add_color_property
        ColorsProperty.set_color_scheme = self.set_color_scheme
        ColorsProperty.property_reinit = self.property_reinit
	self.config_file = lib.config.ConfigureSection(self, 'Editor.Colors')
        self.liststore = gtk.ListStore(str)
        self.treeview = gtk.TreeView(self.liststore)
        for s in schemes:
            self.liststore.append([s])
        self.search_local()
        tvcolumn = gtk.TreeViewColumn(_("Choose a color scheme"))
        celltext = gtk.CellRendererText()
        tvcolumn.pack_start(celltext, True)
        tvcolumn.set_attributes(celltext, text=0)
        self.treeview.append_column(tvcolumn)
        self.treeview.show_all()
        self.pack_start(self.treeview, True, True)
        self.button = gtk.Button(_("Save custom color scheme"))
        self.button.connect('clicked', self.button_clicked_event)
        self.button.set_sensitive(False)
        self.button.show()
        self.pack_start(self.button, False, False)
	self.read_config()
        self.property_list = []
    def add_color_property(self, color_property, syntax_name):
        gui.main_gui.preferences.add('editor.colors.%s' % syntax_name.lower(), _('Editor.Colors.%s') % syntax_name, color_property)
        self.property_list.append(color_property)
    def search_local(self):
        p = os.environ['HOME'] + '/.pyX4/'
        for f in os.listdir(p):
            s = f.split('.')
            if s[-1]=='color':
                self.liststore.append([s[0]])
    def check(self, color_scheme):
        iter = self.liststore.get_iter_root()
        while iter:
            v = self.liststore.get_value(iter, 0)
            if v==color_scheme:
                return self.liststore.get_path(iter)
            iter = self.liststore.iter_next(iter)
        return None
    def set_color_scheme(self, color_scheme):
        path = self.check(color_scheme)
        if path:
            ColorsProperty.color_scheme = color_scheme
            self.treeview.set_cursor(path)
        if color_scheme=='custom':
            self.button.set_sensitive(True)
        else:
            self.button.set_sensitive(False)
    def property_reinit(self):
        for x in self.property_list:
            x.end_init()        
    def show_widget(self):
        pass
    def apply_clicked(self, button, data=None):
        path, column = self.treeview.get_cursor()
        iter = self.liststore.get_iter(path)
        self.set_color_scheme(self.liststore.get_value(iter, 0))
        self.property_reinit()
        lib.core.events.emit_event('conf.editor.colors')
    def read_config(self):
        if self.config_file.has_option('Color scheme'):
            color_scheme = self.config_file.get('Color scheme')
            self.set_color_scheme(color_scheme)
    def update_config(self):
        self.config_file.set('Color scheme', ColorsProperty.color_scheme)
    def button_clicked_event(self, button, data=None):
        data = {}
        for x in self.property_list:
            attr = x.get_colors().get_dict()
            for n, v in attr.iteritems():
                data[n] = v
        dialog = widget.dialogs.InputDialog(_("Enter new scheme name"))
        response = dialog.run()
        for x in schemes:
            if x==dialog.get_text():
                response = gtk.RESPONSE_REJECT
                message = gtk.MessageDialog(None, gtk.DIALOG_MODAL, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, _("Filename \"%s\" exists and\ncannot be overwrite !") % x)
                message.run()
                message.destroy()
                dialog.destroy()
                return
        if response==gtk.RESPONSE_ACCEPT:
            p = os.environ['HOME'] + '/.pyX4/' + dialog.get_text() + '.color'
            try:
                f = open(p, 'r')
                f.close()
                message = gtk.MessageDialog(None, gtk.DIALOG_MODAL, gtk.MESSAGE_WARNING, gtk.BUTTONS_YES_NO, _("Filename \"%s\" exists.\nOverwrite ?") % dialog.get_text())
                response = message.run()
                message.destroy()
                if response==gtk.RESPONSE_NO:
                    dialog.destroy()
                    return
            except:
                pass
            f = open(p, 'w')
            f.write('#\n# This file was generated\n#\n\n')
            for k, v in data.iteritems():
                c = '#%02X%02X%02X' % (v.color.red/257, v.color.green/257, v.color.blue/257)
                s = 'NORMAL'
                if (v.font_style & lib.draw.FONT_STYLE_BOLD)==lib.draw.FONT_STYLE_BOLD:
                    s = 'BOLD'
                    if (v.font_style & lib.draw.FONT_STYLE_ITALIC)==lib.draw.FONT_STYLE_ITALIC:
                        s = s + ' | ITALIC'
                elif (v.font_style & lib.draw.FONT_STYLE_ITALIC)==lib.draw.FONT_STYLE_ITALIC:
                    s = 'ITALIC'
                f.write("set_color(\"%s\", \"%s\", %s)\n" % (k, c, s))
            f.close()
            self.liststore.append([dialog.get_text()])
        dialog.destroy()
