# -*- coding: iso-8859-2 -*-

#	Programmer:	Piotr Borek
#	E-mail:		piotrborek@op.pl
#
#	Copyright 2006  Piotr Borek
#
#	Distributed under the terms of the GPL (GNU Public License)
#
#    pyX4 is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import lib.core
import lib.config
import conf.base
import conf.null
import conf.plugins
import conf.editor.general
import conf.editor.font
import conf.editor.lineview
import conf.editor.indent
import conf.editor.colors
import conf.terminal.font

from lib.splash import SplashWindow
SplashWindow.pulse_text_import("conf.preferences")

import gtk
import types

class PreferencesBox(gtk.VBox):
    """
    Preferences Box class. You can add your own preferences widgets to it.
    """
    def __init__(self):
	gtk.VBox.__init__(self)
	self.config_file = lib.config.ConfigureSection(self, 'PreferencesBox')
	self.my_widget = conf.null.NullProperty()
	self.my_widget.show()
        self.dictionary = {}
	self.connect("delete-event", self.delete_event)
	self.hpaned = gtk.HPaned()
	self.hpaned.show()
        if self.config_file.has_option("hpaned_position"):
            hpos = self.config_file.get_int("hpaned_position")
        else:
            hpos = 120
	self.hpaned.set_position(hpos)
	self.pack_start(self.hpaned, True, True)
	self.store = gtk.TreeStore(str, str)
	treeview = gtk.TreeView(self.store)
	treeview.set_headers_visible(False)
	treeview.connect("cursor-changed", self.cursor_changed_signal);
	treeview.show()
	tvcolumn = gtk.TreeViewColumn(None)
	treeview.append_column(tvcolumn)
	cell = gtk.CellRendererText()
	tvcolumn.pack_start(cell, True)
	tvcolumn.add_attribute(cell, 'text', 0)
	scroll = gtk.ScrolledWindow()
	scroll.show()
	scroll.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
	scroll.add(treeview)
	self.hpaned.pack1(scroll)
        self.main_vbox = gtk.VBox()
        self.main_vbox.show()
        self.main_vbox.pack_start(self.my_widget, True, True, 0)
        self.hpaned_widget = self.my_widget
        self.hpaned.pack2(self.main_vbox)
        self.init_properties()
    def init_properties(self):
        self.add('editor.general', _("Editor.General"), conf.editor.general.GeneralProperty())
        self.add('editor.font', _("Editor.Font"), conf.editor.font.FontProperty())
        self.add('editor.lineview', _("Editor.Lineview"), conf.editor.lineview.LineviewProperty())
        self.add('editor.indentation', _("Editor.Indentation"), conf.editor.indent.IndentProperty())
        self.add('editor.colors', _("Editor.Colors"), conf.editor.colors.ColorsProperty())
        self.add('terminal.font', _("Terminal.Font"), conf.terminal.font.FontProperty())
        self.add('plugins', _("Plugins"), conf.plugins.PluginsProperty())
    def update_config(self):
        hpos = self.hpaned.get_position()
        self.config_file.set("hpaned_position", hpos)
    def find(self, iter, string):
	if iter==None:
	    iter = self.store.get_iter_root()
	    if iter==None:
		return None
	else:
	    iter = self.store.iter_children(iter)
	    if iter==None:
		return None
	v = self.store.get_value(iter, 0)
	while v!=string:
	    iter = self.store.iter_next(iter)
	    if iter==None:
		return None
	    v = self.store.get_value(iter, 0)
	return iter
    def add(self, prop_name, full_path, widget=None):
        prop_name = "conf." + prop_name
        if isinstance(widget, conf.base.PropertyBase):
            widget.hide()
            widget.apply_button.connect('clicked', self.apply_button_clicked, prop_name)
            lib.core.share_data.register_data(prop_name, widget.get_static())
	    lib.core.events.register_event(lib.core.share_data, prop_name)
            self.dictionary[prop_name] = widget
            self.main_vbox.pack_start(widget, True, True, 0)
	path = full_path.split(".")
	tmp = None
	for x in path:
	    iter = self.find(tmp, x)
	    if iter==None:
                if x==path[-1]:
		    tmp = self.store.append(tmp, [x, prop_name])
                else:
                    tmp = self.store.append(tmp, [x, ''])
	    else:
		tmp = iter
    def delete_event(self, widget, event, data=None):
        self.hpaned_widget.hide_widget()
	return True
    def show_preferences(self):
        self.hpaned_widget.show_widget()
    def cursor_changed_signal(self, treeview, data=None):
	selection = treeview.get_selection()
	model, iter = selection.get_selected()
        path = model.get_value(iter, 1)
        self.hpaned_widget.hide()
        self.hpaned_widget.hide_widget()
        try:
            self.hpaned_widget = self.dictionary[path]
        except KeyError:
            self.hpaned_widget = self.my_widget
        self.hpaned_widget.show()
        self.hpaned_widget.show_widget()
    def apply_button_clicked(self, button, sig_name):
        lib.core.events.emit_event(sig_name)
