# -*- coding: iso-8859-2 -*-

#	Programmer:	Piotr Borek
#	E-mail:		piotrborek@op.pl
#
#	Copyright 2006  Piotr Borek
#
#	Distributed under the terms of the GPL (GNU Public License)
#
#    pyX4 is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

__all__ = ["Application"]

import lib.core
import lib.config
import lib.tools
import lib.text
import lib.tools
import gui.main_gui
import widget.text
import widget.filedialog
import widget.terminal
import syntax.main

from lib.splash import SplashWindow
SplashWindow.pulse_text_import("lib.main_app")

import gtk

class Application:
    """
    Glowna klasa, od ktorej zaczyna sie dzialanie calego programu.
    Tworzy rowniez wyglad glownego okna - uzywa do tego celu zmiennych
    z pliku gui.main_gui.
    """
    def __init__(self, width=640, height=480):
        """
        Konstruktor
        @param width: szerokosc tworzonego okna
        @param height: wysokosc tworzonego okna
        """
	self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
	self.fullscreen = False
	self.window.set_size_request(320,200)
	self.x_pos = -1
	self.y_pos = -1
	self.config_file = lib.config.ConfigureSection(self, 'MainWindow')
	if self.config_file.has_option("width"):
	    width = self.config_file.get_int("width")
	if self.config_file.has_option("height"):
	    height = self.config_file.get_int("height")
	if self.config_file.has_option("x_pos"):
	    self.x_pos = self.config_file.get_int("x_pos")
	if self.config_file.has_option("y_pos"):
	    self.y_pos = self.config_file.get_int("y_pos")
	self.window.resize(width, height)
	if (self.x_pos!=-1) & (self.y_pos!=-1):
	    self.window.move(self.x_pos, self.y_pos)
	self.width = width
	self.height = height
	self.window.set_title('pyX4')
	self.window.set_resizable(True)
	self.window.connect('destroy', self.destroy_event)
	self.window.connect('delete-event', self.delete_event)
	self.window.connect('configure-event', self.configure_event)
        self.window.connect('key-press-event', self.keypress_event)
	if self.config_file.has_option("fullscreen"):
	    temp = self.config_file.get("fullscreen")
	    if temp=="True":
		self.fullscreen_clicked()
        self.create_events()
    def create_gui(self):
        """Funckja ta tworzy gui dla glownego okna"""
        self.window.add(gui.main_gui.vbox)
	if self.config_file.has_option("hpaned_position"):
	    gui.main_gui.hpaned.set_position(self.config_file.get_int("hpaned_position"))
	else:
	    gui.main_gui.hpaned.set_position(128)
	gui.main_gui.vbox.pack_start(gui.main_gui.menu, False, False, 0)
	gui.main_gui.vbox.pack_start(gui.main_gui.toolbar, False, False, 0)
	gui.main_gui.vbox.pack_start(gui.main_gui.hbox, True, True, 0)
	gui.main_gui.hbox.pack_start(gui.main_gui.plugintoolbar, False, False, 0)
	gui.main_gui.hbox.pack_start(gui.main_gui.hpaned, True, True, 0)
	gui.main_gui.hpaned.pack2(gui.main_gui.notebook)
	gui.main_gui.vbox.pack_start(gui.main_gui.statusbar, False, False, 0)
    def init_syntax(self):
        syntax.main.initialize_syntax()
    def create_events(self):
        lib.core.events.register_event(self, 'main.file.open')
    def main(self):
        """Petla glowna programu"""
        self.window.show()
        self.new_file('/home/piotr/programowanie/Python/pyX4-gray/test.cpp')
	gtk.main()
    def create_textview(self, file_name=None, encoding=None):
        text_view = widget.text.TextView()
        text_view.show()
        buffer = lib.text.TextBuffer()
        if file_name==None:
            buffer.append_string(lib.text.String(u""))
        else:
            p = lib.core.share_data.get_data('conf.editor.general')
            if encoding:
                buffer.read_file(file_name, encoding)
            else:
                buffer.read_file(file_name, p.encoding)
        controller = text_view.get_text_controller()
        controller.set_text_buffer(buffer, 0)
        if file_name:
            r, n = lib.tools.get_file_type(file_name)
            controller.set_syntax(n)
        return text_view        
    def new_file(self, file_name=None, encoding=None):
        """Creates new text(empty) widget in notebook or open selected file by file chooser dialog"""
        lib.core.events.emit_event('main.file.open', [file_name])
    def open_file_dialog(self):
        """Shows new file chooser dialog for opening files"""
        fc = widget.filedialog.FileDialog(action=gtk.FILE_CHOOSER_ACTION_OPEN)
        response = fc.run()
        if response==gtk.RESPONSE_ACCEPT:
            self.new_file(fc.get_filename(), fc.get_encoding())
        fc.destroy()
    def show_about_dialog(self):
        dialog = gtk.AboutDialog()
        dialog.set_name('pyX4')
        dialog.set_authors(['Piotr Borek'])
        dialog.set_version('1.0')
        logo = gtk.gdk.pixbuf_new_from_file("graph/X4logo.png")
        dialog.set_logo(logo)
        dialog.set_copyright(_("Simple editor written in python\nCopyright 2005-2007 Piotr Borek"))
        dialog.set_license(lib.GPL.license)
        dialog.run()
        dialog.destroy()
    def exit_enabled(self):
        """Funckja zwraca wartosc True, jesli uzytkownik ma prawo do zamkniecia glownego okna"""
	return True
    def fullscreen_change(self):
        """
        Funckja ta powoduje przejscie do trypu pelnoekranowego lub odwrotnie - z
        trybu pelnoekranowego do trybu okienkowego.
        """
	if self.fullscreen:
	    self.window.unfullscreen()
	    self.fullscreen = False
	else:
	    self.window.fullscreen()
	    self.fullscreen = True
    def exit_application(self):
        """
        Funckja ta powoduje wyjscie z programu pod warunkiem, ze funkcja exit_enabled()
        zwroci wartosc True.
        """
	if self.exit_enabled():
	    self.window.destroy()
    def update_config(self):
        """Funkcja uaktualniajaca plik konfiguracyny programu"""
	self.config_file.set("width", self.width)
	self.config_file.set("height", self.height)
	self.config_file.set("hpaned_position", gui.main_gui.hpaned.get_position())
	self.config_file.set("fullscreen", self.fullscreen)
	self.config_file.set("x_pos", self.x_pos)
	self.config_file.set("y_pos", self.y_pos)
    def destroy_event(self, widget, data=None):
	self.config_file.save_all()
	gtk.main_quit()
    def delete_event(self, widget, event, data=None):
	self.x_pos, self.y_pos = self.window.get_position()
	return not self.exit_enabled()
    def configure_event(self, widget, event, data=None):
	if not self.fullscreen:
	    self.width = event.width
	    self.height = event.height
    def keypress_event(self, w, event, data=None):
        gui.main_gui.notebook.emit("key-press-event", event)
        if isinstance(gui.main_gui.notebook.get_current_widget(), widget.text.TextView):
            return True
        if isinstance(gui.main_gui.notebook.get_current_widget(), widget.terminal.Terminal):
            return True
        return False
