# -*- coding: iso-8859-2 -*-

#	Programmer:	Piotr Borek
#	E-mail:		piotrborek@op.pl
#	Note:		You must reply to the verification e-mail to get through.
#
#	Copyright 2006  Piotr Borek
#
#	Distributed under the terms of the GPL (GNU Public License)
#
#    pyX4 is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import lib.indent.options
import widget.menus
import gui.callbacks
import gui.main_gui

from lib.splash import SplashWindow
SplashWindow.pulse_text_import("gui.menu")

import gtk
import lib.GPL

class FileMenu(widget.menus.SubMenu):
    def __init__(self):
        super(FileMenu, self).__init__(_('File'))
        self.create_item('New', _('New'), None, 'gtk-new', gui.callbacks.new_file_clicked)
        self.create_item('Open', _('Open'), None, 'gtk-open', gui.callbacks.open_file_clicked)
        self.create_item('Save', _('Save'), None, 'gtk-save', gui.callbacks.save_file_clicked)
        self.create_item('Save As', _('Save as'), None, 'gtk-save-as', gui.callbacks.save_file_as_clicked)
        self.create_item('Save All Files', _('Save all files'), None, None, gui.callbacks.save_all_files_clicked)
        self.create_separator()
        self.create_item('Exit', _('Exit'), None, 'gtk-quit', gui.callbacks.exit_app_clicked)
        self.show()

class EditMenu(widget.menus.SubMenu):
    def __init__(self):
        super(EditMenu, self).__init__(_('Edit'))
        self.filetype_menu = widget.menus.RadioGroupSubMenu(_('Set filetype'), self.toggled_event)
        self.filetype_menu.show()
        self.append(self.filetype_menu)
        self.show()
    def toggled_event(self, widget, data=None):
        if widget.get_active():
            textview = gui.main_gui.notebook.get_current_widget()
            controller = textview.get_text_controller()
            controller.set_syntax(data)

class ProjectMenu(widget.menus.SubMenu):
    def __init__(self):
        super(ProjectMenu, self).__init__(_('Project'))
        self.show()

class IndentMenu(widget.menus.SubMenu):
    def __init__(self):
        super(IndentMenu, self).__init__(_('Indent'))
        check = self.create_checkitem(_('Enable Indent'), self.enable_switch)
        check.set_active(False)
        self.create_separator()
        self.none = self.create_radioitem(None, _('None'))
        self.gnu =  self.create_radioitem(self.none, _('GNU style'))
        self.kr =   self.create_radioitem(self.none, _('Kernighan & Ritchie style'))
        self.orig = self.create_radioitem(self.none, _('Berkeley style'))
        self.create_separator()
        self.custom = self.create_submenu(_('Custom'))
        self.custom_check = self.custom.create_checkitem(_('Enable custom style'), self.custom_switch)
        self.custom_check.set_active(False)
        self.load = self.custom.create_submenu(_('Load style'))
        self.custom.create_submenu(_('Save style'))
        self.custom.create_separator()
        self.create_custom_options()
        self.set_custom_sensitive(False)
        self.none.set_active(True)
        self.set_sensitive(False)
        self.custom.set_sensitive(False)
        self.show()
    def create_custom_options(self):
        self.custom_menus = [
            self.custom.create_submenu(_('Blank lines')),
            self.custom.create_submenu(_('Long lines')),
            self.custom.create_submenu(_('Declarations')),
            self.custom.create_submenu(_('Statemants')),
            self.custom.create_submenu(_('Comments')),
            self.custom.create_submenu(_('Indentation')),
            self.custom.create_submenu(_('Miscellaneous'))
        ]
        for x in lib.indent.options.list:
            if x[2]==lib.indent.options.BLANK_LINES:
                self.custom_menus[0].create_checkitem(x[1])
            if x[2]==lib.indent.options.LONG_LINES:
                self.custom_menus[1].create_checkitem(x[1])
            if x[2]==lib.indent.options.DECLARATIONS:
                self.custom_menus[2].create_checkitem(x[1])
            if x[2]==lib.indent.options.STATEMANTS:
                self.custom_menus[3].create_checkitem(x[1])
            if x[2]==lib.indent.options.COMMENTS:
                self.custom_menus[4].create_checkitem(x[1])
            if x[2]==lib.indent.options.INDENTATION:
                self.custom_menus[5].create_checkitem(x[1])
            if x[2]==lib.indent.options.MISCELLANEOUS:
                self.custom_menus[6].create_checkitem(x[1])
    def set_custom_sensitive(self, sensitive):
        for x in self.custom_menus:
            x.set_sensitive(sensitive)
        self.load.set_sensitive(sensitive)
    def set_sensitive(self, sensitive):
        self.none.set_sensitive(sensitive)
        self.gnu.set_sensitive(sensitive)
        self.kr.set_sensitive(sensitive)
        self.orig.set_sensitive(sensitive)
    def enable_switch(self, w, data=None):
        self.none.set_active(w.get_active())
        self.custom_check.set_active(not w.get_active())
        self.set_sensitive(w.get_active())
        self.custom.set_sensitive(w.get_active())
    def custom_switch(self, w, data=None):
        self.set_sensitive(not w.get_active())
        self.set_custom_sensitive(w.get_active())

        

class HelpMenu(widget.menus.SubMenu):
    def __init__(self):
        super(HelpMenu, self).__init__(_('Help'))
        self.create_item('About', _('About'), None, 'gtk-about', gui.callbacks.about_clicked)
        self.show()

class MenuBar(gtk.MenuBar):
    """Klasa, ktora definiuje glowne menu aplikacji pyX4."""
    def __init__(self):
        """Konstruktor"""
	gtk.MenuBar.__init__(self)
	# file menu
        self.file_menu = FileMenu()
        self.append(self.file_menu)
        # edit menu
        self.edit_menu = EditMenu()
        self.append(self.edit_menu)
        # project menu
        self.project_menu = ProjectMenu()
        self.append(self.project_menu)
        # ident menu
        self.indent_menu = IndentMenu()
        self.append(self.indent_menu)
	# help menu
        self.help_menu = HelpMenu()
        self.append(self.help_menu)
