# -*- coding: iso-8859-2 -*-

#	Programmer:	Piotr Borek
#	E-mail:		piotrborek@op.pl
#
#	Copyright 2006  Piotr Borek
#
#	Distributed under the terms of the GPL (GNU Public License)
#
#    pyX4 is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from lib.splash import SplashWindow
SplashWindow.pulse_text_import("lib.config")

import os
import xml.dom.minidom

class XMLConfig(object):
    def __init__(self, file_name):
        self.file_name = file_name
	self.dir = os.environ['HOME'] + '/.pyX4/'
	exist = True
	try:
	    os.mkdir(self.dir)
	except:
	    pass
	try:
	    f = open(self.dir + file_name, 'r')
	    f.close()
	except IOError:
	    exist = False
        if not exist:
	    f = open(self.dir + file_name, 'w')
	    f.close()
        self.read()
    def get_tag(self, root, tag_id, tag_name, attr_name='name', enable_split=True):
        if enable_split:
            names = tag_name.split('.')
        else:
            names = [tag_name]
        tmp = root
        for x in names:
            tag_list = tmp.getElementsByTagName(tag_id)    
            tag = None
            for t in tag_list:
                if t.getAttribute(attr_name)==x:
                    tag = t
                    tmp = t
            if tag==None:
                break
        return tag
    def create_section(self, section_name):
        names = section_name.split('.')
        tmp = self.config
        for x in names:
            exists = self.get_tag(tmp, 'section', x)
            if exists==None:
                section = self.root.createElement('section')
                section.setAttribute('name', x)
                tmp.appendChild(section)
                tmp = section
            else:
                tmp = exists
        return tmp
    def set(self, section_name, name, value):
        section = self.get_tag(self.config, 'section', section_name)
        if section==None:
            section = self.create_section(section_name)
        key = self.get_tag(section, 'key', name)
        if key==None:
            key = self.root.createElement('key')    
            section.appendChild(key)
        key.setAttribute('name', str(name))
        key.setAttribute('value', str(value))
    def get(self, section_name, name):
        section = self.get_tag(self.config, 'section', section_name)
        if section==None:
            return None
        key = self.get_tag(section, 'key', name)
        if key==None:
            return None
        return key.getAttribute('value')
    def set_list(self, section_name, name, value):
        section = self.get_tag(self.config, 'section', section_name)
        if section==None:
            section = self.create_section(section_name)
        key = self.get_tag(section, 'list', name, enable_split=False)
        if key==None:
            key = self.root.createElement('list')
            key.setAttribute('name', str(name))
            section.appendChild(key)
        for item in value:
            i = self.get_tag(key, 'an', str(item), 'item', False)
            if i==None:
                i = self.root.createElement('an')
                key.appendChild(i)
            i.setAttribute('item', str(item))
    def get_list(self, section_name, name):
        section = self.get_tag(self.config, 'section', section_name)
        if section==None:
            return None
        key = self.get_tag(section, 'list', name)
        if key==None:
            return None
        list = key.getElementsByTagName('an')
        items = []
        for i in list:
            items.append(i.getAttribute('item'))
        return items
    def has_section(self, section_name):
        section = self.get_tag(self.config, 'section', section_name)
        if section:
            return True
        return False
    def has_option(self, section_name, name):
        section = self.get_tag(self.config, 'section', section_name)
        if section==None:
            return False
        key = self.get_tag(section, 'key', name)
        if key==None:
            key = self.get_tag(section, 'list', name)
        if key:
            return True
        return False
    def save(self):
        f = open(self.dir + self.file_name, 'w')
        text = self.root.toprettyxml("    ", "\n").split("\n")
        for x in text:
            if len(x.replace(' ', '')) != 0:
                f.write(x)
                f.write("\n")
        f.close()
    def read(self):
        try:
            self.root = xml.dom.minidom.parse(self.dir + self.file_name)
        except:
            self.root = xml.dom.minidom.parseString('<config program=\"pyX4\"></config>')
        self.config = self.root.getElementsByTagName('config')[0]

class ConfigureSection:
    """
    Klasa, ktrej odziedziczenie powoduje utworzenie odpowiedniej sekcji
    w pliku konfiguracyjnym ".pyx4rc". Klasa dziedziczaca musi wywolac
    konstruktor z odpowiednimi parametrami oraz musi zdefiniowac funkcje
    update_config().
    """
    xml_file = XMLConfig('pyX4config.xml')
    list = []
    def __init__(self, class_ptr, section_name):
        """
        @param class_ptr: wskaznik do klasy macierzystej
        @param section_name: unikalna nazwa podsekcji w pliku konfiguracyjnym
        """
	self.section_name = section_name
#	if not self.file.has_section(section_name):
#	    self.file.add_section(section_name)
	self.register(class_ptr)
    def has_section(self, section_name):
        """
        Funkcja sprawdza czy istnieje definicja sekcji o podanej nazwie
        @return: True lub False
        """
        return self.xml_file.has_section(section_name)
    def has_option(self, option_name):
        """
        Funkcja sprawdza, czy w macierzystej sekcji znajduje sie opcja o podanej nazwie
        @param section_name: nazwa sekcji
        @return: True lub False
        """
        return self.xml_file.has_option(self.section_name, option_name)
    def set(self, value_name, value):
        """
        Funkcja ustawia pare nazwa, wartosc w macierzystej sekcji
        @param value_name: nazwa parametru
        @param value: wartosc parametru
        """
        self.xml_file.set(self.section_name, value_name, value)
    def set_list(self, list_name, list):
        self.xml_file.set_list(self.section_name, list_name, list)
    def get(self, value_name):
        """
        Funckja zwaraca wartosc podanego parametru
        @param value_name: wartosc parametru
        @return: wartosc typu "string" podanego paramteru
        """
        return self.xml_file.get(self.section_name, value_name)
    def get_int(self, value_name):
        """
        Funckja zwaraca wartosc podanego parametru
        @param value_name: wartosc parametru
        @return: wartosc typu "int" podanego paramteru
        """
        return int(self.get(value_name))
    def get_bool(self, value_name):
        """
        Funckja zwaraca wartosc podanego parametru
        @param value_name: wartosc parametru
        @return: wartosc typu "boolean" podanego paramteru
        """
        rval = self.get(value_name)
        if rval == 'True':
            return bool(1)
        return bool(0)
    def get_list(self, list_name):
        return self.xml_file.get_list(self.section_name, list_name)
    def register(self, class_ptr):
        """Funckja wewnetrzna, sluzy do rejestrowania klasy w systemie konfiguracji"""
	self.list.append(class_ptr)
    def save_all(self):
        """
        Funckja wywoluje funckje update_config() dla wszystkich zarejestrowanych klas i
        nagrywa zmiany do pliku.
        """
	for x in self.list:
            try:
	        x.update_config()
            except AttributeError:
                print _("Warning: %s has no update_config() method") % (x.__class__)
        self.xml_file.save()
