# -*- coding: iso-8859-2 -*-

#	Programmer:	Piotr Borek
#	E-mail:		piotrborek@op.pl
#
#	Copyright 2006  Piotr Borek
#
#	Distributed under the terms of the GPL (GNU Public License)
#
#    pyX4 is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from lib.splash import SplashWindow
SplashWindow.pulse_text_import("lib.draw")

import gc
import gtk.gdk
import pango
import cairo

FONT_STYLE_NORMAL = 1
FONT_STYLE_ITALIC = 2
FONT_STYLE_BOLD   = 4

class CairoFontExt(object):
    def __init__(self):
        window = gtk.gdk.get_default_root_window()
        drawable = gtk.gdk.Pixmap(window, 4, 4)
        self.font_family = ''
        self.font_size = 0
        self.current_font_style = FONT_STYLE_NORMAL
        self.ctx_font_n = drawable.cairo_create()
        self.ctx_font_i = drawable.cairo_create()
        self.ctx_font_b = drawable.cairo_create()
        self.ctx_font_ib = drawable.cairo_create()
        self.set_font('Luxi Sans 11')
    def draw_text(self, x, y, text, font_style=None):
        if font_style==None:
            font_style = self.current_font_style        
        fe = self.get_font_extents(font_style)
        y = y + fe[2] - fe[1]
        if font_style & FONT_STYLE_ITALIC:
            if font_style & FONT_STYLE_BOLD:
                self.ctx_font_ib.move_to(x, y)
                self.ctx_font_ib.show_text(text)
            else:
                self.ctx_font_i.move_to(x, y)
                self.ctx_font_i.show_text(text)
        elif font_style & FONT_STYLE_BOLD:
            self.ctx_font_b.move_to(x, y)
            self.ctx_font_b.show_text(text)
        else:
            self.ctx_font_n.move_to(x, y)
            self.ctx_font_n.show_text(text)
    def set_clip_rect(self, rect):
        self.ctx_font_n.reset_clip()
        self.ctx_font_n.rectangle(rect.x, rect.y, rect.width, rect.height)
        self.ctx_font_n.clip()
        self.ctx_font_i.reset_clip()
        self.ctx_font_i.rectangle(rect.x, rect.y, rect.width, rect.height)
        self.ctx_font_i.clip()
        self.ctx_font_b.reset_clip()
        self.ctx_font_b.rectangle(rect.x, rect.y, rect.width, rect.height)
        self.ctx_font_b.clip()
        self.ctx_font_ib.reset_clip()
        self.ctx_font_ib.rectangle(rect.x, rect.y, rect.width, rect.height)
        self.ctx_font_ib.clip()
    def set_drawable(self, drawable):
        self.ctx_font_n = drawable.cairo_create()
        self.ctx_font_i = drawable.cairo_create()
        self.ctx_font_b = drawable.cairo_create()
        self.ctx_font_ib = drawable.cairo_create()
        gc.collect()
        self.__set_font(self.font_family, self.font_size)
    def set_source_pattern(self, pattern):
        self.ctx_font_n.set_source(pattern)
        self.ctx_font_i.set_source(pattern)
        self.ctx_font_b.set_source(pattern)
        self.ctx_font_ib.set_source(pattern)
    def set_source_color(self, color):
        self.ctx_font_n.set_source_color(color)
        self.ctx_font_i.set_source_color(color)
        self.ctx_font_b.set_source_color(color)
        self.ctx_font_ib.set_source_color(color)
    def set_current_font_style(self, font_style):
        self.current_font_style = font_style
    def set_font(self, font_name):
        font_desc = pango.FontDescription(font_name)
        family = font_desc.get_family()
        size = pango.PIXELS(font_desc.get_size())
        if self.font_family==family and self.font_size==size:
            return
        self.font_family = family
        self.font_size = size
        self.__set_font(family, size)
    def __set_font(self, font_family, size):
        self.ctx_font_n.select_font_face(font_family, cairo.FONT_SLANT_NORMAL, cairo.FONT_WEIGHT_NORMAL)
        self.ctx_font_i.select_font_face(font_family, cairo.FONT_SLANT_ITALIC, cairo.FONT_WEIGHT_NORMAL)
        self.ctx_font_b.select_font_face(font_family, cairo.FONT_SLANT_NORMAL, cairo.FONT_WEIGHT_BOLD)
        self.ctx_font_ib.select_font_face(font_family, cairo.FONT_SLANT_ITALIC, cairo.FONT_WEIGHT_BOLD)

        self.ctx_font_n.set_font_size(size + 4)
        self.ctx_font_i.set_font_size(size + 4)
        self.ctx_font_b.set_font_size(size + 4)
        self.ctx_font_ib.set_font_size(size + 4)

        self.font_extents_n = self.ctx_font_n.font_extents()
        self.font_extents_i = self.ctx_font_i.font_extents()
        self.font_extents_b = self.ctx_font_b.font_extents()
        self.font_extents_ib = self.ctx_font_ib.font_extents()
    def get_context(self, font_style=None):
        if font_style==None:
            font_style = self.current_font_style
        if font_style & FONT_STYLE_ITALIC:
            if font_style & FONT_STYLE_BOLD:
                return self.ctx_font_ib
            return self.ctx_font_i
        elif font_style & FONT_STYLE_BOLD:
            return self.ctx_font_b
        return self.ctx_font_n
    def get_text_size(self, text, font_style=None):
        width = 0
        height = 0
        text_extents = None
        font_extents = None
        if font_style==None:
            font_style = self.current_font_style
        if font_style & FONT_STYLE_ITALIC:
            if font_style & FONT_STYLE_BOLD:
                text_extents = self.ctx_font_ib.text_extents(text)
                font_extents = self.font_extents_ib
            else:
                text_extents = self.ctx_font_i.text_extents(text)
                font_extents = self.font_extents_i
        elif font_style & FONT_STYLE_BOLD:
            text_extents = self.ctx_font_b.text_extents(text)
            font_extents = self.font_extents_b
        else:
            text_extents = self.ctx_font_n.text_extents(text)
            font_extents = self.font_extents_n
        width  = int(text_extents[4])
        height = int(font_extents[2] + 1)
        return (width, height)
    def get_text_width(self, text, font_style=None):
	width, height = self.get_text_size(text, font_style)
	return width
    def get_font_extents(self, font_style=None):
        if font_style==None:
            font_style = self.current_font_style
        if font_style & FONT_STYLE_ITALIC:
            if font_style & FONT_STYLE_BOLD:
                return self.font_extents_ib
            return self.font_extents_i
        elif font_style & FONT_STYLE_BOLD:
            return self.font_extents_b
        return self.font_extents_n

class CairoDraw(object):
    def __init__(self):
        self.drawable = None
        self.context = None
        self.clip_rect = None
        self.width = 0
        self.height = 0
        self.color = gtk.gdk.Color(0, 0, 0)
        self.gc = gtk.gdk.get_default_root_window().new_gc()
        self.copy_gc = gtk.gdk.get_default_root_window().new_gc()
        self.font_ext = CairoFontExt()
        self.set_font('Luxi Sans 11')
    def create_size(self, width, height):
        if width==0 or height==0:
            return
        if self.width!=width or self.height!=height:
            window = gtk.gdk.get_default_root_window()
            self.drawable = gtk.gdk.Pixmap(window, width, height)
            self.context = self.drawable.cairo_create()
            self.context.set_source_color(self.color)
            self.width = width
            self.height = height
            self.font_ext.set_drawable(self.drawable)
            gc.collect()
        self.set_clip_rect()
    def scroll_dx(self, dx, x, y, width, height):
        if self.drawable==None or x > self.width or y > self.height:
            return
        if x < 0:
            x = 0
        if y < 0:
            y = 0
        if x + width > self.width:
            width = self.width - x
        if y + height > self.height:
            height = self.height - y
        if dx >= height:
            return
        if dx >=0: # scroll right
            w = width / dx
            r = width % dx
            x += width
            for i in range(1, w):
                self.drawable.draw_drawable(self.gc, self.drawable, x - (i + 1) * dx, y, x - i * dx, y , dx, height)
            if r:
                self.drawable.draw_drawable(self.gc, self.drawable, x - width, y, x - (h - 1) * dx - r, y , r, height)
        else: # scroll left
            dx = -dx
            w = width / dx
            r = width % dx
            for i in range(1, w):
                self.drawable.draw_drawable(self.gc, self.drawable, x + i * dx, y, x + (i - 1) * dx, y, dx, height)
            if r:
                self.drawable.draw_drawable(self.gc, self.drawable, x + h * dx, y, x + (h - 1) * dx, y, r, height)
    def scroll_dy(self, dy, x, y, width, height):
        if self.drawable==None or x > self.width or y > self.height:
            return
        if x < 0:
            x = 0
        if y < 0:
            y = 0
        if x + width > self.width:
            width = self.width - x
        if y + height > self.height:
            height = self.height - y
        if dy >= height:
            return
        if dy >=0: # scroll up
            h = height / dy
            r = height % dy
            y += height
            for i in range(1, h):
                self.drawable.draw_drawable(self.gc, self.drawable, x, y - (i + 1) * dy, x, y - i * dy, width, dy)
            if r:
                self.drawable.draw_drawable(self.gc, self.drawable, x, y - height, x, y - (h - 1) * dy - r, width, r)
        else: # scroll down
            dy = -dy
            h = height / dy
            r = height % dy
            for i in range(1, h):
                self.drawable.draw_drawable(self.gc, self.drawable, x, y + i * dy, x, y + (i - 1) * dy, width, dy)
            if r:
                self.drawable.draw_drawable(self.gc, self.drawable, x, y + h * dy, x, y + (h - 1) * dy, width, r)
    def set_font(self, font_name):
        self.font_ext.set_font(font_name)
    def set_font_description(self, font_desc):
        self.font_ext.set_font(font_desc.to_string())
    def set_font_style(self, font_style):
        self.font_ext.set_current_font_style(font_style)
    def clear(self):
        self.draw_rectangle(True, 0, 0, self.width, self.height)
    def set_color_str(self, str_color):
        self.color = gtk.gdk.color_parse(str_color)
        if self.context==None:
            return
        self.context.set_source_color(self.color)
    def set_color_rgb(self, red, green, blue):
        self.color = gtk.gdk.Color(red * 257, green * 257, blue * 257)
        if self.context==None:
            return
        self.context.set_source_color(self.color)
    def set_color_gdk(self, color):
        if self.context==None:
            return
        self.context.set_source_color(color)
    def set_color_gl(self, red, green, blue):
        self.color = gtk.gdk.Color(int(red * 0xFFFF), int(green * 0xFFFF), int(blue * 0xFFFF))
        if self.context==None:
            return
        self.context.set_source_color(self.color)
    def set_font_color_gdk(self, color):
        self.font_ext.set_source_color(color)
    def set_font_color_rgb(self, red, green, blue):
        self.color = gtk.gdk.Color(red * 257, green * 257, blue * 257)
        self.font_ext.set_source_color(color)
    def set_font_color_gl(self, red, green, blue):
        self.color = gtk.gdk.Color(int(red * 0xFFFF), int(green * 0xFFFF), int(blue * 0xFFFF))
        self.font_ext.set_source_color(color)
    def set_clip_rect(self, rect=None):
        if rect==None:
            rect = gtk.gdk.Rectangle(0, 0, self.width, self.height)
        self.clip_rect = rect
        if self.context==None:
            return
        self.context.reset_clip()
        self.context.rectangle(rect.x, rect.y, rect.width, rect.height)
        self.context.clip()
    def set_font_clip_rect(self, rect=None):
        if rect==None:
            rect = gtk.gdk.Rectangle(0, 0, self.width, self.height)
        self.font_ext.set_clip_rect(rect)
    def draw_rectangle(self, fill, x, y, width, height):
        if self.context==None:
            return
        self.context.rectangle(x, y, width, height)
        if fill:
            self.context.fill()
        else:
            self.context.stroke()
    def draw_text(self, x, y, text):
        if self.context==None:
            return
        self.font_ext.draw_text(x, y, text)
        return self.font_ext.get_context().text_extents(text)[4]
    def get_text_size(self, text, font_style=None):
        return self.font_ext.get_text_size(text, font_style)
    def get_text_width(self, text, font_style=None):
	width, height = self.font_ext.get_text_size(text, font_style)
	return width
    def push_group(self):
        if self.context==None:
            return
        self.context.push_group_with_content(cairo.CONTENT_COLOR_ALPHA)
    def pop_group_to_source(self):
        if self.context==None:
            return
        pattern = self.context.pop_group()
        self.font_ext.set_source_pattern(pattern)
        self.context.set_source(pattern)
    def get_source_pattern(self):
        if self.context==None:
            return None
        return self.context.get_source()
    def set_source_pattern(self, pattern):
        self.font_ext.set_source_pattern(pattern)
        if self.context==None:
            return
        self.context.set_source(pattern)
    def flip(self, target):
        if self.drawable==None:
            return
        target.draw_drawable(self.copy_gc,
                             self.drawable,
                             self.clip_rect.x, self.clip_rect.y,
                             self.clip_rect.x, self.clip_rect.y,
                             self.clip_rect.width, self.clip_rect.height)
