# -*- coding: iso-8859-2 -*-

#	Programmer:	Piotr Borek
#	E-mail:		piotrborek@op.pl
#
#	Copyright 2006  Piotr Borek
#
#	Distributed under the terms of the GPL (GNU Public License)
#
#    pyX4 is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import lib.text
import lib.indent.ast

from lib.splash import SplashWindow
SplashWindow.pulse_text_import('lib.indent.common')

(
    TOKEN_ENDLINE,
    TOKEN_TEXT,
    TOKEN_COMMENT,
    TOKEN_BEGIN_COMMENT,
    TOKEN_END_COMMENT,
    TOKEN_BEGIN_PROC,
    TOKEN_OPEN_BRACKET,
    TOKEN_CLOSE_BRACKET,
    TOKEN_SEMICOLON,
    TOKEN_COMMA
) = range(1, 11)

class Param:
    def __init__(self, lang, buffer):
        self.lang   = lang
        self.buffer = buffer
        self.lnum   = -1
        self.i      = None
        self.line   = None
        self.token  = None
        self.tlen   = None
        self.indent = None
        self.stack  = []

def next_line(p):
    p.i = 0
    p.lnum += 1
    try:
        p.line = p.buffer[p.lnum].get()
    except:
        p.line = None
    return p.line
    

def compare(p, s):
    p.tlen = len(s)
    return p.line[p.i:p.i + len(s)]==s

def check(p, name):
    try:
        s = p.lang[name]
        if compare(p, s):
            p.i += len(s)
            return True
        return False
    except:
        return False

def get_word(p):
    try:
        c = p.line[p.i]
    except:
        return None
    f = p.lang["CHARS"]
    word = u""
    while f.rfind(c) >= 0:
        word += c
        p.i += 1
        try:
            c = p.line[p.i]
        except:
            break
    if word==u"":
        word = c
        p.i += 1
    return word

def skip_whitespace(p):
    p.indent = 0
    try:
        c = p.line[p.i]
        while (c==' ' or c=='\t'):
            p.i += 1
            p.indent += 1
            c = p.line[p.i]
    except:
        pass

def get_token(p):
    skip_whitespace(p)
    if check(p, "COMMENT"):
        p.token = TOKEN_COMMENT
    elif check(p, "BEGIN_COMMENT"):
        p.token = TOKEN_BEGIN_COMMENT
    elif check(p, "END_COMMENT"):
        p.token = TOKEN_END_COMMENT
    elif check(p, "OPEN_BRACKET"):
        p.token = TOKEN_OPEN_BRACKET
    elif check(p, "CLOSE_BRACKET"):
        p.token = TOKEN_CLOSE_BRACKET
    elif check(p, "BEGIN_PROC"):
        p.token = TOKEN_BEGIN_PROC
    elif check(p, "SEMICOLON"):
        p.token = TOKEN_SEMICOLON
    elif check(p, "COMMA"):
        p.token = TOKEN_COMMA
    else:
        s = get_word(p)
        if s:
            p.token = TOKEN_TEXT
            p.stack.append((p.indent, s))
        else:
            p.token = TOKEN_ENDLINE
