# -*- coding: iso-8859-2 -*-

#	Programmer:	Piotr Borek
#	E-mail:		piotrborek@op.pl
#
#	Copyright 2006  Piotr Borek
#
#	Distributed under the terms of the GPL (GNU Public License)
#
#    pyX4 is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import lib.indent.ast
import lib.indent.parse
    
from lib.splash import SplashWindow
SplashWindow.pulse_text_import('lib.indent.indent')

class Indent:
    def __init__(self, text_buffer, ast_tree, lang, opts):
        self.text_buffer = text_buffer
        self.ast_tree = ast_tree
        self.lang = lang
        self.opts = opts
        self.s = None
        self.last = None
    def new_line(self):
        self.s = lib.text.String()
        self.text_buffer.append_string(self.s)
    def append_string(self, text, ast=None):
        if ast:
            self.s.append_string(u" " * ast.indent)
        self.s.append_string(text)
    def disable_opt(self, name):
        rval = None
        if self.opts.has_key(name):
            rval = self.opts.pop(name)
        return rval
    def enable_opt(self, name, value):
        if value:
            self.opts[name] = value
    def create_small_comment(self, ast):
        self.append_string(self.lang['COMMENT'], ast)
        self.append_string(ast.comment)
    def create_big_comment(self, ast):
            self.append_string(self.lang['BEGIN_COMMENT'], ast)
            if self.opts.has_key("fmc"):
                self.new_line()
            for i, x in enumerate(ast.comments):
                if self.opts.has_key("fmc"):
                    self.append_string(self.lang['CONT_COMMENT'], ast)
                    if i > 0:
                        b = 0
                        for j in range(0, ast.indent + len(self.lang['CONT_COMMENT'])):
                            if x[j]==u" " or x[j]==u"\t":
                                b += 1
                            else:
                                break
                        if x[b] != u" " or x[b] != u"\t":
                            self.append_string(u" ")
                        self.append_string(x[b:])
                    else:
                        if x[0] != u" " or x[0] != u"\t":
                            self.append_string(u" ")
                        self.append_string(x)
                else:
                    self.append_string(x)
                if len(ast.comments) != i + 1:
                    self.new_line()
            if self.opts.has_key("fmc"):
                self.new_line()
                self.append_string(u" " * (ast.indent + 1))
            self.append_string(self.lang['END_COMMENT'])
            if self.opts.has_key("fmc"):
                self.new_line()
    def create_function(self, ast):
        fmc = self.disable_opt("fmc")
        self.main_loop(ast.rtype)
        self.main_loop(ast.name)
        self.main_loop(ast.args)
        self.enable_opt("fmc", fmc)
    def create_text(self, ast):
        self.append_string(ast.text)
    def create_endline(self, ast):
        if not (self.opts.has_key("rel") and self.last==lib.indent.ast.END_LINE):
            self.new_line()
    def do(self, ast):
        if ast.type==lib.indent.ast.SMALL_COMMENT:
            self.create_small_comment(ast)
        elif ast.type==lib.indent.ast.BIG_COMMENT:
            self.create_big_comment(ast)
        elif ast.type==lib.indent.ast.FUNCTION:
            self.create_function(ast)
        elif ast.type==lib.indent.ast.TEXT:
            self.create_text(ast)
        elif ast.type==lib.indent.ast.END_LINE:
            self.create_endline(ast)    
    def main_loop(self, ast_tree):
        for ast in ast_tree:
            self.do(ast)
            self.last = ast.type        
    def indent(self):
        self.text_buffer.clear()
        self.new_line()
        self.last = lib.indent.ast.END_LINE
        self.main_loop(self.ast_tree)
        if len(self.s)==0:
            self.text_buffer.pop_string()
        
def test():
    import lib.indent.langdef
    import lib.text
    file = lib.text.TextBuffer()
    file.read_file('lib/indent/test.in', 'iso8859-2')
    ast = lib.indent.parse.create_ast(file, lib.indent.langdef.c)
    indent = Indent(file, ast, lib.indent.langdef.c, {"fmc" : 1, "rel" : 1})
    indent.indent()
    file.write_file_as('lib/indent/test.out')
