# -*- coding: iso-8859-2 -*-

#	Programmer:	Piotr Borek
#	E-mail:		piotrborek@op.pl
#
#	Copyright 2006  Piotr Borek
#
#	Distributed under the terms of the GPL (GNU Public License)
#
#    pyX4 is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import lib.text
import lib.indent.ast
from lib.indent.common import *

from lib.splash import SplashWindow
SplashWindow.pulse_text_import('lib.indent.parse')


def pop_text(ast_tree):
    x = -1
    while ast_tree[x].type==lib.indent.ast.END_LINE:
        x -= 1
    if ast_tree[x].type==lib.indent.ast.TEXT:
        rval = []
        while x < 0:
            rval.insert(0, ast_tree.pop())
            x += 1
        return rval
    return None

def ast_create_small_comment(p):
    ast = lib.indent.ast.SmallComment(p.line[p.i:])
    ast.indent = p.indent
    return ast

def ast_create_big_comment(p):
    comments = []
    ast = lib.indent.ast.BigComment(comments)
    ast.indent = p.indent
    line = u""
    get_token(p)
    while p.token!=TOKEN_END_COMMENT:
        if p.token==TOKEN_ENDLINE:
            comments.append(line)
            next_line(p)
            line = u""
        else:
            indent, s = p.stack.pop()
            line += u" " * p.indent + s
        get_token(p)
    comments.append(line + u" " * p.indent)
    return ast

def ast_create_text(p):
    s = u""
    while len(p.stack) > 0:
        indent, text = p.stack.pop()
        s = u" " * indent + text + s
    ast = lib.indent.ast.Text(s)
    return ast

def ast_create_lang_text(p, name):
    return lib.indent.ast.Text(u" " * p.indent + p.lang[name])

def ast_create_function(p, rtype, name):
    args = [ ast_create_lang_text(p, 'OPEN_BRACKET') ]
    process = p.line
    while process != None and p.token != TOKEN_CLOSE_BRACKET:
        get_token(p)
        process = create_ast_iter(p, args)
    ast = lib.indent.ast.Function(rtype, name, args)
    return ast

def create_ast_iter(p, ast_tree):
    process = p.line
    if p.token==TOKEN_ENDLINE:
        ast_tree.append(lib.indent.ast.EndLine())
        process = next_line(p)
    elif p.token==TOKEN_COMMENT:
        ast_tree.append(ast_create_small_comment(p))
        ast_tree.append(lib.indent.ast.EndLine())
        process = next_line(p)
    elif p.token==TOKEN_BEGIN_COMMENT:
        ast_tree.append(ast_create_big_comment(p))
    elif p.token==TOKEN_TEXT:
        ast_tree.append(ast_create_text(p))
    elif p.token==TOKEN_OPEN_BRACKET:
        name = pop_text(ast_tree)
        rtype = pop_text(ast_tree)
        ast_tree.append(ast_create_function(p, rtype, name))
    elif p.token==TOKEN_CLOSE_BRACKET:
        ast_tree.append(ast_create_lang_text(p, 'CLOSE_BRACKET'))
    elif p.token==TOKEN_SEMICOLON:
        ast_tree.append(ast_create_lang_text(p, 'SEMICOLON'))
    elif p.token==TOKEN_COMMA:
        ast_tree.append(ast_create_lang_text(p, 'COMMA'))
    return process
    
def create_ast(text_buffer, lang):
    ast_tree = []
    p = Param(lang, text_buffer)
    process = next_line(p)
    while process != None:
        get_token(p)
        process = create_ast_iter(p, ast_tree)
    return ast_tree
