# -*- coding: iso-8859-2 -*-

#	Programmer:	Piotr Borek
#	E-mail:		piotrborek@op.pl
#
#	Copyright 2006  Piotr Borek
#
#	Distributed under the terms of the GPL (GNU Public License)
#
#    pyX4 is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

__all__ = ["Plugin"]

import lib.config
import gui.main_gui
import graph.xpm
import widget.buttons

from lib.splash import SplashWindow
SplashWindow.pulse_text_import("lib.plugin")

import gtk
import gobject

STATE_NONE      = 0
STATE_VISIBLE   = 1
STATE_WINDOWED  = 2
STATE_MAXIMIZED = 4

class Plugin(gtk.VBox):
    """Podstawowa klasa dla wszystkich pluginow"""
    __plugname__ = 'Plugin base'
    def __init__(self):
        """Konstruktor"""
	gtk.VBox.__init__(self)
	gtk.VBox.show(self)
	self.config_file = lib.config.ConfigureSection(self, self.__plugname__)
	self.plugin_state = STATE_NONE
	self.scroll = gtk.ScrolledWindow()
	self.scroll.show()
	self.scroll.set_shadow_type(gtk.SHADOW_NONE)
	self.scroll.set_border_width(2)
	self.scroll.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_NEVER)
	self.__viewport = gtk.Viewport()
	self.__viewport.show()
	self.scroll.add(self.__viewport)
	self.init_plugin_buttons()
	self.pack_start(self.scroll, True, True, 0)
	self.plugin_window = gtk.Window()
	self.plugin_window.set_title(self.__plugname__)
	self.plugin_window.connect("delete-event", self.plugin_window_close_event)
	width = 150
	height = 480
	if self.config_file.has_option("win_width"):
	    width = int(self.config_file.get("win_width"))
	if self.config_file.has_option("win_height"):
	    height = int(self.config_file.get("win_height"))
	self.plugin_window.resize(width, height)
	if self.config_file.has_option("pos_x") & self.config_file.has_option("pos_y"):
	    x = int(self.config_file.get("pos_x"))
	    y = int(self.config_file.get("pos_y"))
	    self.plugin_window.move(x, y)
	if self.config_file.has_option("state"):
	    self.plugin_state = int(self.config_file.get("state"))
	if self.plugin_state & STATE_VISIBLE:
	    if self.plugin_state & STATE_WINDOWED:
		self.plugin_window.add(self)
		self.plugin_window.show()
	    elif self.plugin_state & STATE_MAXIMIZED:
		if gui.main_gui.hpaned.get_child1()==None:
		    gui.main_gui.hpaned.get_child2().hide()
		    gui.main_gui.hpaned.pack1(self)
	    else:
		gui.main_gui.hpaned.pack1(self)    
    def init_plugin_buttons(self):
        """Funkcja ta tworzy przyciski znajdujace sie przy gornej krawedzi okna pluginu"""
	self.plugin_buttons = gtk.HBox()
	self.plugin_buttons.show()
        self.__create_plugin_buttons()
        self.pack_start(self.plugin_buttons, False, False, 0)
    def reinit_plugin_buttons(self):
        """Funkcja ta na nowo tworzy przyciski znajdujacej sie przy gornej krawedzi okna pluginu"""
        children = self.plugin_buttons.get_children()
        for x in children:
            self.plugin_buttons.remove(x)
        self.__create_plugin_buttons()
    def __create_plugin_buttons(self):
        """Funckja pomocnicza dla funkcji init_plugin_buttons() i reinit_plugin_buttons()"""
        label = gtk.Label('')
	label.show()
	self.plugin_buttons.pack_start(label, True, True, 0)
	button = widget.buttons.XPMToolItem(graph.xpm.XPMUp, _('Show plugin in window'))
	button.connect("clicked", self.show_plugin_in_window)
	button.show()
	self.plugin_buttons.pack_start(button, False, False, 0)
	button = widget.buttons.XPMToolItem(graph.xpm.XPMDown, _('Show plugin in toolbar'))
	button.connect("clicked", self.show_plugin_in_toolbar)
	button.show()
	self.plugin_buttons.pack_start(button, False, False, 0)
	button = widget.buttons.XPMToolItem(graph.xpm.XPMMaximize, _('Maximize/minimize plugin window'))
	button.connect("clicked", self.maximize_plugin_window)
	button.show()
	self.plugin_buttons.pack_start(button, False, False, 0)
	button = widget.buttons.XPMToolItem(graph.xpm.XPMClose, _('Close plugin window'))
	button.connect("clicked", self.close_plugin_window)
	button.show()
	self.plugin_buttons.pack_start(button, False, False, 0)
    def add_icon(self, stock_id, tip):
        """
        Funkcja ta dodaje ikonke do plugintoolbar, ktorej nacisniecie powoduje
        pojawienie sie danego pluginu
        """
	gui.main_gui.plugintoolbar.register_icon(self.icon_clicked, stock_id, tip)
    def add(self, widget):
	self.__viewport.add(widget)
    def show(self):
	gtk.VBox.show(self)
	self.plugin_state = self.plugin_state | STATE_VISIBLE
    def hide(self):
	gtk.VBox.hide(self)
	self.plugin_state = self.plugin_state & ~STATE_VISIBLE
    def update_config(self):
	self.config_file.set("state", self.plugin_state)
	width, height = self.plugin_window.get_size()
	self.config_file.set("win_width", width)
	self.config_file.set("win_height", height)
	x, y = self.plugin_window.get_position()
	self.config_file.set("pos_x", x)
	self.config_file.set("pos_y", y)
    def icon_clicked(self, button=None, data=None):
	gui.main_gui.hpaned.pack1(self)
	self.plugin_state = self.plugin_state | STATE_VISIBLE
    def show_plugin_in_window(self, button=None, data=None):
        """
        Funckja ta powoduje usuniecie pluginu z odpowiedniego panelu i pojawienie
        sie pluginu w oknie.
        """
	if self.plugin_state & STATE_WINDOWED:
	    pass
	else:
	    self.plugin_state = self.plugin_state | STATE_WINDOWED
	    gui.main_gui.hpaned.remove(self)
            self.reinit_plugin_buttons()
	    self.plugin_window.add(self)
	    self.plugin_window.show()
    def show_plugin_in_toolbar(self, button=None, data=None):
        """
        Funckja ta powoduje ukrycie okna z pluginem i pojawienie sie go w odpowiednum panelu
        """
	if self.plugin_state & STATE_WINDOWED:
	    self.plugin_state = self.plugin_state & ~STATE_WINDOWED
            x, y = self.plugin_window.get_position()
	    self.plugin_window.hide()      # Po schowaniu okno powraca do swojej originalnej pozycji
            self.plugin_window.move(x, y)  # Wymuszam stara pozycje okna
	    self.plugin_window.remove(self)
            self.reinit_plugin_buttons()
	    gui.main_gui.hpaned.pack1(self)
    def maximize_plugin_window(self, button=None, data=None):
	child = gui.main_gui.hpaned.get_child2()
	if not (self.plugin_state & STATE_WINDOWED):
	    if self.plugin_state & STATE_MAXIMIZED:
		child.show()
		self.plugin_state = self.plugin_state & ~STATE_MAXIMIZED
	    else:
		child.hide()
		self.plugin_state = self.plugin_state | STATE_MAXIMIZED
    def close_plugin_window(self, button=None, data=None):
        """
        Funckja ta powoduje schowanie sie pluginu lub schowanie okienka z pluginem
        (jesli jest w okienku).
        """
	if self.plugin_state & STATE_WINDOWED:
            self.plugin_state = self.plugin_state & ~STATE_WINDOWED
	    self.plugin_state = self.plugin_state & ~STATE_VISIBLE
            x, y = self.plugin_window.get_position()
	    self.plugin_window.hide()      # Po schowaniu okno powraca do swojej originalnej pozycji
            self.plugin_window.move(x, y)  # Wymuszam stara pozycje okna
	    self.plugin_window.remove(self)
	else:
	    gui.main_gui.hpaned.remove(self)
	    self.plugin_state = self.plugin_state & ~STATE_VISIBLE
        if self.plugin_state & STATE_MAXIMIZED:
            self.plugin_state = self.plugin_state & ~STATE_MAXIMIZED
            child = gui.main_gui.hpaned.get_child2()
            child.show()
        self.reinit_plugin_buttons()
    def plugin_window_close_event(self, widget=None, event=None, data=None):
        """
        Funckja ta powoduje schowanie sie okienka z zawartoscia pluginu
        """
        self.close_plugin_window()
	return True
