# -*- coding: iso-8859-2 -*-

#	Programmer:	Piotr Borek
#	E-mail:		piotrborek@op.pl
#
#	Copyright 2006  Piotr Borek
#
#	Distributed under the terms of the GPL (GNU Public License)
#
#    pyX4 is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import lib.core
from lib.syntax.flags import *

from lib.splash import SplashWindow
SplashWindow.pulse_text_import('syntax.lib.dfa')

import gtk.gdk
try:
    import ctypes
    libc = ctypes.cdll.LoadLibrary('libc.so.6')
except:
    libc = None

class UnknownStateTypeException(Exception):
    def __init__(self, state):
        self.state = state
    def __str__(self):
        return self.state.__str__()

class DFA(object):    
    def __init__(self):
        self.syntax = None
        self.input = None
        self.state = None
        self.colors = None
        self.output_buffer = ""
        self.output_attr = None
        self.run = True
        self.str_start = 0
        self.str_lenght = 0
    def set_syntax(self, syntax_name):
        self.syntax = lib.core.share_data.get_data("syntax.dfa.%s" % syntax_name, True)
        self.colors = lib.core.share_data.get_data("conf.editor.colors.%s" % syntax_name, True)
    def set_input(self, text):
        self.str_start = 0
        self.str_lenght = len(text)
        if self.str_lenght==0:
            self.input = None
        else:
            self.input = text
    def set_state(self, state, attr=None):
        if state==None:
            self.state = self.syntax.get_idle_state()
            self.output_attr = self.colors.Text
        else:
            self.state = state
        if attr:
            self.output_attr = attr
    def get_state(self):
        return self.state
    def next_state(self):
        self.output_buffer = ''
        self.run = True
        if self.state==None or self.input==None or self.syntax==None:
            return False
        while self.run:
            if self.str_start >= self.str_lenght:
                return False
            if self.state.type==TYPE_CHAR:
                self.run_char_state()
            elif self.state.type==TYPE_STR:
                self.run_str_state()
            else:
                raise UnknownStateTypeException(self.state)
        return True
    def run_str_state(self):
        any = True
        for element in self.state.list:
            string, state, color, flags = element
            x = self.input[self.str_start:]
            try:
                cond = libc.strncmp(ctypes.c_char_p(x), ctypes.c_char_p(string), len(string))==0
            except:
                cond = self.input[self.str_start:].find(string)==0
            if cond:
                any = False
                self.check_flags(flags, string, color)
                self.state = getattr(self.syntax, state)
                break
        if any:
            self.run_any_state()
    def run_char_state(self):
        any = True
        letter = self.input[self.str_start]
        for element in self.state.list:
            chars, state, color, flags = element
            if letter in chars:
                any = False
                self.check_flags(flags, letter, color)
                self.state = getattr(self.syntax, state)
                break
        if any:
            self.run_any_state()
    def run_any_state(self):
        state, color, flags = self.state.any
        letter = self.input[self.str_start]
        self.check_flags(flags, letter, color)
        self.state = getattr(self.syntax, state)
    def check_flags(self, flags, string, color):
        if flags==FLAGS_NONE:
            self.output_buffer += string
            self.str_start += len(string)
            self.run = False
        elif (flags & FLAGS_BUFFER) == FLAGS_BUFFER:
            self.output_buffer += string
            self.str_start += len(string)
        elif (flags & FLAGS_FLUSH) == FLAGS_FLUSH:
            self.run = False
        if color!=None:
            self.output_attr = getattr(self.colors, color)
    def get_output_string(self):
        return self.output_buffer
    def get_output_attr(self):
        return self.output_attr
