# -*- coding: iso-8859-2 -*-

#	Programmer:	Piotr Borek
#	E-mail:		piotrborek@op.pl
#
#	Copyright 2006  Piotr Borek
#
#	Distributed under the terms of the GPL (GNU Public License)
#
#    pyX4 is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from lib.splash import SplashWindow
SplashWindow.pulse_text_import("lib.syntax.parser")

class TextObject(object):
    __slots__ = ('string', 'attr', 'state', 'width')
    def __init__(self, ostring, oattr, ostate, width=None):
        self.string = ostring
        self.attr = oattr
        self.state = ostate
        self.width = width

class TextList(object):
    __slots__ = ('text')
    def __init__(self):
        self.text = []
    def append_string(self, string, attr, state, width):
        self.text.append(TextObject(string, attr, state, width))

class SyntaxParser(object):
    def __init__(self, syntax_dfa, width_function=None):
	self.syntax_dfa = syntax_dfa
	if width_function:
	    self.width_function = width_function
	else:
	    self.width_function = self.default_width_function
    def default_width_function(self, string, font_style):
	return 0
    def process_text_init(self):
        self.__tmp_state = self.syntax_dfa.get_state()
        self.syntax_dfa.next_state()
        self.__tmp_attr = self.syntax_dfa.get_output_attr()
        self.__tmp_text = self.syntax_dfa.get_output_string()
        return True
    def process_text(self):
        attr = self.__tmp_attr
        state = self.__tmp_state
        text = self.__tmp_text
        process = True        
        while process:
            process = self.syntax_dfa.next_state()
            old_attr = attr
            old_state = state
            attr = self.syntax_dfa.get_output_attr()
            state = self.syntax_dfa.get_state()
            if old_attr==attr: # and old_state==state:
                text += self.syntax_dfa.get_output_string()
            else:
                self.__tmp_attr = attr
                self.__tmp_state = state
                attr = old_attr
                state = old_state
                self.__tmp_text = self.syntax_dfa.get_output_string()
                process = True
                break
        if not process:
            self.__tmp_text = ''
        return process, text, state, attr
    def create_formatted_text(self, text, state=None, attr=None):
        formatted = TextList()
        self.syntax_dfa.set_state(state, attr)
        self.syntax_dfa.set_input(text + '\n')
        process = self.process_text_init()
        while process:
            process, string, state, attr = self.process_text()
            if len(string) > 0:
                string = string.replace('\n', ' ')
                width = self.width_function(string, attr.font_style)
                formatted.append_string(string, attr, state, width)
        return formatted
