# -*- coding: iso-8859-2 -*-

#	Programmer:	Piotr Borek
#	E-mail:		piotrborek@op.pl
#
#	Copyright 2006  Piotr Borek
#
#	Distributed under the terms of the GPL (GNU Public License)
#
#    pyX4 is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

__all__ = ['init']

import lib.core
import lib.proxy
import lib.config
import lib.draw
import gui.main_gui
import conf.base
import widget.view
import widget.colorselection

from lib.splash import SplashWindow
SplashWindow.pulse_text_import('lib.syntax.property')

import gtk
import types
import re
import os

class Attributes:
    pass

class Colors:
    def __init__(self):
        self.color = None
        self.font_style = None

class ColorProperty(conf.base.PropertyBase):
    def __init__(self, name):
        conf.base.PropertyBase.__init__(self, 'Color preferences: %s' % name)
        self.attrs = Attributes()
        self.static_vars = lib.proxy.ClassProxy(self.attrs)
        p = lib.core.share_data.get_data('conf.editor.colors')
        p.add_color_property(self, name)
        self.config_file = lib.config.ConfigureSection(self, 'Editor.Colors')
        self.property_name = name
        self.syntax_view = None
        self.color_name = ''
        self.create_gui()
    def create_gui(self):
        self.label = gtk.Label(_("Choose a color."))
        self.label.set_alignment(0.0, 0.5)
        self.label.show()
        self.button = gtk.Button(_("Set to default"))
        self.button.connect('clicked', self.button_clicked_event)
        self.button.show()
        hbox = gtk.HBox()
        hbox.show()
        hbox.pack_start(self.label, True, True)
        hbox.pack_start(self.button, False, False)
        self.pack_start(hbox, False, False)
        self.select = widget.colorselection.ColorWidget()
        self.select.show_font_style()
        self.select.set_color_callback(self.clicked_callback)
        self.select.set_attr_callback(self.toggled_callback)
        self.select.show()
        self.pack_start(self.select, True, True, 0)        
    def end_init(self):
        p = lib.core.share_data.get_data('conf.editor.colors')
        if p.color_scheme=='custom':
            self.exec_colorscheme('default.color')
            self.read_config()
        else:
            self.exec_colorscheme(p.color_scheme + '.color')
        self.check_missing()
        if self.syntax_view:
            for name, attr in self.attrs.__dict__.iteritems():
                try:
                    self.syntax_view.set_color(name, attr.color)
                    self.syntax_view.set_font_style(name, attr.font_style)
                except:
                    print "Warning: \"%s\" SynaxView isn't using all define colors" % self.property_name
    def exec_colorscheme(self, color_scheme):
        namespace = { 'set_color' : self.set_color, 'NORMAL' : lib.draw.FONT_STYLE_NORMAL, 'BOLD' : lib.draw.FONT_STYLE_BOLD, 'ITALIC' : lib.draw.FONT_STYLE_ITALIC }
        try:
            p = os.environ['HOME'] + '/.pyX4/'
            execfile(p + color_scheme, {}, namespace)
        except:
            execfile(lib.core.share_data.get_data('env.path.colors') + color_scheme, {}, namespace)
    def read_config(self):
        for name, attr in self.attrs.__dict__.iteritems():
            if self.config_file.has_option(name):
                color, font_style = self.config_file.get(name).split(':')
                attr.color = self.get_colormap().alloc_color(gtk.gdk.color_parse(color))
                attr.font_style = int(font_style)
    def check_missing(self):
        for name, attr in self.attrs.__dict__.iteritems():
            if attr.color==None:
                print
                print "Error: Color \"%s\" not defined." % name
                print "       Check the syntax file or the color scheme files."
                print
                raise AttributeError                
    def create_attr(self, attr_name):
        col = Colors()
        setattr(self.attrs, attr_name, col)
    def set_color(self, attr_name, color, font_style):
        col = None
        if type(color)==types.StringType:
            col = self.get_colormap().alloc_color(gtk.gdk.color_parse(color))
        elif isinstance(color, gtk.gdk.Color):
            col = color
        else:
            raise AttributeError
        if hasattr(self.attrs, attr_name):
            cc = getattr(self.attrs, attr_name)
            cc.color = col
            cc.font_style = font_style
    def create_view(self, text):
        try:
            self.remove(self.syntax_view)
        except:
            pass
        separator = gtk.HSeparator()
        separator.show()
        self.pack_start(separator, False, False, 2)
        self.syntax_view = widget.view.SyntaxView(self.property_name, self.attrs.__dict__, text)
        self.syntax_view.show()
        self.syntax_view.set_callback(self.color_clicked)
        self.pack_start(self.syntax_view, False, False)
    def color_clicked(self, color_name):
        self.color_name = color_name
        self.label.set_text(_("Color: ") + color_name)
        self.select.set_color(self.syntax_view.get_color(self.color_name))
        if self.color_name=='Background':
            self.select.set_font_style(lib.draw.FONT_STYLE_NORMAL)
        else:
            self.select.set_font_style(self.syntax_view.get_font_style(self.color_name))
    def get_colors(self):
        return self.get_static()
    def close_widget(self):
        self.select.reset_color()
    def apply_clicked(self, button, data=None):
        p = lib.core.share_data.get_data('conf.editor.colors')
        for k, v in self.attrs.__dict__.items():
            try:
                v.color = self.syntax_view.get_color(k)
                v.font_style = self.syntax_view.get_font_style(k)
            except:
                pass
        p.set_color_scheme('custom')
        self.update_config()
        p.property_reinit()
        lib.core.events.emit_event('conf.editor.colors')
    def update_config(self):
        p = lib.core.share_data.get_data('conf.editor.colors')
        if p.color_scheme=='custom':
            for name in self.attrs.__dict__.keys():
                attr = getattr(self.attrs, name)
                s = '#%02X%02X%02X:%i' % (attr.color.red/257, attr.color.green/257, attr.color.blue/257, attr.font_style)
                self.config_file.set('%s' % name, s)
    def toggled_callback(self, font_style):
        if self.color_name:
            self.syntax_view.set_font_style(self.color_name, font_style)
    def clicked_callback(self, gdk_color):
        if self.color_name:
            self.syntax_view.set_color(self.color_name, gdk_color)
    def button_clicked_event(self, widget, data=None):
        if self.color_name:
            attr = getattr(self.attrs, self.color_name)
            self.syntax_view.set_font_style(self.color_name, attr.font_style)
            self.select.set_font_style(attr.font_style)
            self.syntax_view.set_color(self.color_name, attr.color)

def init(name, file_name, regex):
    """Registers a file type.
       @param name: a text that is visible in preferences window in tree widget
       @param file_name: name of syntax file and color file
       @param regex: regular expression pattern to recognize type of files
    """
    if regex:
        r = re.compile(regex)
    else:
        r = None
    try:
        list = lib.core.share_data.get_data('syntax.re.list')
    except:
        list = []
        lib.core.share_data.register_data('syntax.re.list', list)
    n, e = file_name.split('.')
    list.append([r, name])
    i = __import__(lib.core.share_data.get_data('env.path.syntax').replace('/', '.') + n, globals(), locals(), ['*'], -1)
    lib.core.share_data.register_data('syntax.dfa.%s' % name, i, True)
    if name == 'Text':
        gui.main_gui.menu.edit_menu.filetype_menu.create_radioitem(name, 0)
        gui.main_gui.menu.edit_menu.filetype_menu.set_active('Text', True)
    else:
        gui.main_gui.menu.edit_menu.filetype_menu.create_radioitem(name)
    return ColorProperty(name)
