# -*- coding: iso-8859-2 -*-

#	Programmer:	Piotr Borek
#	E-mail:		piotrborek@op.pl
#
#	Copyright 2006  Piotr Borek
#
#	Distributed under the terms of the GPL (GNU Public License)
#
#    pyX4 is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from lib.syntax.flags import *

def create_idle(g, list):
    class IdleState:
        type = TYPE_CHAR
        any  = ['IdleState', 'Text', FLAGS_NONE]
        list = []
    def get_idle_state():
        return IdleState
    for x in list:
        IdleState.list.append(x)
    g['IdleState'] = IdleState
    g['get_idle_state'] = get_idle_state

def create_wait_for_eat(g, name, return_state, characters, ignore=None, char_color=None, any_color=None):
    class EndLineChars:
        type = TYPE_CHAR
        any = [name, any_color, FLAGS_NONE]
        list = [[characters, return_state, char_color, FLAGS_NONE]
            ]
    if ignore:
        EndLineChars.list.append([ignore, name, None, FLAGS_BUFFER])
    g[name] = EndLineChars

def create_wait_for_noeat(g, name, return_state, characters, ignore=None, char_color=None, any_color=None):
    class UselessChars:
        type = TYPE_CHAR
        any  = [name, any_color, FLAGS_NONE]
        list = [[characters, return_state, char_color, FLAGS_FLUSH]
            ]
    if ignore:
        UselessChars.list.append([ignore, name, None, FLAGS_BUFFER])
    g[name] = UselessChars

def create_comment(g, name, return_state, begin_comment, end_comment, color='Comment', append=True):
    class BeginComment:
        type = TYPE_STR
        any  = [return_state, None, FLAGS_NONE]
        list = [[begin_comment, name + '__EndComment', color, FLAGS_NONE]
            ]
    class EndComment:
        type = TYPE_STR
        any  = [name + '__EndComment', None, FLAGS_NONE]
        list = [[end_comment, return_state, None, FLAGS_NONE]
            ]
    g[name] = BeginComment
    g[name + '__EndComment'] = EndComment
    if append:
        g['IdleState'].list.append([begin_comment[0], name, None, FLAGS_NOEAT])

def create_c_numbers(g, name, return_state, append=True):
    CHARACTERS = '_qwertyuiopasdfghjklzxcvbnmQWERTYUIOPASDFGHJKLZXCVBNM'
    class FirstNumberState:
        type = TYPE_STR
        any  = [name + '__NumberState', 'Number', FLAGS_NOEAT]
        list = [['0x', name + '__HexState', 'HexNumber', FLAGS_BUFFER],
            ]
    class NumberState:
        type = TYPE_CHAR
        any  = [return_state, None, FLAGS_FLUSH]
        list = [['0123456789', name + '__NumberState',       None,  FLAGS_BUFFER],
                ['.',          name + '__FloatState',       'Text', FLAGS_BUFFER],
                [CHARACTERS,   name + '__NumberErrorState', 'Text', FLAGS_NONE]
            ]
    class FloatState:
        type = TYPE_CHAR
        any  = [return_state, None, FLAGS_FLUSH]
        list = [['0123456789', name + '__FloatState',       'FloatNumber', FLAGS_BUFFER],
                [CHARACTERS,   name + '__NumberErrorState', 'Text',        FLAGS_NONE],
                ['.',          name + '__NumberErrorState', 'Text',        FLAGS_NONE]
            ]
    class HexState:
        type = TYPE_CHAR
        any  = [return_state, None, FLAGS_FLUSH]
        list = [['0123456789',   name + '__HexState',          None,  FLAGS_BUFFER],
                ['aAbBcCdDeEfF', name + '__HexState',          None,  FLAGS_BUFFER],
                [CHARACTERS,     name + '__NumberErrorState', 'Text', FLAGS_NONE],
                ['.',            name + '__NumberErrorState', 'Text', FLAGS_NONE]
            ]
    class NumberErrorState:
        type = TYPE_CHAR
        any  = [return_state, None, FLAGS_FLUSH]
        list = [[CHARACTERS,    name + '__NumberErrorState', 'Text', FLAGS_NONE],
                ['.0123456789', name + '__NumberErrorState', 'Text', FLAGS_NONE]
            ]
    g[name] = FirstNumberState
    g[name + '__NumberState'] = NumberState
    g[name + '__FloatState'] = FloatState
    g[name + '__HexState'] = HexState
    g[name + '__NumberErrorState'] = NumberErrorState
    if append:
        g['IdleState'].list.append(['0123456789', name, 'Number', FLAGS_NOEAT])

def create_import(g, name, return_state, return_characters):
    CHARACTERS = '*_qwertyuiopasdfghjklzxcvbnmQWERTYUIOPASDFGHJKLZXCVBNM'
    class CheckImportState:
        type = TYPE_CHAR
        any  = [return_state, 'Text', FLAGS_FLUSH]
        list = [[' \t',             name + '__BeginImportState', None, FLAGS_NONE],
                [return_characters, return_state,                None, FLAGS_FLUSH],
            ]    
    class BeginImportState:
        type = TYPE_CHAR
        any  = [return_state, None, FLAGS_NOEAT]
        list = [[' \t',             name + '__BeginImportState', None,           FLAGS_NONE],
                [CHARACTERS,        name + '__ImportState',     'ImportKeyword', FLAGS_NONE],
                [return_characters, return_state,                None,           FLAGS_FLUSH],
            ]
    class ImportState:
        type = TYPE_CHAR
        any  = [name + '__ImportState', 'ImportKeyword', FLAGS_NONE]
        list = [[return_characters, return_state,                 None,         FLAGS_FLUSH],
                ['\n',              return_state,                'Text',        FLAGS_NONE],
                [',',               name + '__BeginImportState', 'Characters2', FLAGS_NONE],
                [' \t',             name + '__EndImportState',    None,         FLAGS_NONE]
            ]
    class EndImportState:
        type = TYPE_CHAR
        any  = [name + '__EndImportState', 'ImportKeyword', FLAGS_NONE]
        list = [[return_characters, return_state,                 None,         FLAGS_FLUSH],
                [CHARACTERS,        return_state,                 None,         FLAGS_FLUSH],
                [' \t',             name + '__EndImportState',    None,         FLAGS_NONE],
                [',',               name + '__BeginImportState', 'Characters2', FLAGS_NONE],
                
            ]
    g[name] = CheckImportState
    g[name + '__BeginImportState'] = BeginImportState
    g[name + '__ImportState'] = ImportState
    g[name + '__EndImportState'] = EndImportState

def create_c_text(g, name, return_state, format_string, format_char='%', letter_char='\\', ignore=None, text_color='Text2', format_color='FormatString', append=True):
    CHARACTERS = '_qwertyuiopasdfghjklzxcvbnmQWERTYUIOPASDFGHJKLZXCVBNM'
    class BeginTextState:
        type = TYPE_CHAR
        any  = [name + '__TextState', text_color, FLAGS_NONE]
        list = []
    class TextState:
        type = TYPE_CHAR
        any  = [name + '__TextState', text_color, FLAGS_NONE]
        list = [['\"',        return_state,                 text_color,   FLAGS_NONE],
                [format_char, name + '__FormatStringState', format_color, FLAGS_BUFFER],
                [letter_char, name + '__LetterState',       format_color, FLAGS_NONE]
            ]
    class FormatStringState:
        type = TYPE_CHAR
        any  = [name + '__FormatStringState', None, FLAGS_BUFFER]
        list = [[CHARACTERS, name + '__EndFormatState', None,       FLAGS_NOEAT],
                [' \"\n',    name + '__TextState',      text_color, FLAGS_NOEAT],
            ]
    if ignore:
        FormatStringState.list.insert(0, [ignore, name + '__FormatStringState', None, FLAGS_BUFFER])
    class LetterState:
        type = TYPE_CHAR
        any  = [name + '__TextState', None, FLAGS_NONE]
        list = [
            ]
    class EndFormatState:
        type = TYPE_CHAR
        any  = [name + '__TextState', text_color, FLAGS_FLUSH]
        list = [[format_string, name + '__TextState',  None, FLAGS_NONE]
            ]
    g[name] = BeginTextState
    g[name + '__TextState'] = TextState
    g[name + '__FormatStringState'] = FormatStringState
    g[name + '__LetterState'] = LetterState
    g[name + '__EndFormatState'] = EndFormatState
    if append:
        g['IdleState'].list.append(['\"', name, 'Text2', FLAGS_NOEAT])

def create_c_macro(g, name, return_state, begin_char='#', color='Macro', append=True):
    class MacroState:
        type = TYPE_CHAR
        any  = [name, None, FLAGS_NONE]
        list = [['\n', return_state,                  None, FLAGS_NONE],
                ['\\', name + '__ContinueMacroState', None, FLAGS_NONE]
            ]
    class ContinueMacroState:
        type = TYPE_CHAR
        any  = [name, None, FLAGS_NONE]
        list = [[' \t\\', name + '__ContinueMacroState', None, FLAGS_NONE]
            ]
    g[name] = MacroState
    g[name + '__ContinueMacroState'] = ContinueMacroState
    if append:
        g['IdleState'].list.append(['#', name, color, FLAGS_NONE])
