# -*- coding: iso-8859-2 -*-

#	Programmer:	Piotr Borek
#	E-mail:		piotrborek@op.pl
#
#	Copyright 2006  Piotr Borek
#
#	Distributed under the terms of the GPL (GNU Public License)
#
#    pyX4 is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import lib.core

if __name__!="__main__":
    from lib.splash import SplashWindow
    SplashWindow.pulse_text_import("lib.text")

import types
import codecs
import gtk.gdk

class String(object):
    def __init__(self, value = u""):
        self.check_value(value)
        self.__str = []
        for letter in value:
            self.__str.append(letter)
    def append(self, value):
        self.check_value(value)
        self.__str.append(value[0])
    def append_string(self, string):
        self.check_value(string)
        for c in string:
            self.append(c)
    def remove(self, i):
        return self.__str.pop(i)
    def insert(self, i, o):
        self.check_value(o)
        self.__str.insert(i, o)
    def clear(self):
        while len(self.__str):
            self.__str.pop(-1)
    def get(self, tab = 0):
        s = u""
        c = 0
        for letter in self.__str:
            if tab !=0 and letter == u" ":
                c += 1
                if c == tab:
                    c = 0
                    s += u"\t"
            else:
                if c > 0:
                    s += u" " * c
                    c = 0
                s += letter
        return s
    def __len__(self):
        return len(self.__str)
    def __getitem__(self, key):
        self.check_key(key)
        return self.__str[key]
    def __setitem__(self, key, value):
        self.check_key(key)
        self.check_value(value)
        self.__str[key] = value[0]
    def check_key(self, key):
        if type(key)!=types.IntType and type(key)!=types.SliceType:
            raise KeyError('Key has to be int type')
    def check_value(self, value):
        if type(value)!=types.UnicodeType:
            raise AttributeError('Value has to be unicode type')

class TextBuffer(object):
    def __init__(self):
        self.buffer = []
        self.file_path = None
        self.file_name = None
        self.encoding = 'UTF-8'
        self.modified = False
    def append_string(self, string):
        self.buffer.append(string)
        self.modified = True
    def insert_string(self, i, o):
        self.buffer.insert(i, o)
        self.modified = True
    def remove_string(self, i):
        self.modified = True
        return self.buffer.pop(i)
    def pop_string(self):
        self.modified = True
        return self.buffer.pop()
    def get_string(self, key):
        return self.buffer[key]
    def is_modified(self):
        return self.modified
    def get_file_name(self):
        return self.file_name
    def get_file_path(self):
        return self.file_path
    def read_file(self, file_name, encoding=None):
        if encoding == None:
            encoding = self.encoding
        error = False
        if file_name == None or file_name == '':
            raise Exception(_('Invalid filename or none'))
        self.file_path, self.file_name = lib.tools.get_full_path(file_name)
        try:
            f = open(self.file_path + self.file_name, 'rb')
        except:
            f = open(self.file_path + self.file_name, 'wb')
            f.close()
            f = open(self.file_path + self.file_name, 'rb')        
        line = f.readline()
        end = 0
        p = lib.core.share_data.get_data('conf.editor.indentation')
        while line:
            end = len(line)
            line = line.strip('\x0A\x0D').replace("\t", " " * p.tab_width)
            end -= len(line)
            try:
                enc_line = line.decode(encoding)
            except:
                enc_line = line.decode(encoding, 'replace')
                error = True
            self.append_string(String(enc_line))
	    line = f.readline()
	f.close()
        if end:
            self.append_string(String(u""))
	else:
	    if len(self.buffer)==0:
		self.append_string(String(u""))
        self.encoding = encoding
        self.modified = False
        return error
    def write_file_as(self, file_name, encoding = None, endline = '\n'):
        self.file_path, self.file_name = lib.tools.get_full_path(file_name)
        return self.write_file(encoding, endline)
    def write_file(self, encoding = None, endline = '\n'):
        p = lib.core.share_data.get_data('conf.editor.indentation')
        if encoding == None:
            encoding = self.encoding
        f = open(self.file_path + self.file_name, 'wb')
        error = False
        for v, line in enumerate(self.buffer):
            if p.replace_tabs:
                s = line.get(p.tab_width)
            else:
                s = line.get()
            try:
                enc_s = s.encode(encoding)
            except:
                enc_s = s.encode(encoding, 'replace')
                error = True
            f.write(enc_s)
	    if (v + 1) != len(self.buffer) or len(s) != 0:
        	f.write(endline)
        f.close()
        self.modified = False
        return error
    def clear(self):
        while len(self.buffer):
            self.buffer.pop(-1)
    def __len__(self):
        return len(self.buffer)
    def __getitem__(self, key):
        return self.buffer[key]

if __name__=="__main__":
    s = String(u'')
    s[0] = u"a"
    print s.get()
    t = TextBuffer()
    print t.read_file('test', 'iso8859-2')
    for x in t:
        print x
        print x.get()
    print len(t)
    print t.write_file('output', 'iso8859-2')
