# -*- coding: iso-8859-2 -*-

#	Programmer:	Piotr Borek
#	E-mail:		piotrborek@op.pl
#
#	Copyright 2006  Piotr Borek
#
#	Distributed under the terms of the GPL (GNU Public License)
#
#    pyX4 is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from lib.plugin import Plugin
import lib.core
import lib.tools

import gtk

class FileHistoryBase(gtk.ScrolledWindow):
    def __init__(self, label=''):
        gtk.ScrolledWindow.__init__(self)
        self.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        self.label = gtk.Label(label)
        self.label.show()
        self.list_store = gtk.ListStore(str, str, int)
        self.tree_view = gtk.TreeView(self.list_store)
        self.tree_view.set_headers_visible(False)
        self.tree_view.show()
        self.tv_column = gtk.TreeViewColumn(None)
        self.tree_view.append_column(self.tv_column)
        text_cell = gtk.CellRendererText()
        self.tv_column.pack_start(text_cell, True)
        self.tv_column.set_attributes(text_cell, text=0)
        self.add(self.tree_view)
    def add_file(self, file_name, file_path, file_date):
        iter = self.list_store.get_iter_root()
        found = False
        while iter:
            n = self.list_store.get_value(iter, 0)
            p = self.list_store.get_value(iter, 1)
            d = self.list_store.get_value(iter, 2)
            if n==file_name and p==file_path:
                self.list_store.set_value(iter, 2, file_date)
                found = True
                break
            iter = self.list_store.iter_next(iter)
        if not found:
            self.list_store.append([file_name, file_path, file_date])
    def sort(self):
        self.list_store.set_sort_column_id(0, gtk.SORT_ASCENDING)

class FileHistoryByDate(FileHistoryBase):
    def __init__(self):
        FileHistoryBase.__init__(self, _('Date'))
        self.list_store.set_sort_func(0, self.sort_me)
    def sort_me(self, model, iter1, iter2, data=None):
        v1, v2 = model.get_value(iter1, 2), model.get_value(iter2, 2)
        if v1 < v2:
            return -1
        elif v1 > v2:
            return 1
        return 0

class FileHistoryByName(FileHistoryBase):
    def __init__(self):
        FileHistoryBase.__init__(self, _('Name'))
        self.list_store.set_sort_func(0, self.sort_me)
    def sort_me(self, model, iter1, iter2, data=None):
        v1, v2 = model.get_value(iter1, 0), model.get_value(iter2, 0)
        if v1 < v2:
            return -1
        elif v1 > v2:
            return 1
        return 0

class FileHistoryByType(FileHistoryBase):
    def __init__(self):
        FileHistoryBase.__init__(self, _('Type'))
        self.list_store.set_sort_func(0, self.sort_me)
    def sort_me(self, model, iter1, iter2, data=None):
        v1, v2 = model.get_value(iter1, 0), model.get_value(iter2, 0)
        if v1.find('.') < 0:
            v1 = ''
        else:
            v1 = v1.split('.')[-1]
        if v2.find('.') < 0:
            v2 = ''
        else:
            v2 = v2.split('.')[-1]
        if v1 < v2:
            return -1
        elif v1 > v2:
            return 1
        return 0

class FileHistory(Plugin):
    __plugname__ = 'FileHistory'
    def __init__(self):
        Plugin.__init__(self)
        self.add_icon('gtk-index', _("Recent opened files"))
        notebook = gtk.Notebook()
        notebook.show()
        self.add(notebook)
        self.f_name = FileHistoryByName()
        self.f_name.show()
        notebook.append_page(self.f_name, self.f_name.label)
        self.f_date = FileHistoryByDate()
        self.f_date.show()
        notebook.append_page(self.f_date, self.f_date.label)
        self.f_type = FileHistoryByType()
        self.f_type.show()
        notebook.append_page(self.f_type, self.f_type.label)
        self.f_name.tree_view.connect('row-activated', self.row_activated_event)
        self.f_type.tree_view.connect('row-activated', self.row_activated_event)
        self.f_date.tree_view.connect('row-activated', self.row_activated_event)
        if self.config_file.has_option('last'):
            for x in self.config_file.get_list('last'):
                path, name = lib.tools.get_full_path(x)
                time = lib.tools.get_file_time(x)
                self.add_file(name, path, time)
            self.sort()
        else:
            print "false"
        lib.core.events.connect(self, 'main.file.open', self.file_open_event)
    def add_file(self, file_name, file_path, file_date):
        self.f_name.add_file(file_name, file_path, file_date)
        self.f_date.add_file(file_name, file_path, file_date)
        self.f_type.add_file(file_name, file_path, file_date)
    def sort(self):
        self.f_name.sort()
        self.f_date.sort()
        self.f_type.sort()
    def row_activated_event(self, treeview, path, view_column, data=None):
        model = treeview.get_model()
        iter = model.get_iter(path)
        file_name = model.get_value(iter, 1) + model.get_value(iter, 0)
        lib.core.events.emit_event('main.file.open', [file_name])
    def file_open_event(self, obj, param, data=None):
        file_name = param[1]
        path, name = lib.tools.get_full_path(file_name)
        time = lib.tools.get_file_time(file_name)
        self.add_file(name, path, time)
    def update_config(self):
        Plugin.update_config(self)
        store = self.f_name.list_store
        list = []
        iter = store.get_iter_root()
        while iter:
            s = store.get_value(iter, 1) + store.get_value(iter, 0)
            list.append(s)
            iter = store.iter_next(iter)
        self.config_file.set_list('last', list)

def get_plugin_class():
    return FileHistory
