# -*- coding: iso-8859-2 -*-

#	Programmer:	Piotr Borek
#	E-mail:		piotrborek@op.pl
#
#	Copyright 2006  Piotr Borek
#
#	Distributed under the terms of the GPL (GNU Public License)
#
#    pyX4 is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import gtk

import os, stat

from lib.plugin import Plugin
import gui.tooltips

class FileView(gtk.ScrolledWindow):
    def __init__(self, headers_visible=True):
	gtk.ScrolledWindow.__init__(self)
	self.show()
	self.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
	self.list = gtk.ListStore(str)
	self.treeview = gtk.TreeView(self.list)
	self.add(self.treeview)
	self.treeview.set_headers_visible(headers_visible)
	self.treeview.show()
	self.tvcolumn = gtk.TreeViewColumn(None)
	self.treeview.append_column(self.tvcolumn)
	text_cell = gtk.CellRendererText()
	self.tvcolumn.pack_start(text_cell, True)
	self.tvcolumn.set_attributes(text_cell, text=0)

class FileViewer(Plugin):
    __plugname__ = "File Viewer"
    def __init__(self):
	Plugin.__init__(self)
	self.add_icon('gtk-find', _('File viewer'))
	self.dir_name = os.path.abspath('./')
	vbox = gtk.VBox()
	vbox.show()
	self.combobox = gtk.combo_box_new_text()
	self.combobox.set_size_request(80, -1)
	self.combobox.show()
	self.combobox.append_text(_('All files (*.*)'))
	self.combobox.append_text('C/C++ (*.c,*.cpp,*.cc,*.h,*.hpp)')
	self.combobox.append_text('C# (*.cs)')
	self.combobox.append_text('Python (*.py)')
	self.combobox.append_text('Perl (*.pl)')
	self.combobox.append_text('Bash (*.sh)')
	if self.config_file.has_option('combo_active'):
	    self.combobox.set_active(int(self.config_file.get('combo_active')))
	else:
	    self.combobox.set_active(0)
	self.dir_view = FileView()
	self.dir_view.treeview.connect('row-activated', self.open_dir_event)
	self.file_view = FileView(False)
	self.make_dirlist(self.dir_view)
	self.make_filelist(self.file_view)
	self.vpaned = gtk.VPaned()
	self.vpaned.show()
	self.vpaned.pack1(self.file_view, True, False)
	self.vpaned.pack2(self.dir_view, True, False)
	vbox.pack_start(self.combobox, False, False, 0)
	vbox.pack_start(self.vpaned, True, True, 0)
	if self.config_file.has_option('vpaned_pos'):
	    self.vpaned.set_position(int(self.config_file.get('vpaned_pos')))
	self.add(vbox)
    def make_dirlist(self, file_view):
	file_view.list.clear()
	gui.tooltips.tips.set_tip(file_view.treeview, self.dir_name)
	file_view.tvcolumn.set_title(self.dir_name)
	files = []
	for f in os.listdir(self.dir_name):
	    filestat = os.stat(self.dir_name + '/' + f)
	    if stat.S_ISDIR(filestat.st_mode):
		files.append(f)
	files.sort()
	files = ['..'] + files
	for f in files:
	    file_view.list.append([f])
    def make_filelist(self, file_view):
	file_view.list.clear()
	files = []
	for f in os.listdir(self.dir_name):
	    filestat = os.stat(self.dir_name + '/' + f)
	    if stat.S_ISREG(filestat.st_mode):
		files.append(f)
	files.sort()
	for f in files:
	    file_view.list.append([f])
    def open_dir_event(self, treeview, path, view_column):
	model = treeview.get_model()
	iter = model.get_iter(path)
	self.dir_name = os.path.join(self.dir_name, model.get_value(iter, 0))
	self.dir_name = os.path.abspath(self.dir_name)
	self.make_dirlist(self.dir_view)
	self.make_filelist(self.file_view)
    def update_config(self):
	Plugin.update_config(self)
	self.config_file.set('vpaned_pos', self.vpaned.get_position())
	self.config_file.set('combo_active', self.combobox.get_active())

def get_plugin_class():
    return FileViewer
