# -*- coding: iso-8859-2 -*-

#	Programmer:	Piotr Borek
#	E-mail:		piotrborek@op.pl
#
#	Copyright 2006  Piotr Borek
#
#	Distributed under the terms of the GPL (GNU Public License)
#
#    pyX4 is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import lib.syntax.property
from lib.syntax.flags import *
from lib.syntax.template import *

CHARACTERS         = '_qwertyuiopasdfghjklzxcvbnmQWERTYUIOPASDFGHJKLZXCVBNM'
SPECIAL_CHARACTERS = ' \t\n=\"\'~!@#$%^&*()-+[]{}\|;:,.<>/?'
g                  = globals()

def init():
    p = lib.syntax.property.init('Java', 'java.py', r"^.*\.(java)$")
    p.create_attr('Text')
    p.create_attr('Background')
    p.create_attr('Keyword')
    p.create_attr('SpecialKeyword1')
    p.create_attr('SpecialKeyword2')
    p.create_attr('TypeKeyword')
    p.create_attr('ImportKeyword')
    p.create_attr('Characters1')
    p.create_attr('Characters2')
    p.create_attr('Text1')
    p.create_attr('Text2')
    p.create_attr('Comment')
    p.create_attr('Number')
    p.create_attr('FloatNumber')
    p.create_attr('HexNumber')
    p.create_attr('FormatString')
    p.end_init()
    p.create_view("""package Test;

import org.gnome.gtk.*;
/* Comment */
public class App extends VBox {
    protected static void function(String str, int i, float f) {
        this.i = 100 + 0xFF + 3.14159 + len(str);
        str = "A text %.5f" + 'another text';
    }
}""")

create_idle(g,
    list = [[CHARACTERS,        'KeywordState',      'Text',        FLAGS_NOEAT],
            ['-~!@$%^&*+=\<>?', 'IdleState',         'Characters1', FLAGS_NONE],
            ['.',               'DotState',          'Characters2', FLAGS_NOEAT],
            ['/',               'BeginCommentState', 'Comment',     FLAGS_NOEAT],
            [',()[]{}:;',       'IdleState',         'Characters2', FLAGS_NONE],
    ]
)
create_comment       (g, 'TextState1',         'IdleState', '\'',               '\'', color = 'Text1')
create_comment       (g, 'CommentState1',      'IdleState', '//',               '\n', append=False)
create_comment       (g, 'CommentState2',      'IdleState', '/*',               '*/', append=False) 
create_import        (g, 'ImportState',        'IdleState', ';')
create_wait_for_noeat(g, 'UselesscharState'  , 'IdleState', SPECIAL_CHARACTERS,       any_color='Text')
create_c_text        (g, 'TextState2',         'IdleState', 'cdeEfgiosxX',            ignore='hl')
create_c_numbers     (g, 'NumberState',        'IdleState')

class BeginCommentState:
    type = TYPE_STR
    any  = ['IdleState', 'Characters1', FLAGS_NONE]
    list = [['//', 'CommentState1', None, FLAGS_NOEAT],
            ['/*', 'CommentState2', None, FLAGS_NOEAT]
        ]

class DotState:
    type = TYPE_STR
    any = ['IdleState', None, FLAGS_NONE]
    list = [['...', 'IdleState',   'TypeKeyword', FLAGS_NONE],
            ['.0',  'NumberState', 'Number',      FLAGS_NOEAT],
            ['.1',  'NumberState', 'Number',      FLAGS_NOEAT],
            ['.2',  'NumberState', 'Number',      FLAGS_NOEAT],
            ['.3',  'NumberState', 'Number',      FLAGS_NOEAT],
            ['.4',  'NumberState', 'Number',      FLAGS_NOEAT],
            ['.5',  'NumberState', 'Number',      FLAGS_NOEAT],
            ['.6',  'NumberState', 'Number',      FLAGS_NOEAT],
            ['.7',  'NumberState', 'Number',      FLAGS_NOEAT],
            ['.8',  'NumberState', 'Number',      FLAGS_NOEAT],
            ['.9',  'NumberState', 'Number',      FLAGS_NOEAT]
        ]

class KeywordState:
    type = TYPE_STR
    any  = ['UselesscharState', 'Text', FLAGS_NONE]
    # Very important !!! Sort this list !!!
    # Longer strings put on the start of the list,
    # and shorter strings put on the end of the list !!!
    list = [['synchronized', 'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ['implements',   'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['instanceof',   'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['transient',    'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ['interface',    'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['notifyAll',    'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ['protected',    'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['abstract',     'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['finalize',     'UselesscharState', 'SpecialKeyword1', FLAGS_BUFFER],
            ['getClass',     'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ['hashCode',     'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ['toString',     'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ['continue',     'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ['volatile',     'UselesscharState', 'SpecialKeyword1', FLAGS_BUFFER],
            ['private',      'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['generic',      'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ['boolean',      'UselesscharState', 'TypeKeyword',     FLAGS_BUFFER],
            ['byvalue',      'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ['extends',      'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['package',      'ImportState',      'SpecialKeyword1', FLAGS_BUFFER],
            ['finally',      'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ['default',      'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ['notify',       'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ['public',       'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['import',       'ImportState',      'Keyword',         FLAGS_BUFFER],
            ['throws',       'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ['future',       'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ['native',       'UselesscharState', 'SpecialKeyword1', FLAGS_BUFFER],
            ['equals',       'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['double',       'UselesscharState', 'TypeKeyword',     FLAGS_BUFFER],
            ['return',       'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ['static',       'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['switch',       'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ['const',        'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['throw',        'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ['float',        'UselesscharState', 'TypeKeyword',     FLAGS_BUFFER],
            ['break',        'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ['super',        'UselesscharState', 'SpecialKeyword1', FLAGS_BUFFER],
            ['clone',        'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ['final',        'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['inner',        'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ['outer',        'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ['short',        'UselesscharState', 'TypeKeyword',     FLAGS_BUFFER],
            ['while',        'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ['catch',        'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ['class',        'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ['false',        'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['this',         'UselesscharState', 'SpecialKeyword1', FLAGS_BUFFER],
            ['byte',         'UselesscharState', 'TypeKeyword',     FLAGS_BUFFER],
            ['cast',         'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ['rest',         'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ['null',         'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['wait',         'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ['case',         'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ['char',         'UselesscharState', 'TypeKeyword',     FLAGS_BUFFER],
            ['long',         'UselesscharState', 'TypeKeyword',     FLAGS_BUFFER],
            ['void',         'UselesscharState', 'TypeKeyword',     FLAGS_BUFFER],
            ['true',         'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['else',         'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ['goto',         'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ['var',          'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ['int',          'UselesscharState', 'TypeKeyword',     FLAGS_BUFFER],
            ['for',          'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ['new',          'UselesscharState', 'SpecialKeyword1', FLAGS_BUFFER],
            ['try',          'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ['do',           'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ['if',           'UselesscharState', 'Keyword',         FLAGS_BUFFER]
    ]
