# -*- coding: iso-8859-2 -*-

#	Programmer:	Piotr Borek
#	E-mail:		piotrborek@op.pl
#
#	Copyright 2006  Piotr Borek
#
#	Distributed under the terms of the GPL (GNU Public License)
#
#    pyX4 is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import lib.syntax.property
from lib.syntax.flags import *
from lib.syntax.template import *

CHARACTERS         = '_qwertyuiopasdfghjklzxcvbnmQWERTYUIOPASDFGHJKLZXCVBNM'
SPECIAL_CHARACTERS = ' \t\n=\"\'~!@#$%^&*()-+[]{}\|;:,.<>/?'
g                  = globals()

def init():
    p = lib.syntax.property.init('Pascal', 'pascal.py', r"^.*\.[pP][aA][sS]$")
    p.create_attr('Text')
    p.create_attr('Background')
    p.create_attr('Keyword')
    p.create_attr('SpecialKeyword1')
    p.create_attr('SpecialKeyword2')
    p.create_attr('TypeKeyword')
    p.create_attr('ImportKeyword')
    p.create_attr('Characters1')
    p.create_attr('Characters2')
    p.create_attr('Text2')
    p.create_attr('Comment')
    p.create_attr('Number')
    p.create_attr('FloatNumber')
    p.create_attr('HexNumber')
    p.end_init()
    p.create_view("""program Test;
{ Comment }
TObject = object
    X, Y: string;
    constructor Init(z: integer);
    begin
        z = 100 + 0xFF + 3.14159;
        X = \'Test\';
    end;
end;""")

create_idle(g,
    list = [[CHARACTERS,         'KeywordState', 'Text',        FLAGS_NOEAT],
            ['~!@$%^&*+-\<>?:=', 'IdleState',    'Characters1', FLAGS_NONE],
            ['.',                'DotState',     'Characters1', FLAGS_NOEAT],
            [',()[];',           'IdleState',    'Characters2', FLAGS_NONE],
    ]
)
create_comment       (g, 'TextState1',         'IdleState',        '\'',               '\'', color = 'Text2')
create_comment       (g, 'CommentState1',      'IdleState',        '//',               '\n', append=False)
create_comment       (g, 'CommentState2',      'IdleState',        '{',                '}') 
create_wait_for_noeat(g, 'UselesscharState'  , 'IdleState',        SPECIAL_CHARACTERS,       any_color='Text')
create_c_numbers     (g, 'NumberState',        'IdleState')
create_import        (g, 'ImportState',        'UselesscharState', ';')

class DotState:
    type = TYPE_STR
    any = ['IdleState', None, FLAGS_NONE]
    list = [['..',  'IdleState',   'TypeKeyword', FLAGS_NONE],
            ['.0',  'NumberState', 'Number',      FLAGS_NOEAT],
            ['.1',  'NumberState', 'Number',      FLAGS_NOEAT],
            ['.2',  'NumberState', 'Number',      FLAGS_NOEAT],
            ['.3',  'NumberState', 'Number',      FLAGS_NOEAT],
            ['.4',  'NumberState', 'Number',      FLAGS_NOEAT],
            ['.5',  'NumberState', 'Number',      FLAGS_NOEAT],
            ['.6',  'NumberState', 'Number',      FLAGS_NOEAT],
            ['.7',  'NumberState', 'Number',      FLAGS_NOEAT],
            ['.8',  'NumberState', 'Number',      FLAGS_NOEAT],
            ['.9',  'NumberState', 'Number',      FLAGS_NOEAT]
        ]

class KeywordState:
    type = TYPE_STR
    any  = ['UselesscharState', 'Text', FLAGS_NONE]
    # Very important !!! Sort this list !!!
    # Longer strings put on the start of the list,
    # and shorter strings put on the end of the list !!!
    list = [["initialization", 'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ["implementation", 'UselesscharState', 'SpecialKeyword1', FLAGS_BUFFER],
            ["dispinterface",  'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ["constructor",    'UselesscharState', 'SpecialKeyword1', FLAGS_BUFFER],
            ["destructor",     'UselesscharState', 'SpecialKeyword1', FLAGS_BUFFER],
            ["assembler",      'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ["inherited",      'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ["interface",      'UselesscharState', 'SpecialKeyword1', FLAGS_BUFFER],
            ["interrupt",      'UselesscharState', 'SpecialKeyword1', FLAGS_BUFFER],
            ["otherwise",      'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ["procedure",      'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ["protected",      'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ["published",      'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ["function",       'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ["external",       'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ["absolute",       'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ["abstract",       'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ["continue",       'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ["operator",       'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ["override",       'UselesscharState', 'SpecialKeyword1', FLAGS_BUFFER],
            ["property",       'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ["readonly",       'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ["register",       'UselesscharState', 'SpecialKeyword1', FLAGS_BUFFER],
            ["safecall",       'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ["exports",        'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ["dispose",        'UselesscharState', 'SpecialKeyword1', FLAGS_BUFFER],
            ["dynamic",        'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ["finally",        'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ["forward",        'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ["library",        'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ["stdcall",        'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ["private",        'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ["program",        'ImportState',      'Keyword',         FLAGS_BUFFER],
            ["virtual",        'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ["integer",        'UselesscharState', 'TypeKeyword',     FLAGS_BUFFER],
            ["boolean",        'UselesscharState', 'TypeKeyword',     FLAGS_BUFFER],
            ["pointer",        'UselesscharState', 'TypeKeyword',     FLAGS_BUFFER],
            ["except",         'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ["export",         'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ["downto",         'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ["dispid",         'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ["inline",         'UselesscharState', 'SpecialKeyword1', FLAGS_BUFFER],
            ["object",         'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ["packed",         'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ["pascal",         'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ["public",         'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ["record",         'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ["repeat",         'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ["string",         'UselesscharState', 'TypeKeyword',     FLAGS_BUFFER],
            ["begin",          'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ["array",          'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ["break",          'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ["cdecl",          'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ["class",          'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ["const",          'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ["until",          'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ["false",          'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ["label",          'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ["raise",          'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ["while",          'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ["else",           'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ["case",           'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ["byte",           'UselesscharState', 'TypeKeyword',     FLAGS_BUFFER],
            ["true",           'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ["type",           'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ["unit",           'ImportState',      'SpecialKeyword1', FLAGS_BUFFER],
            ["uses",           'ImportState',      'Keyword',         FLAGS_BUFFER],
            ["exit",           'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ["fail",           'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ["file",           'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ["goto",           'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ["near",           'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ["self",           'UselesscharState', 'SpecialKeyword1', FLAGS_BUFFER],
            ["then",           'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ["with",           'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ["char",           'UselesscharState', 'TypeKeyword',     FLAGS_BUFFER],
            ["and",            'UselesscharState', 'Characters1',     FLAGS_BUFFER],
            ["asm",            'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ["div",            'UselesscharState', 'Characters1',     FLAGS_BUFFER],
            ["end",            'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ["new",            'UselesscharState', 'SpecialKeyword1', FLAGS_BUFFER],
            ["nil",            'UselesscharState', 'SpecialKeyword1', FLAGS_BUFFER],
            ["not",            'UselesscharState', 'Characters1',     FLAGS_BUFFER],
            ["set",            'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ["shl",            'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ["shr",            'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ["try",            'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ["far",            'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ["for",            'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ["var",            'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ["mod",            'UselesscharState', 'Characters1',     FLAGS_BUFFER],
            ["xor",            'UselesscharState', 'Characters1',     FLAGS_BUFFER],
            ["as",             'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ["do",             'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ["of",             'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ["on",             'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ["or",             'UselesscharState', 'Characters1',     FLAGS_BUFFER],
            ["if",             'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ["in",             'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ["is",             'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ["to",             'UselesscharState', 'Keyword',         FLAGS_BUFFER]
        ]
