# -*- coding: iso-8859-2 -*-

#	Programmer:	Piotr Borek
#	E-mail:		piotrborek@op.pl
#
#	Copyright 2006  Piotr Borek
#
#	Distributed under the terms of the GPL (GNU Public License)
#
#    pyX4 is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import lib.syntax.property
from lib.syntax.flags import *
from lib.syntax.template import *

CHARACTERS         = '_qwertyuiopasdfghjklzxcvbnmQWERTYUIOPASDFGHJKLZXCVBNM'
SPECIAL_CHARACTERS = ' \t\n=\"\'~!@#$%^&*()-+[]{}\|;:,.<>/?'
g                  = globals()

def init():
    p = lib.syntax.property.init('Perl', 'perl.py', r"^.*\.(pl|pm)$")
    p.create_attr('Text')
    p.create_attr('Background')
    p.create_attr('Keyword')
    p.create_attr('Variable')
    p.create_attr('ArrayVariable')
    p.create_attr('HashVariable')
    p.create_attr('SpecialVariable')
    p.create_attr('ImportKeyword')
    p.create_attr('SpecialKeyword1')
    p.create_attr('SpecialKeyword2')
    p.create_attr('Characters1')
    p.create_attr('Characters2')
    p.create_attr('Text1')
    p.create_attr('Text2')
    p.create_attr('Comment')
    p.create_attr('Number')
    p.create_attr('FloatNumber')
    p.create_attr('HexNumber')
    p.create_attr('FormatString')
    p.end_init()
    p.create_view("""package Test;
# Comment
my ($var, @tab, %hash);
sub test {
    my $x = shift + 100 + 0xFF + 3.14159;
    my $y = @_[0];
    my $z = sprintf("Test: $x %i" . 'Nothing', $i);
    return length($z);
}""")

#If you are using lib.syntax.template this is the first function you have to write
create_idle(g,
    list = [[CHARACTERS,       'KeywordState',      'Text',        FLAGS_NOEAT],
            ['~!^&*+\/<>?;-',  'IdleState',         'Characters1', FLAGS_NONE],
            [',()[]{}:',       'IdleState',         'Characters2', FLAGS_NONE],
            ['$%@',            'VariableState',      None,         FLAGS_NOEAT],
            ['.',              'DotState',          'Characters1', FLAGS_NOEAT],
            ['=',              'Comment2State',     'Characters1', FLAGS_NOEAT]
        ]
)
create_comment       (g, 'CommentState',      'IdleState', '#',      '\n')
create_comment       (g, 'Comment2State',     'IdleState', '=head1', '=cut',             append=False);
create_comment       (g, 'TextState1',        'IdleState', '\'',     '\'',               color='Text1')
create_import        (g, 'ImportState',       'UselesscharState',    '$;')
create_wait_for_noeat(g, 'UselesscharState',  'IdleState',           SPECIAL_CHARACTERS, any_color='Text')
create_wait_for_noeat(g, 'EndVariableState1', 'IdleState',           SPECIAL_CHARACTERS)
create_wait_for_noeat(g, 'EndVariableState5', 'IdleState',           SPECIAL_CHARACTERS, char_color='SpecialVariable')
create_c_numbers     (g, 'NumberState',       'IdleState')
create_c_text        (g, 'TextState2',        'IdleState',           'dDiouxXfegEascpn')

class DotState:
    type = TYPE_STR
    any  = ['IdleState', None, FLAGS_NONE]
    list = [['.0', 'NumberState', 'Number', FLAGS_NOEAT],
            ['.1', 'NumberState', 'Number', FLAGS_NOEAT],
            ['.2', 'NumberState', 'Number', FLAGS_NOEAT],
            ['.3', 'NumberState', 'Number', FLAGS_NOEAT],
            ['.4', 'NumberState', 'Number', FLAGS_NOEAT],
            ['.5', 'NumberState', 'Number', FLAGS_NOEAT],
            ['.6', 'NumberState', 'Number', FLAGS_NOEAT],
            ['.7', 'NumberState', 'Number', FLAGS_NOEAT],
            ['.8', 'NumberState', 'Number', FLAGS_NOEAT],
            ['.9', 'NumberState', 'Number', FLAGS_NOEAT]
        ]

class VariableState:
    type = TYPE_CHAR
    any  = []
    list = [['%', 'EndVariableState1',  'HashVariable',  FLAGS_BUFFER],
            ['$', 'EndVariableState2',  'Variable',      FLAGS_BUFFER],
            ['@', 'EndVariableState3',  'ArrayVariable', FLAGS_BUFFER]
        ]

class EndVariableState2:
    type = TYPE_CHAR
    any  = ['EndVariableState1', None, FLAGS_NOEAT]
    list = [['_./,\"\\#*?[];!@:0$<>()%=-~|&`\'+', 'EndVariableState5', None, FLAGS_BUFFER],
            ['^',                                 'EndVariableState4', None, FLAGS_BUFFER]
        ]

class EndVariableState3:
    type = TYPE_STR
    any  = ['EndVariableState1', None, FLAGS_NOEAT]
    # Very important !!! Sort this list !!!
    # Longer strings put on the start of the list,
    # and shorter strings put on the end of the list !!!
    list = [['EXPORT_OK', 'EndVariableState5', None, FLAGS_BUFFER],
            ['OVERLORD',  'EndVariableState5', None, FLAGS_BUFFER],
            ['EXPORT',    'EndVariableState5', None, FLAGS_BUFFER],
            ['INC',       'EndVariableState5', None, FLAGS_BUFFER],
            ['ISA',       'EndVariableState5', None, FLAGS_BUFFER],
            ['ENV',       'EndVariableState5', None, FLAGS_BUFFER],
            ['SIG',       'EndVariableState5', None, FLAGS_BUFFER],
            ['_',         'EndVariableState5', None, FLAGS_BUFFER],
        ]

class EndVariableState4:
    type = TYPE_CHAR
    any  = ['EndVariableState1', None, FLAGS_NONE]
    list = [['ADEILPTWX', 'EndVariableState5', None, FLAGS_BUFFER]
        ]

class KeywordState:
    type = TYPE_STR
    any  = ['UselesscharState', 'Text', FLAGS_NONE]
    # Very important !!! Sort this list !!!
    # Longer strings put on the start of the list,
    # and shorter strings put on the end of the list !!!
    list = [['getprotobynumber', 'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['getprotobyname',   'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['gethostbyaddr',    'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['gethostbyname',    'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['getservbyname',    'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['getservbyport',    'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['getnetbyaddr',     'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['getnetbyname',     'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['getpriority',      'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['setpriority',      'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['getpeername',      'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['getsockname',      'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['endprotoent',      'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['getprotoent',      'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['setprotoent',      'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['diagnostics',      'UselesscharState', 'SpecialKeyword1', FLAGS_BUFFER],
            ['getservent',       'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['serservent',       'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['endservent',       'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['getsockopt',       'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['setsockopt',       'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['socketpair',       'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['endhostent',       'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['gethostent',       'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['sethostent',       'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['localtime',        'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['quotemeta',        'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['rewinddir',        'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['endnetent',        'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['wantarray',        'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['getnetent',        'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['setnetent',        'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['endpwent',         'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['getpwent',         'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['getpwnam',         'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['getpwuid',         'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['setpwent',         'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['endgrent',         'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['getgrgid',         'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['getgrnam',         'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['getgrent',         'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['setgrent',         'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['overload',         'UselesscharState', 'SpecialKeyword1', FLAGS_BUFFER],
            ['truncate',         'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['readlink',         'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['dbmclose',         'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['syswrite',         'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['formline',         'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['closedir',         'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['getlogin',         'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['shutdown',         'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['shmwrite',         'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['package',          'ImportState',      'SpecialKeyword1', FLAGS_BUFFER],
            ['require',          'ImportState',      'SpecialKeyword1', FLAGS_BUFFER],
            ['foreach',          'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ['STDARGV',          'UselesscharState', 'SpecialVariable', FLAGS_BUFFER],
            ['reverse',          'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['unshift',          'UselesscharState', 'SpecialKeyword1', FLAGS_BUFFER],
            ['integer',          'UselesscharState', 'SpecialKeyword1', FLAGS_BUFFER],
            ['lcfirst',          'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['ucfirst',          'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['opendir',          'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['readdir',          'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['seekdir',          'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['telldir',          'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['sigtrap',          'UselesscharState', 'SpecialKeyword1', FLAGS_BUFFER],
            ['symlink',          'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['binmade',          'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['dbmopen',          'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['binmade',          'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['sprintf',          'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['sysopen',          'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['sysread',          'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['getpgrp',          'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['getppid',          'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['setpgrp',          'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['syscall',          'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['waitpid',          'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['connect',          'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['shmread',          'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['defined',          'UselesscharState', 'SpecialKeyword1', FLAGS_BUFFER],
            ['chroot',           'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['import',           'ImportState',      'SpecialKeyword1', FLAGS_BUFFER],
            ['delete',           'UselesscharState', 'SpecialKeyword1', FLAGS_BUFFER],
            ['return',           'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ['caller',           'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ['unless',           'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ['STDOUT',           'UselesscharState', 'SpecialVariable', FLAGS_BUFFER],
            ['STDERR',           'UselesscharState', 'SpecialVariable', FLAGS_BUFFER],
            ['exists',           'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['gmtime',           'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['unpack',           'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['length',           'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['rindex',           'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['substr',           'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['scalar',           'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['splice',           'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['values',           'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['msgctl',           'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['msgget',           'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['msgsnd',           'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['msgrcv',           'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['semctl',           'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['semget',           'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['shmctl',           'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['shmget',           'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['strict',           'UselesscharState', 'SpecialKeyword1', FLAGS_BUFFER],
            ['rename',           'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['unlink',           'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['fileno',           'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['printf',           'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['select',           'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['system',           'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['accept',           'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['listen',           'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['socket',           'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['while',            'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ['elsif',            'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ['break',            'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ['until',            'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ['print',            'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['untie',            'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ['bless',            'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ['atan2',            'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['srand',            'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['crypt',            'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['chomp',            'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['index',            'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['shift',            'UselesscharState', 'SpecialKeyword1', FLAGS_BUFFER],
            ['split',            'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['STDIN',            'UselesscharState', 'SpecialVariable', FLAGS_BUFFER],
            ['flock',            'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['ioctl',            'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['chmod',            'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['chown',            'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['lstat',            'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['mkdir',            'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['rmdir',            'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['utime',            'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['close',            'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['fcntl',            'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['write',            'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['alarm',            'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['chdir',            'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['umask',            'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['times',            'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['sleep',            'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['local',            'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['semop',            'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['undef',            'UselesscharState', 'SpecialKeyword1', FLAGS_BUFFER],
            ['reset',            'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['else',             'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ['goto',             'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ['next',             'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ['last',             'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ['each',             'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['rand',             'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['sqrt',             'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['time',             'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['pack',             'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['chop',             'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['eval',             'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['grep',             'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['join',             'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['keys',             'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['push',             'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['sort',             'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['less',             'UselesscharState', 'SpecialKeyword1', FLAGS_BUFFER],
            ['vars',             'UselesscharState', 'SpecialKeyword1', FLAGS_BUFFER],
            ['link',             'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['stat',             'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['getc',             'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['open',             'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['read',             'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['seek',             'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['tell',             'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['exec',             'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['exit',             'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['fork',             'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['pipe',             'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['glob',             'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['kill',             'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['wait',             'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['warn',             'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['bind',             'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['recv',             'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['send',             'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['dump',             'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['DATA',             'UselesscharState', 'SpecialVariable', FLAGS_BUFFER],
            ['sub',              'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ['for',              'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ['and',              'UselesscharState', 'Characters1',     FLAGS_BUFFER],
            ['not',              'UselesscharState', 'Characters1',     FLAGS_BUFFER],
            ['use',              'ImportState',      'SpecialKeyword1', FLAGS_BUFFER],
            ['map',              'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['pop',              'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['abs',              'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['cos',              'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['sin',              'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['exp',              'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['int',              'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['log',              'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['chr',              'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['hex',              'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['oct',              'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['ord',              'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['vec',              'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['our',              'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ['ref',              'UselesscharState', 'SpecialKeyword1', FLAGS_BUFFER],
            ['lib',              'UselesscharState', 'SpecialKeyword1', FLAGS_BUFFER],
            ['ops',              'UselesscharState', 'SpecialKeyword1', FLAGS_BUFFER],
            ['eof',              'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['die',              'UselesscharState', 'SpecialKeyword1', FLAGS_BUFFER],
            ['cmp',              'UselesscharState', 'Characters1',     FLAGS_BUFFER],
            ['if',               'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ['do',               'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ['my',               'UselesscharState', 'Keyword',         FLAGS_BUFFER],
            ['or',               'UselesscharState', 'Characters1',     FLAGS_BUFFER],
            ['ge',               'UselesscharState', 'Characters1',     FLAGS_BUFFER],
            ['le',               'UselesscharState', 'Characters1',     FLAGS_BUFFER],
            ['gt',               'UselesscharState', 'Characters1',     FLAGS_BUFFER],
            ['lt',               'UselesscharState', 'Characters1',     FLAGS_BUFFER],
            ['eq',               'UselesscharState', 'Characters1',     FLAGS_BUFFER],
            ['ne',               'UselesscharState', 'Characters1',     FLAGS_BUFFER],
            ['lc',               'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER],
            ['uc',               'UselesscharState', 'SpecialKeyword2', FLAGS_BUFFER]
        ]
