# -*- coding: iso-8859-2 -*-

#	Programmer:	Piotr Borek
#	E-mail:		piotrborek@op.pl
#
#	Copyright 2006  Piotr Borek
#
#	Distributed under the terms of the GPL (GNU Public License)
#
#    pyX4 is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import lib.syntax.property
from lib.syntax.flags import *
from lib.syntax.template import *

CHARACTERS         = '_qwertyuiopasdfghjklzxcvbnmQWERTYUIOPASDFGHJKLZXCVBNM'
SPECIAL_CHARACTERS = ' \t\n=\"\'~!@#$%^&*()-+[]{}\|;:,.<>/?'
g                  = globals()

def init():
    p = lib.syntax.property.init('Python', 'python.py', r"^.*\.py$")
    p.create_attr('Text')
    p.create_attr('Background')
    p.create_attr('Keyword')
    p.create_attr('SpecialKeyword1')
    p.create_attr('SpecialKeyword2')
    p.create_attr('ImportKeyword')
    p.create_attr('Characters1')
    p.create_attr('Characters2')
    p.create_attr('Text1')
    p.create_attr('Text2')
    p.create_attr('Comment')
    p.create_attr('Number')
    p.create_attr('FloatNumber')
    p.create_attr('HexNumber')
    p.create_attr('FormatString')
    p.end_init()
    p.create_view("""from gtk import VBox
# comment
class X(object):
    def __init__(self, v):
        self.text = "A text %s" % 'another text'
        self.hex = 0xFF
        self.x = 100 + 3.14159 + len(self.text)""")

create_idle(g, 
    list = [[CHARACTERS,           'KeywordState',     'Text',        FLAGS_NOEAT],
            ['~!@$%^&*-+=\/<>?:',  'IdleState',        'Characters1', FLAGS_NONE],
            [',()[]{}',            'IdleState',        'Characters2', FLAGS_NONE],
            ['.',                  'DotState',         'Characters1', FLAGS_NOEAT]
        ]
)
create_comment       (g, 'CommentState',       'IdleState',        '#',                '\n')
create_comment       (g, 'TextState1',         'IdleState',        '\'',               '\'', color='Text1')
create_import        (g, 'ImportState',        'IdleState', ' \t#\n')
create_wait_for_noeat(g, 'UselesscharState',   'IdleState',        SPECIAL_CHARACTERS,       any_color='Text')
create_wait_for_noeat(g, 'BeginFunctionState', 'IdleState',        '([',                     ignore=' \t', any_color='Text')
create_c_text        (g, 'TextState2',         'IdleState',        'diouxXeEfFgGcrs')
create_c_numbers     (g, 'NumberState',        'IdleState')

class DotState:
    type = TYPE_STR
    any = ['IdleState', None, FLAGS_NONE]
    list = [['.0',  'NumberState', 'Number', FLAGS_NOEAT],
            ['.1',  'NumberState', 'Number', FLAGS_NOEAT],
            ['.2',  'NumberState', 'Number', FLAGS_NOEAT],
            ['.3',  'NumberState', 'Number', FLAGS_NOEAT],
            ['.4',  'NumberState', 'Number', FLAGS_NOEAT],
            ['.5',  'NumberState', 'Number', FLAGS_NOEAT],
            ['.6',  'NumberState', 'Number', FLAGS_NOEAT],
            ['.7',  'NumberState', 'Number', FLAGS_NOEAT],
            ['.8',  'NumberState', 'Number', FLAGS_NOEAT],
            ['.9',  'NumberState', 'Number', FLAGS_NOEAT]
        ]

class KeywordState:
    type = TYPE_STR
    any  = ['UselesscharState', 'Text', FLAGS_NONE]
    # Very important !!! Sort this list !!!
    # Longer strings put on the start of the list,
    # and shorter strings put on the end of the list !!!
    list = [['enumerate', 'BeginFunctionState',       'SpecialKeyword2', FLAGS_BUFFER],
            ['continue',  'UselesscharState',         'Keyword',         FLAGS_BUFFER],
            ['finally',   'UselesscharState',         'Keyword',         FLAGS_BUFFER],
            ['setattr',   'BeginFunctionState',       'SpecialKeyword2', FLAGS_BUFFER],
            ['getattr',   'BeginFunctionState',       'SpecialKeyword2', FLAGS_BUFFER],
            ['globals',   'BeginFunctionState',       'SpecialKeyword2', FLAGS_BUFFER],
            ['return',    'UselesscharState',         'Keyword',         FLAGS_BUFFER],
            ['import',    'ImportState',              'Keyword',         FLAGS_BUFFER],
            ['object',    'UselesscharState',         'SpecialKeyword2', FLAGS_BUFFER],
            ['except',    'UselesscharState',         'Keyword',         FLAGS_BUFFER],
            ['global',    'UselesscharState',         'Keyword',         FLAGS_BUFFER],
            ['locals',    'BeginFunctionState',       'SpecialKeyword2', FLAGS_BUFFER],
            ['class',     'UselesscharState',         'Keyword',         FLAGS_BUFFER],
            ['print',     'UselesscharState',         'Keyword',         FLAGS_BUFFER],
            ['raise',     'UselesscharState',         'Keyword',         FLAGS_BUFFER],
            ['while',     'UselesscharState',         'Keyword',         FLAGS_BUFFER],
            ['range',     'BeginFunctionState',       'SpecialKeyword2', FLAGS_BUFFER],
            ['break',     'UselesscharState',         'Keyword',         FLAGS_BUFFER],
            ['False',     'UselesscharState',         'SpecialKeyword2', FLAGS_BUFFER],
            ['float',     'BeginFunctionState',       'SpecialKeyword1', FLAGS_BUFFER],
            ['yield',     'UselesscharState',         'Keyword',         FLAGS_BUFFER],
            ['self',      'UselesscharState',         'SpecialKeyword1', FLAGS_BUFFER],
            ['elif',      'UselesscharState',         'Keyword',         FLAGS_BUFFER],
            ['else',      'UselesscharState',         'Keyword',         FLAGS_BUFFER],
            ['from',      'ImportState',              'Keyword',         FLAGS_BUFFER],
            ['None',      'UselesscharState',         'SpecialKeyword2', FLAGS_BUFFER],
            ['True',      'UselesscharState',         'SpecialKeyword2', FLAGS_BUFFER],
            ['pass',      'UselesscharState',         'Keyword',         FLAGS_BUFFER],
            ['open',      'BeginFunctionState',       'SpecialKeyword2', FLAGS_BUFFER],
            ['def',       'UselesscharState',         'Keyword',         FLAGS_BUFFER],
            ['try',       'UselesscharState',         'Keyword',         FLAGS_BUFFER],
            ['len',       'BeginFunctionState',       'SpecialKeyword2', FLAGS_BUFFER],
            ['str',       'BeginFunctionState',       'SpecialKeyword2', FLAGS_BUFFER],
            ['del',       'UselesscharState',         'Keyword',         FLAGS_BUFFER],
            ['and',       'UselesscharState',         'Keyword',         FLAGS_BUFFER],
            ['int',       'BeginFunctionState',       'SpecialKeyword2', FLAGS_BUFFER],
            ['for',       'UselesscharState',         'Keyword',         FLAGS_BUFFER],
            ['min',       'BeginFunctionState',       'SpecialKeyword2', FLAGS_BUFFER],
            ['max',       'BeginFunctionState',       'SpecialKeyword2', FLAGS_BUFFER],
            ['in',        'UselesscharState',         'Keyword',         FLAGS_BUFFER],
            ['if',        'UselesscharState',         'Keyword',         FLAGS_BUFFER],
            ['__',        'SpecialClassKeywordState',  None,             FLAGS_NOEAT],
            ['or',        'UselesscharState',         'Keyword',         FLAGS_BUFFER],
            ['as',        'UselesscharState',         'Keyword',         FLAGS_BUFFER]
        ]

class SpecialClassKeywordState:
    type = TYPE_STR
    any = ['IdleState', 'Text', FLAGS_NONE]
    list = [['__getattribute__', 'BeginFunctionState', 'SpecialKeyword1', FLAGS_BUFFER],
            ['__metaclass__',    'UselesscharState',   'SpecialKeyword1', FLAGS_BUFFER],
            ['__builtin__',      'UselesscharState',   'SpecialKeyword1', FLAGS_BUFFER],
            ['__getattr__',      'BeginFunctionState', 'SpecialKeyword1', FLAGS_BUFFER],
            ['__setattr__',      'BeginFunctionState', 'SpecialKeyword1', FLAGS_BUFFER],
            ['__delattr__',      'BeginFunctionState', 'SpecialKeyword1', FLAGS_BUFFER],
            ['__nonzero__',      'BeginFunctionState', 'SpecialKeyword1', FLAGS_BUFFER],
            ['__unicode__',      'BeginFunctionState', 'SpecialKeyword1', FLAGS_BUFFER],
            ['__future__',       'UselesscharState',   'SpecialKeyword1', FLAGS_BUFFER],
            ['__delete__',       'BeginFunctionState', 'SpecialKeyword1', FLAGS_BUFFER],
            ['__slots__',        'UselesscharState',   'SpecialKeyword1', FLAGS_BUFFER],
            ['__class__',        'UselesscharState',   'SpecialKeyword1', FLAGS_BUFFER],
            ['__init__',         'BeginFunctionState', 'SpecialKeyword1', FLAGS_BUFFER],
            ['__main__',         'UselesscharState',   'SpecialKeyword1', FLAGS_BUFFER],
            ['__name__',         'UselesscharState',   'SpecialKeyword1', FLAGS_BUFFER],
            ['__dict__',         'UselesscharState',   'SpecialKeyword1', FLAGS_BUFFER],
            ['__hash__',         'BeginFunctionState', 'SpecialKeyword1', FLAGS_BUFFER],
            ['__repr__',         'BeginFunctionState', 'SpecialKeyword1', FLAGS_BUFFER],
            ['__rcmp__',         'BeginFunctionState', 'SpecialKeyword1', FLAGS_BUFFER],
            ['__new__',          'BeginFunctionState', 'SpecialKeyword1', FLAGS_BUFFER],
            ['__del__',          'BeginFunctionState', 'SpecialKeyword1', FLAGS_BUFFER],
            ['__str__',          'BeginFunctionState', 'SpecialKeyword1', FLAGS_BUFFER],
            ['__get__',          'BeginFunctionState', 'SpecialKeyword1', FLAGS_BUFFER],
            ['__set__',          'BeginFunctionState', 'SpecialKeyword1', FLAGS_BUFFER],
            ['__cmp__',          'BeginFunctionState', 'SpecialKeyword1', FLAGS_BUFFER],
            ['__lt__',           'BeginFunctionState', 'SpecialKeyword1', FLAGS_BUFFER],
            ['__le__',           'BeginFunctionState', 'SpecialKeyword1', FLAGS_BUFFER],
            ['__eq__',           'BeginFunctionState', 'SpecialKeyword1', FLAGS_BUFFER],
            ['__ne__',           'BeginFunctionState', 'SpecialKeyword1', FLAGS_BUFFER],
            ['__gt__',           'BeginFunctionState', 'SpecialKeyword1', FLAGS_BUFFER],
            ['__ge__',           'BeginFunctionState', 'SpecialKeyword1', FLAGS_BUFFER]
        ]
