/***************************************************************************
 *   Copyright (C) 2005 by Piotr Borek                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

using System.Xml;
using System;
using Gtk;
using Glade;
using System.Collections;

class MainApp
{
    public enum ToggleState {
	MoveNode,
	CreateNode,
	CreateLink
    }
    static ToggleState state;
    public static ToggleState State {
	get {
	    return state;
	}
    }
    static ComboBox ComboBoxLinkType;
    public static int GetLinkType {
	get {
	    return ComboBoxLinkType.Active;
	}
    }
    [Widget] Window window_app;
    [Widget] ToggleToolButton togglebutton_mysz;
    [Widget] ToggleToolButton togglebutton_nowy;
    [Widget] ToggleToolButton togglebutton_polacz;
    [Widget] VBox vbox1;
    [Widget] ComboBox link_type;
    MyApp.Workspace workspace;
    bool toggle_lock;
    XmlDocument project;
    string project_name;
    public static void Main(string[] args)
    {
	new MainApp(args);
    }
    public MainApp(string[] args)
    {
	Application.Init();
	state = ToggleState.MoveNode;
	project = new XmlDocument();
	project.LoadXml("<?xml version=\"1.0\" encoding=\"iso8859-2\"?><document></document>");
	project_name = null;
	Glade.XML gxml = new Glade.XML("edytor.glade", "window_app", null);
	gxml.Autoconnect(this);
	toggle_lock = false;
	workspace = new MyApp.Workspace("BezNazwy", project);
	workspace.Show();
	vbox1.PackStart(workspace, true, true, 0);
	link_type.Active = 0;
	ComboBoxLinkType = link_type;
	Application.Run();
    }
    public void SaveProject()
    {
	try {
	    project.Save(project_name);
	}
	catch (Exception e) {
	    MessageDialog dialog = new MessageDialog(window_app,
		DialogFlags.Modal,
		MessageType.Error,
		ButtonsType.Ok,
		"Plik nie zostal zapisany !\nZla nazwa pliku.", 0);
	    dialog.Run();
	    dialog.Destroy();
	    return;
	}
	workspace.Changed = false;
    }
    public bool SaveChanges()
    {
	if (workspace.Changed)
	{
	    MessageDialog dialog = new MessageDialog(window_app,
		DialogFlags.Modal,
		MessageType.Question,
		ButtonsType.None,
		"Czy zapisac zmiany ?", 0);
	    dialog.AddButton("gtk-yes", ResponseType.Yes);
	    dialog.AddButton("gtk-no", ResponseType.No);
	    dialog.AddButton("gtk-cancel", ResponseType.Cancel);
	    int rval = dialog.Run();
	    if (rval==(int)ResponseType.Yes)
		OnZapisz(null, null);
	    dialog.Destroy();
	    if (rval==(int)ResponseType.Yes ||
		rval==(int)ResponseType.No) return true;
	    return false;
	}
	return true;
    }
    public void OpenProject(string filename)
    {
	bool rval = SaveChanges();
	if (rval)
	{
	    project.Load(filename);
	    project_name = filename;
	    workspace.NewProject(project);
	}
    }
    // signals
    public void OnZapisz(object o, EventArgs args)
    {
	if (project_name==null || project_name=="")
	{
	    OnZapiszJako(o, args);
	    return;
	}
	SaveProject();
    }
    public void OnOtworz(object o, EventArgs args)
    {
	MyApp.FileChooserDialog file = new MyApp.FileChooserDialog("Otworz projekt",
	    window_app, FileChooserAction.Open);
	file.Run();
	if (file.RVal)
	    OpenProject(file.Filename);
	file.Destroy();
    }
    public void OnZapiszJako(object o, EventArgs args)
    {
	MyApp.InputBox i = new MyApp.InputBox("Zapisz jako", "Podaj nazwe pliku", window_app);
	i.Run();
	if (i.RVal==ResponseType.Ok)
	{
	    project_name = i.Entry;
	    SaveProject();
	}
	i.Destroy();
    }
    public void OnNowy(object o, EventArgs args)
    {
	MyApp.InputBox i = new MyApp.InputBox("Nowa przestrzen ", "Nazwa", window_app);
	i.Run();
	if (i.RVal==ResponseType.Ok)
	{
	    if (i.Entry!="")
		workspace.AppendPage(i.Entry);
	}
	i.Destroy();
    }
    public void OnInformacje(object o, EventArgs args)
    {
	string str = "P r o j e k t   z   s y s t e m o w    e k s p e r t o w y c h\n"+
	    "Edytor Sieci Semantycznych v 1.0\n"+
	    "Program napisal : Piotr Borek\n"+
	    "nr indeksu 108494";
	MessageDialog dialog = new MessageDialog(window_app,
	    DialogFlags.Modal,
	    MessageType.Info,
	    ButtonsType.OkCancel,
	    str, 0);
	dialog.Run();
	dialog.Destroy(); 
	
    }
    public void OnZakoncz(object o, EventArgs args)
    {
	bool rval = SaveChanges();
	if (rval) Application.Quit();
    }
    public void OnDeleteWindow(object o, DeleteEventArgs args)
    {
	Application.Quit();
    }
    public void OnEdytujToggled(object o, EventArgs args)
    {
	if (toggle_lock==false)
	{
	    state = ToggleState.MoveNode;
	    toggle_lock = true;
	    togglebutton_mysz.Active = true;
	    togglebutton_nowy.Active = false;
	    togglebutton_polacz.Active = false;
	    toggle_lock = false;
	}
    }
    public void OnNowyToggled(object o, EventArgs args)
    {
	if (toggle_lock==false)
	{
	    state = ToggleState.CreateNode;
	    toggle_lock = true;
	    togglebutton_mysz.Active = false;
	    togglebutton_nowy.Active = true;
	    togglebutton_polacz.Active = false;
	    toggle_lock = false;
	}
    }
    public void OnPolaczToggled(object o, EventArgs args)
    {
	if (toggle_lock==false)
	{
	    state = ToggleState.CreateLink;
	    toggle_lock = true;
	    togglebutton_mysz.Active = false;
	    togglebutton_nowy.Active = false;
	    togglebutton_polacz.Active = true;    
	    toggle_lock = false;
	}
    }
    bool IsInArray(ArrayList array, string str)
    {
	for (int i = 0; i < array.Count; i++)
	{
	    string tmp = (string)array[i];
	    if (tmp.Equals(str)) return false;
	}
	return true;
    }
    ArrayList FindObjects(XmlElement workspace)
    {
	ArrayList array = new ArrayList();
	XmlNodeList w_list = workspace.GetElementsByTagName("node");
	for (int i = 0; i < w_list.Count; i++)
	{
	    XmlElement element = (XmlElement)w_list.Item(i);
	    XmlNodeList c_list = element.GetElementsByTagName("connect");
	    for (int k = 0; k < c_list.Count; k++)
	    {
		XmlElement con = (XmlElement)c_list.Item(k);
		if (con.GetAttribute("type").Equals("object"))
		{
		    string name = con.GetAttribute("id");
		    if (IsInArray(array, name)) array.Add(name);
		}
	    }
	}
	return array;
    }
    bool IsObject(XmlElement node, string attr)
    {
	XmlNodeList list = node.GetElementsByTagName("connect");
	for (int i = 0; i < list.Count; i++)
	{
	    XmlElement element = (XmlElement)list.Item(i);
	    if (element.GetAttribute("id").Equals(attr)) return true;
	}
	return false;
    }
    string GetName(XmlNodeList list, string id)
    {
	for (int i = 0; i < list.Count; i++)
	{
	    XmlElement element = (XmlElement)list.Item(i);
	    if (element.GetAttribute("id").Equals(id))
		return element.GetAttribute("name");
	}
	return "None";
    }
    void nextOSA(XmlNodeList list, string ids, TextBuffer buffer, ref TextIter iter)
    {
	XmlNodeList cons = null;
	XmlElement node = null;
	for (int i = 0; i<list.Count; i++)
	{
	    XmlElement tmp = (XmlElement)list.Item(i);
	    if (tmp.GetAttribute("id").Equals(ids))
	    {
		cons = tmp.GetElementsByTagName("connect");
		node = tmp;
		break;
	    }
	}
	for (int k = 0; k < cons.Count; k++)
	{
	    XmlElement e = (XmlElement)cons.Item(k);
	    if (e.GetAttribute("type").Equals("object"))
	    {
		buffer.Insert(ref iter, e.GetAttribute("to") + " ");
	    }
	}
	for (int k = 0; k < cons.Count; k++)
	{
	    XmlElement e = (XmlElement)cons.Item(k);
	    if (e.GetAttribute("type").Equals("value"))
		buffer.Insert(ref iter, e.GetAttribute("to") + " ");
	    if (node.GetAttribute("name").Equals("lub") && k < cons.Count - 1)
		buffer.Insert(ref iter, "lub ");
	}
	for (int k = 0; k < cons.Count; k++)
	{
	    XmlElement e = (XmlElement)cons.Item(k);
	    if (e.GetAttribute("type").Equals("value"))
		nextOSA(list, e.GetAttribute("id"), buffer, ref iter);
	}
    }
    public void OnGenerujZdania(object o, EventArgs args)
    {
	ArrayList objects;
	Window GenWindow = new Window(WindowType.Toplevel);
	GenWindow.SetSizeRequest(400,250);
	GenWindow.Title = "Zdania";
	ScrolledWindow sw = new ScrolledWindow();
	sw.Show();
	GenWindow.Add(sw);
	TextView view = new TextView();
	view.Show();
	sw.Add(view);
	GenWindow.ShowAll();
	TextBuffer buffer = view.Buffer;
	TextIter iter = buffer.StartIter;
	buffer.Insert(ref iter, "Generuje zdania...\n");
	XmlNodeList works = project.GetElementsByTagName("workspace");
	for (int i=0; i < works.Count; i++)
	{
	    XmlElement work_e = (XmlElement)works.Item(i);
	    buffer.Insert(ref iter, "\n\nZdania dla przestrzeni \"");
	    buffer.Insert(ref iter, work_e.GetAttribute("name"));
	    buffer.Insert(ref iter, "\":\n\n");
	    objects = FindObjects(work_e);
	    for (int k=0; k < objects.Count; k++)
	    {
		XmlNodeList nodes = work_e.GetElementsByTagName("node");
		buffer.Insert(ref iter, GetName(nodes, (string)objects[k]) + " ");
		for (int z = 0; z < nodes.Count; z++)
		{
		    XmlElement a_node = (XmlElement)nodes.Item(z);
		    if (IsObject(a_node, (string)objects[k]))
		    {
 			buffer.Insert(ref iter, a_node.GetAttribute("name")+" ");
			XmlNodeList v_list = a_node.GetElementsByTagName("connect");
			for (int y = 0; y < v_list.Count; y++)
			{
			    XmlElement val = (XmlElement)v_list.Item(y);
			    if (val.GetAttribute("type").Equals("value"))
			    {
				if (!val.GetAttribute("to").Equals("lub"))
				    buffer.Insert(ref iter, val.GetAttribute("to")+" ");
				nextOSA(nodes, val.GetAttribute("id"), buffer, ref iter);
			    }
			}
		    }    
		}
		buffer.Insert(ref iter, "\n");
	    }
	}
    }
}
