# -*- coding: iso-8859-2 -*-

#	Programmer:	Piotr Borek
#	E-mail:		piotrborek@op.pl
#
#	Copyright 2006  Piotr Borek
#
#	Distributed under the terms of the GPL (GNU Public License)
#
#    pyX4 is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import lib.draw
import gui.tooltips

from lib.splash import SplashWindow
SplashWindow.pulse_text_import("widget.buttons")

import gtk
import pango
import cairo

class StockToolItem(gtk.ToolItem):
    def __init__(self, stock_name, tooltip=None):
	gtk.ToolItem.__init__(self)
	self.button = gtk.Button()
	self.button.set_relief(gtk.RELIEF_NONE)
	self.button.set_focus_on_click(False)
	self.button.show()
	if tooltip!=None:
	    gui.tooltips.tips.set_tip(self.button, tooltip)
	self.add(self.button)
	image = gtk.Image()
	image.set_from_stock(stock_name, gtk.ICON_SIZE_MENU)
	image.show()
	self.button.add(image)
    def connect(self, signal_name, func, arg1 = None):
	self.button.connect(signal_name, func, arg1)

class XPMToolItem(gtk.ToolItem):
    def __init__(self, xpm, tooltip=None):
	gtk.ToolItem.__init__(self)
	self.button = gtk.Button()
	self.button.set_relief(gtk.RELIEF_NONE)
	self.button.set_focus_on_click(False)
	self.button.show()
	if tooltip!=None:
	    gui.tooltips.tips.set_tip(self.button, tooltip)
	self.add(self.button)
	image = gtk.Image()
	pixbuf = gtk.gdk.pixbuf_new_from_xpm_data(xpm)
	image.set_from_pixbuf(pixbuf)
	image.show()
	self.button.add(image)
    def connect(self, signal_name, func):
	self.button.connect(signal_name, func)

class SimpleButton(gtk.DrawingArea):
    def __init__(self, text=""):
	gtk.DrawingArea.__init__(self)
        self.width = 10
        self.height = 10
        self.set_size_request(self.width, self.height)
	self.connect("configure-event", self.configure_event)
	self.connect("expose-event", self.expose_event)
        self.connect("enter-notify-event", self.enter_notify_event)
        self.connect("leave-notify-event", self.leave_notify_event)
        self.connect("button-press-event", self.button_press_event)
        self.add_events(gtk.gdk.ENTER_NOTIFY_MASK)
        self.add_events(gtk.gdk.LEAVE_NOTIFY_MASK)
        self.add_events(gtk.gdk.BUTTON_PRESS_MASK)
        self.prelight = False
        self.selected = False
        self.text = text
        self.text_width = 0
        self.text_height = 0
        self.border = 0
        self.set_colors(gtk.gdk.Color(0, 0, 0), gtk.gdk.Color(0xA000, 0xA000, 0xA000))
        self.font_desc = None
        self.font_slant = cairo.FONT_SLANT_NORMAL
        self.font_weight = cairo.FONT_WEIGHT_NORMAL
    def set_font(self, font_name):
        self.font_desc = pango.FontDescription(font_name)
        self.queue_draw()
    def set_font_style(self, style):
        self.font_slant = cairo.FONT_SLANT_NORMAL
        self.font_weight = cairo.FONT_WEIGHT_NORMAL
        if (style & lib.draw.FONT_STYLE_ITALIC)==lib.draw.FONT_STYLE_ITALIC:
            self.font_slant = cairo.FONT_SLANT_ITALIC
        if (style & lib.draw.FONT_STYLE_BOLD)==lib.draw.FONT_STYLE_BOLD:
            self.font_weight = cairo.FONT_WEIGHT_BOLD
        self.queue_draw()
    def get_font_style(self):
        font_style = 0
        if self.font_slant==cairo.FONT_SLANT_NORMAL and self.font_weight==cairo.FONT_WEIGHT_NORMAL:
            font_style = lib.draw.FONT_STYLE_NORMAL
        else:
            if self.font_slant==cairo.FONT_SLANT_ITALIC:
                font_style |= lib.draw.FONT_STYLE_ITALIC
            if self.font_weight==cairo.FONT_WEIGHT_BOLD:
                font_style |= lib.draw.FONT_STYLE_BOLD
        return font_style
    def set_border(self, border):
        self.border = border
        self.queue_draw()
    def set_colors(self, text, background):
        self.color_text = text
        self.color_background = background
        self.queue_draw()
    def set_text_color(self, text):
        self.color_text = text
        self.queue_draw()
    def get_text_color(self):
        return self.color_text
    def set_background_color(self, background):
        self.color_background = background
        self.queue_draw()
    def get_background_color(self):
        return self.color_background
    def set_selected(self, selected):
        self.selected = selected
        self.queue_draw()
    def set_source_color(self, ctx, color):
        ctx.set_source_rgb(color.red/65535.0, color.green/65535.0, color.blue/65535.0)        
    def configure_event(self, widget, event, data=None):
        self.width = event.width
        self.height = event.height
    def expose_event(self, widget, event, data=None):
        ctx = self.window.cairo_create()
        if self.font_desc:
            family = self.font_desc.get_family()
            size = pango.PIXELS(self.font_desc.get_size())
            ctx.select_font_face(family, self.font_slant, self.font_weight)
            ctx.set_font_size(size + 4)
        if self.prelight:
            ctx.set_source_rgba(self.color_text.red/65535.0,
                                self.color_text.green/65535.0,
                                self.color_text.blue/65535.0,
                                0.6)
        else:
            self.set_source_color(ctx, self.color_background)
        ctx.rectangle(0, 0, self.width, self.height)
        ctx.fill()
        if (self.text_width==0 or self.text_height==0) and self.text!="":
            text_extents = ctx.text_extents(self.text)
            self.text_width = int(text_extents[4])
            self.text_height = int(ctx.font_extents()[2] + 1)
            self.set_size_request(self.text_width + self.border, self.text_height + self.border)
            return
        if self.text!="":
            y = (self.height - self.text_height)/2 + self.text_height - int(ctx.font_extents()[1])
            ctx.move_to(self.border, y)
            self.set_source_color(ctx, self.color_text)
            ctx.show_text(self.text)
    def button_press_event(self, widget, event, data=None):
        if event.button==1:
            if self.selected:
                self.set_selected(False)
            else:
                self.set_selected(True)
    def enter_notify_event(self, widget, event, data=None):
        self.prelight = True
        self.queue_draw()
    def leave_notify_event(self, widget, event, data=None):
        self.prelight = False
        self.queue_draw()
