# -*- coding: iso-8859-2 -*-

#	Programmer:	Piotr Borek
#	E-mail:		piotrborek@op.pl
#
#	Copyright 2006  Piotr Borek
#
#	Distributed under the terms of the GPL (GNU Public License)
#
#    pyX4 is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import lib.colors
import lib.draw

from lib.splash import SplashWindow
SplashWindow.pulse_text_import("widget.colorselection")

import gtk
import gobject

class ColorViewWidget(gtk.DrawingArea):
    def __init__(self):
        gtk.DrawingArea.__init__(self)
        self.width = 0
        self.height = 0
        self.connect("expose-event", self.expose_event)
        self.connect("configure-event", self.configure_event)
        self.color = gtk.gdk.Color(0, 0, 0)
    def set_color_from_number(self, number):
        val = lib.colors.color_list[number]
        red   = val[0]*257
        green = val[1]*257
        blue  = val[2]*257
        self.set_color_gdk(gtk.gdk.Color(red, green, blue))
    def set_color_from_dict(self, name):
        val   = lib.colors.color_dict[name]
        red   = val[0]*257
        green = val[1]*257
        blue  = val[2]*257
        self.set_color_gdk(gtk.gdk.Color(red, green, blue))
    def set_color_rgb(self, red, green, blue):
        self.set_color_gdk(gtk.gdk.Color(red*257, green*257, blue*257))
    def set_color_gdk(self, color):
        self.color = self.get_colormap().alloc_color(color)
	self.queue_draw()
    def expose_event(self, widget, event, data=None):
	ctx = self.window.cairo_create()
	ctx.set_source_rgb(self.color.red/65535.0, self.color.green/65535.0, self.color.blue/65535.0)
	ctx.rectangle(0, 0, self.width, self.height)
	ctx.fill()
    def configure_event(self, widget, event, data=None):
        self.width = event.width
        self.height = event.height

class ColorListWidget(gtk.ScrolledWindow):
    """Widget tworzacy przewijana liste kolorow z pliku lib.colors"""
    def __init__(self, width=-1, height=-1):
        """Konstruktor"""
	gtk.ScrolledWindow.__init__(self)
        self.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
	xpm = [
	    '60 10 3 1',
	    '. c None',
	    'x c #000000',
	    '@ c #FFFFFF',
	    '............................................................',
	    '..xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx..',
	    '..x@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@x..',
	    '..x@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@x..',
	    '..x@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@x..',
	    '..x@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@x..',
	    '..x@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@x..',
	    '..x@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@x..',
	    '..xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx..',
	    '............................................................'
	]
	pixbuf = gtk.gdk.pixbuf_new_from_xpm_data(xpm)
	liststore = gtk.ListStore(str, gtk.gdk.Pixbuf, int)
	cnt = 0
	for key, val in lib.colors.color_dict.iteritems():
	    xpm[3] = '@ c #%02X%02X%02X' % (val[0], val[1], val[2])
	    pixbuf = gtk.gdk.pixbuf_new_from_xpm_data(xpm)
	    liststore.append([key, pixbuf, cnt])
	    cnt = cnt + 1
	sort = gtk.TreeModelSort(liststore)
	sort.set_sort_column_id(0, gtk.SORT_ASCENDING)
	self.treeview = gtk.TreeView(sort)
	if width!=-1 or height!=-1:
	    self.set_size_request(width, height)
	self.add(self.treeview)
	tvcolumn = gtk.TreeViewColumn(_('Colors'))
	cell_pixbuf = gtk.CellRendererPixbuf()
	tvcolumn.pack_start(cell_pixbuf, False)
	tvcolumn.set_attributes(cell_pixbuf, pixbuf=1)
	cell_text = gtk.CellRendererText()
	tvcolumn.pack_start(cell_text, False)
	tvcolumn.set_attributes(cell_text, text=0)
    	self.treeview.append_column(tvcolumn)
	self.treeview.show_all()
        self.treeview.set_cursor(0)
        self.treeview.connect('cursor-changed', self.cursor_changed)
        self.callback=None
    def set_callback(self, callback):
        self.callback = callback
    def alloc_color_from_dict(self, name):
        """
        Fukcja ta alokuje kolor o podanym numerze z listy lib.colors
        @param name: nazwa koloru z lib.colors.color_dict
        @return Nadajacy sie do uzytku kolor gtk.gdk.Color
        """
        val   = lib.colors.color_dict[name]
        red   = val[0]*257
        green = val[1]*257
        blue  = val[2]*257
        return self.get_colormap().alloc_color(red, green, blue)
    def alloc_color_from_list(self, number):
        """
        Fukcja ta alokuje kolor o podanym numerze z listy lib.colors
        @param number: numer koloru z listy lib.colors.color_list
        @return Nadajacy sie do uzytku kolor gtk.gdk.Color
        """
        val = lib.colors.color_list[number]
        red   = val[0]*257
        green = val[1]*257
        blue  = val[2]*257
        return self.get_colormap().alloc_color(red, green, blue)
    def cursor_changed(self, treeview, data=None):
        """Obsluga sygnalu zmiany koloru z przewijanej listy kolorow"""
	model, iter = treeview.get_selection().get_selected()
	color_name = model.get_value(iter, 0)
        if self.callback:
            val = lib.colors.color_dict[color_name]
            red   = val[0]
            green = val[1]
            blue  = val[2]
            self.callback(gtk.gdk.Color(red * 257, green * 257, blue * 257))

class ColorTableWidget(gtk.VBox):
    """Widget tworzacy tablice kolorowych przyciskow o rozmiarze 11x11"""
    def __init__(self, columns = 5):
        """Konstruktor"""
        gtk.VBox.__init__(self)
	self.color_list = ['Black', 'DarkGray', 'Gray', 'LightGray', 'White', 
	    'DarkRed', 'Red', 'LightCoral', 'DarkGreen', 'Green', 'LightGreen',
            'DarkBlue', 'Blue', 'CornflowerBlue', 'Brown', 'Brown1', 'Gold4',
            'Yellow', 'Khaki1', 'DarkCyan', 'Cyan', 'Aquamarine', 'DarkMagenta',
            'Magenta', 'Violet']
        self.__sig_function = None
        rows = len(self.color_list)/columns
        if len(self.color_list) % columns:
            rows += 1
        table = gtk.Table(rows, columns, False)
        table.show()
	for i in range(0, rows):
	    for k in range(0, columns):
                try:
                    color = self.color_list[i * rows + k]
                except:
                    break
                colorview = ColorViewWidget()
                colorview.set_color_from_dict(color)
                colorview.show()
                button = gtk.Button()
                button.add(colorview)
		button.set_size_request(22, 22)
                button.set_border_width(0)
		button.set_focus_on_click(False)
		button.connect('clicked', self.color_changed, color)
		button.show()
		table.attach(button, k, k + 1, i, i + 1)
        table.set_border_width(5)
        self.pack_start(table, True, True)
        self.callback = None
    def set_callback(self, callback):
        self.callback = callback
    def alloc_color_from_dict(self, name):
        """
        Fukcja ta alokuje kolor o podanym numerze z listy lib.colors
        @param name: nazwa koloru z lib.colors.color_dict
        @return Nadajacy sie do uzytku kolor gtk.gdk.Color
        """
        val   = lib.colors.color_dict[name]
        red   = val[0]*257
        green = val[1]*257
        blue  = val[2]*257
        return self.get_colormap().alloc_color(red, green, blue)
    def alloc_color_from_list(self, number):
        """
        Fukcja ta alokuje kolor o podanym numerze z listy lib.colors
        @param number: numer koloru z listy lib.colors.color_list
        @return Nadajacy sie do uzytku kolor gtk.gdk.Color
        """
        val = lib.colors.color_list[number]
        red   = val[0]*257
        green = val[1]*257
        blue  = val[2]*257
        return self.get_colormap().alloc_color(red, green, blue)
    def color_changed(self, button, color_name):
        """Obsluga sygnalu zmiany koloru z tablicy kolorow"""
        if self.callback:
            red, green, blue = lib.colors.color_dict[color_name]
            self.callback(gtk.gdk.Color(red * 257, green * 257, blue * 257))

class ColorWidget(gtk.VBox):
    def __init__(self, tab_pos=gtk.POS_LEFT):
        gtk.VBox.__init__(self)
        self.notebook = gtk.Notebook()
        self.notebook.set_tab_pos(tab_pos)
        self.notebook.show()
        self.pack_start(self.notebook, True, True)
        self.color_table = ColorTableWidget()
        self.color_table.show()
        self.append_page(self.color_table, _("Table"))
        self.color_list = ColorListWidget()
        self.color_list.show()
        self.append_page(self.color_list, _("List"))
        self.color_system = gtk.ColorSelection()
        self.handler_color = self.color_system.connect('color-changed', self.color_changed_event)
        self.color_system.show()
        self.append_page(self.color_system, _("System"))
        self.toggle_b = gtk.CheckButton(_("Bold"))
        self.handler_b = self.toggle_b.connect("toggled", self.toggled_event)
        self.toggle_i = gtk.CheckButton(_("Italic"))
        self.handler_i = self.toggle_i.connect("toggled", self.toggled_event)
        self.hbox = gtk.HBox()
        self.pack_start(self.hbox, False, False, 4)
        label = gtk.Label(_("Font style:"))
        label.show()
        self.hbox.pack_start(label, False, False, 4)
        self.toggle_b.show()
        self.hbox.pack_start(self.toggle_b, False, False, 4)            
        self.toggle_i.show()
        self.hbox.pack_start(self.toggle_i, False, False, 4)
        self.attr_callback = None
        self.color_callback = None
    def set_attr_callback(self, attr_callback):
        self.attr_callback = attr_callback
    def set_color_callback(self, color_callback):
        self.color_table.set_callback(color_callback)
        self.color_list.set_callback(color_callback)
        self.color_callback = color_callback
    def show_font_style(self, vis=True):
        if vis:
            self.hbox.show()
        else:
            self.hbox.hide()
    def set_font_style(self, font_style):
        self.toggle_i.handler_block(self.handler_i)
        self.toggle_b.handler_block(self.handler_b)        
        self.toggle_i.set_active(False)
        self.toggle_b.set_active(False)
        if (font_style & lib.draw.FONT_STYLE_ITALIC)==lib.draw.FONT_STYLE_ITALIC:
            self.toggle_i.set_active(True)
        if (font_style & lib.draw.FONT_STYLE_BOLD)==lib.draw.FONT_STYLE_BOLD:
            self.toggle_b.set_active(True)
        self.toggle_i.handler_unblock(self.handler_i)
        self.toggle_b.handler_unblock(self.handler_b)
    def set_color(self, gdk_color):
        self.color_system.handler_block(self.handler_color)
        self.color_system.set_current_color(gdk_color)
        self.color_system.set_previous_color(gdk_color)
        self.color_system.handler_unblock(self.handler_color)
    def append_page(self, widget, text):
        label = gtk.Label(text)
        label.show()
        scrolled = gtk.ScrolledWindow()
        scrolled.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        scrolled.show()
        scrolled.add_with_viewport(widget)
        self.notebook.append_page(scrolled, label)
    def color_changed_event(self, widget, data=None):
        if self.color_callback:
            self.color_callback(widget.get_current_color())
    def toggled_event(self, widget, data=None):
        font_style = 0
        if self.toggle_b.get_active():
            font_style |= lib.draw.FONT_STYLE_BOLD
        if self.toggle_i.get_active():
            font_style |= lib.draw.FONT_STYLE_ITALIC
        if font_style==0:
            font_style = lib.draw.FONT_STYLE_NORMAL
        if self.attr_callback:
            self.attr_callback(font_style)

class ColorSelectionBox(gtk.VBox):
    """Widget tworzacy gui do selekcji wybranego koloru"""
    def __init__(self, attr_vis=False):
        gtk.VBox.__init__(self)
        table = gtk.Table(2, 2)
        table.show()
        hbox = gtk.HBox()
        hbox.show()
        left_vbox = gtk.VBox()
        left_vbox.show()
        right_vbox = gtk.VBox()
        right_vbox.show()
        hbox.pack_start(left_vbox, True, True)
        hbox.pack_start(right_vbox, True, True)
        self.pack_start(hbox, True, True)
        combo = self.create_combobox()
        combo.show()
        left_vbox.pack_start(combo, False, False)
        self.active_widget = 0
        combo.set_active(self.active_widget)

        self.color_selection, self.color_widgets = self.create_colorsels()
        self.vbox_widget = gtk.VBox()
        self.vbox_widget.show()
        self.vbox_widget.pack_start(self.color_widgets[self.active_widget], True, True, 0)
        left_vbox.pack_start(self.vbox_widget)

        self.string_list = gtk.ListStore(str, gtk.gdk.Color, int)
        self.treeview = self.create_treeview(self.string_list)
        self.treeview.show()
        scrolled_window = gtk.ScrolledWindow()
        scrolled_window.show()
        scrolled_window.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
        scrolled_window.add(self.treeview)
        right_vbox.pack_start(scrolled_window)
        
        self.create_colorview()
        right_vbox.pack_start(self.frame, False, False)

        self.toggle_b = gtk.CheckButton(_("Bold"))
        self.toggle_i = gtk.CheckButton(_("Italic"))
        if attr_vis:
            separator = gtk.HSeparator()
            separator.show()
            self.pack_start(separator, False, False, 4)
            hbox = gtk.HBox()
            hbox.show()
            self.pack_start(hbox, False, False, 4)
            label = gtk.Label(_("Font style :"))
            label.show()
            hbox.pack_start(label, False, False, 4)
            self.toggle_b.show()
            hbox.pack_start(self.toggle_b, False, False, 4)            
            self.toggle_i.show()
            hbox.pack_start(self.toggle_i, False, False, 4)
        self.cur_color = self.get_colormap().alloc_color(0, 0, 0)
        self.cur_color_area.set_color_rgb(0, 0, 0)
        self.old_color_area.set_color_rgb(0, 0, 0)
        self.cur_iter = None
    def create_treeview(self, string_list):
        treeview = gtk.TreeView(string_list)
        treeview.set_headers_visible(False)
        treeview.connect("cursor-changed", self.string_list_changed)
        treeview.show()
        tvcolumn = gtk.TreeViewColumn(None)
        treeview.append_column(tvcolumn)
        text_cell = gtk.CellRendererText()
        tvcolumn.pack_start(text_cell, True)
        tvcolumn.set_attributes(text_cell, text=0)
        return treeview
    def create_combobox(self):
        combobox = gtk.combo_box_new_text()
        combobox.connect("changed", self.color_widget_changed)
        combobox.show()
        combobox.append_text(_("Choose color from table"))
        combobox.append_text(_("Choose color from list"))
        combobox.append_text(_("Choose color from system box"))
        return combobox
    def create_colorsels(self):
        color_selection_vbox = gtk.VBox()
        label = gtk.Label()
        label.show()
        color_selection_vbox.pack_start(label, True, True, 0)
        color_selection = gtk.ColorSelection()
        color_selection.show()
        color_selection_vbox.pack_start(color_selection, False, False, 0)
        label = gtk.Label()
        label.show()
        color_selection_vbox.pack_start(label, True, True, 0)
        color_widgets = (ColorTableWidget(), ColorListWidget(), color_selection_vbox)
        color_widgets[0].set_callback(self.color_changed)
        color_widgets[1].set_callback(self.color_changed)
        color_selection.connect("color-changed", self.color_changed_sys)
        for widget in color_widgets:
            widget.show()
        return color_selection, color_widgets
    def create_colorview(self):
        self.frame = gtk.Frame(_("Active color"))
        self.frame.show()
        self.cur_color_area = ColorViewWidget()
        self.cur_color_area.show()
        self.cur_color_area.set_size_request(60, 40)
        self.old_color_area = ColorViewWidget()
        self.old_color_area.show()
        self.old_color_area.set_size_request(60, 40)
        hbox = gtk.HBox()
        hbox.show()
        hbox.pack_start(self.old_color_area, True, True, 0)
        hbox.pack_start(self.cur_color_area, True, True, 0)
        self.frame.add(hbox)
        self.frame.set_border_width(5)
    def append_text(self, text, current_color, font_style=lib.draw.FONT_STYLE_NORMAL):
        """
        Funkcja ta sluzy do dodania do listy w okienku pojedynczej pozycji,
        ktora bedzie sluzyla jako wlasciwosc, ktorej kolor uzytkownik moze sobie
        wybrac.
        @param text: nazwa wyswietlanej wlasciwosci
        @param current_color: aktualny kolor dla podanej wlasciwosci - typu
        gtk.gdk.Color
        """
        if current_color==None:
            current_color = gtk.gdk.Color(0, 0, 0)
        red = current_color.red
        green = current_color.green
        blue = current_color.blue
        default_color = self.get_colormap().alloc_color(red, green, blue)
        if self.string_list.get_iter_root() == None:
            self.string_list.append([text, default_color, font_style])
            self.treeview.set_cursor("0")
        else:
            self.string_list.append([text, default_color, font_style])
    def modify_text(self, text, current_color, font_style=lib.draw.FONT_STYLE_NORMAL):
        if current_color==None:
            current_color = gtk.gdk.Color(0, 0, 0)
        red = current_color.red
        green = current_color.green
        blue = current_color.blue
        default_color = self.get_colormap().alloc_color(red, green, blue)
        iter = self.string_list.get_iter_root()
        while iter!=None:
            if self.string_list.get_value(iter, 0)==text:
                self.string_list.set_value(iter, 1, default_color)
                self.string_list.set_value(iter, 2, font_style)
                break
            iter = self.string_list.iter_next(iter)
        if iter==None:
            self.append_text(text, current_color, font_style)
    def set_current_color(self, name, color):
        """
        Funckja ta ustawia aktualny kolor dla wybranej wlasciwosci.
        Uwaga: wartosc starego koloru jest nadpisywana przez wartosc aktualnego koloru
        @param name: nazwa wlasciwosci
        @param color: wartosc koloru dla wybranej wlasciwosci
        """
        red = color.red
        green = color.green
        blue = color.blue
        color = self.get_colormap().alloc_color(red, green, blue)
        self.color_widgets[2].set_previous_color(self.cur_color)
        self.cur_color = color
        self.cur_color_area.set_color_gdk(color)
        self.color_widgets[2].set_current_color(self.cur_color)
    def apply_current_color(self):
        """
        Funckja ta ustawia aktualny kolor (z pola aktualnego koloru) w polu wyswietlania
        poprzedniego koloru dla wlasciwosci aktualnie zaznaczonej przez uzytkownika
        """
        self.string_list.set_value(self.cur_iter, 1, self.cur_color)
        x = lib.draw.FONT_STYLE_NORMAL
        active = self.toggle_b.get_active()
        if active:
            x |= lib.draw.FONT_STYLE_BOLD
            x &= ~lib.draw.FONT_STYLE_NORMAL
        active = self.toggle_i.get_active()
        if active:
            x |= lib.draw.FONT_STYLE_ITALIC
            x &= ~lib.draw.FONT_STYLE_NORMAL
        self.string_list.set_value(self.cur_iter, 2, x)
        self.old_color_area.set_color_gdk(self.cur_color)
        if self.active_widget == 2:
            self.color_widgets[2].set_previous_color(self.cur_color)
    def get_current_color(self, number):
        """
        Funkcja ta pobiera aktualnie aktywny (wybrany przez uzytkownika) kolor.
        @param number: numer wlasciwosci w liscie dla ktorej pobierana jest wartosc koloru
        @return: wybrany kolor i gotowy do uzycia gtk.gdk.Color
        """
        iter = self.string_list.get_iter(number)
        return self.string_list.get_value(iter, 1)
    def get_current_style(self, number):
        iter = self.string_list.get_iter(number)
        return self.string_list.get_value(iter, 2)
    def reset_color(self):
        """Funckja ta ustawia poczatkowy kolor w polu wyswietlania aktualnego koloru"""
        self.cur_color_area.set_color_gdk(self.string_list.get_value(self.cur_iter, 1))
    def color_changed(self, gdk_color):
        """Obsluga sygnalu zmiany przez uzytkownika koloru na inny"""
#        self.cur_color = self.get_colormap().alloc_color(red*255, green*255, blue*255)
        self.cur_color = self.get_colormap().alloc_color(gdk_color)
        self.cur_color_area.set_color_gdk(self.cur_color)
    def color_changed_sys(self, colorselection, data=None):
        self.cur_color = self.get_colormap().alloc_color(colorselection.get_current_color())
        self.cur_color_area.set_color_gdk(self.cur_color)
    def string_list_changed(self, treeview, data=None):
        """Obsluga sygnalu zmiany wskazywanej pozycji na liscie"""
        selection = treeview.get_selection()
        model, iter = selected = selection.get_selected()
        self.cur_iter = iter
        color = self.string_list.get_value(iter, 1)
        self.cur_color_area.set_color_gdk(color)
        self.color_selection.set_current_color(color)
        color = self.string_list.get_value(iter, 1)
        self.old_color_area.set_color_gdk(color)
        self.color_selection.set_previous_color(color)
        attr =  self.string_list.get_value(iter, 2)
        self.toggle_b.set_active(attr & lib.draw.FONT_STYLE_BOLD)
        self.toggle_i.set_active(attr & lib.draw.FONT_STYLE_ITALIC)
    def color_widget_changed(self, combobox, data=None):
        """Obsluga sygnalu zmiany czesci wyswietlajacej kolory"""
        active = combobox.get_active()
        if active != self.active_widget:
            self.vbox_widget.remove(self.color_widgets[self.active_widget])
            self.vbox_widget.pack_start(self.color_widgets[active], True, True, 0)
            self.active_widget = active
            if active == 2:
                self.frame.hide()
                self.color_selection.set_current_color(self.cur_color)
                self.color_selection.set_previous_color(self.string_list.get_value(self.cur_iter, 1))
            else:
                self.frame.show()

class ColorSelectionDialog(gtk.Dialog):
    """Widget okna wyboru koloru"""
    def __init__(self):
        """Konstruktor"""
	gtk.Dialog.__init__(self)
	self.selection = ColorSelectionBox()
	self.selection.show()
	self.vbox.add(self.selection)
	self.selection.set_size_request(400, 249)
	self.add_button('gtk-ok', gtk.RESPONSE_OK)
	self.add_button('gtk-cancel', gtk.RESPONSE_CANCEL)
    def get_current_color(self):
        """
        Funckja ta zwraca aktualnie wybrany kolor
        @return: Nadajacy sie do uzytku kolor gtk.gdk.Color
        """
	return self.selection.get_current_color()
