# -*- coding: iso-8859-2 -*-

#	Programmer:	Piotr Borek
#	E-mail:		piotrborek@op.pl
#
#	Copyright 2006  Piotr Borek
#
#	Distributed under the terms of the GPL (GNU Public License)
#
#    pyX4 is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import lib.core
import lib.draw

from lib.splash import SplashWindow
SplashWindow.pulse_text_import("widget.lineview")

import gtk
import pango
import gc

TEXT_BORDER = 5
ALLIGN_TOP = 0
ALLIGN_BOTTOM = 1

class LineView(gtk.DrawingArea):
    def __init__(self, border = 6):
	gtk.DrawingArea.__init__(self)
	self.border = border
	self.connect("configure-event", self.configure_event)
	self.connect("expose-event", self.expose_event)
        self.connect("delete-event", self.delete_event)
        self.visible = False
	self.start_num = 1
	self.line_count = 1
	self.line_height = 1
	self.width = 0
        self.height = 0
        self.set_double_buffered(False)
        self.draw = lib.draw.CairoDraw()
        self.drawed = False
	self.y_offset = 0
	self.allign = ALLIGN_TOP
        lib.core.events.connect(self, 'conf.editor.font', self.font_changed)
        lib.core.events.connect(self, 'conf.editor.lineview', self.font_changed)
        self.create_renderer()
        self.set_font()
    def change_allign(self, allign):
	if self.allign != allign:
	    if allign == ALLIGN_TOP:
		self.y_offset = 0
	    else:
		self.y_offset = self.height - 2 * TEXT_BORDER - self.line_count * self.line_height
	    self.allign = allign
	    return True
	return False
    def set_start_number(self, number):
        if self.visible:
            width = self.get_text_width(number) + self.border
            delta = self.start_num - number
            self.start_num = number
	    error = 0
	    if delta < 0:
		if self.change_allign(ALLIGN_BOTTOM):
		    error = self.y_offset
	    else:
		error = -self.y_offset
		self.change_allign(ALLIGN_TOP)
            if width != self.width:
                self.set_size_request(width, -1)
            else:
                self.draw.scroll_dy(delta * self.line_height + error, 0, TEXT_BORDER, self.width, self.height - 2 * TEXT_BORDER)
                if delta < 0:
                    for i in range(self.line_count + delta, self.line_count + 1):
                        self.draw_line(i)
                else:
                    for i in range(delta):
                        self.draw_line(i)
                gtk.DrawingArea.queue_draw(self)
        else:
            self.start_num = number
    def queue_draw(self):
        if self.drawed:
            self.drawed = False
            gtk.DrawingArea.queue_draw(self)
    def show(self):
        self.visible = True
        gtk.DrawingArea.show(self)
    def hide(self):
        self.visible = False
        gtk.DrawingArea.hide(self)
    def set_size_request(self, width, height):
        w, h = self.size_request()
        if width == -1:
            width = w
        if height == -1:
            height = h
        if self.width != width or self.height != height:
            self.width = width
            self.height = height
            gtk.DrawingArea.set_size_request(self, width, height)
            self.draw.create_size(self.width, self.height)
            self.queue_draw()
    def get_text_width(self, number):
        s = "0" * len("%i" % (number + self.line_count)) + ":"
        width1, height = self.draw.get_text_size(s)
        width2, height = self.draw.get_text_size("000:")
        return max(width1, width2)
    def draw_line(self, num):
        y = TEXT_BORDER + self.line_height * num + self.y_offset
        text = "%i:" % (num + self.start_num)
        width, height = self.draw.get_text_size(text)
        if self.height - y - TEXT_BORDER < 0:
            return
        elif self.height - y - TEXT_BORDER < height:
            height = self.height - y - TEXT_BORDER
        clip_rect = gtk.gdk.Rectangle(0, y, self.width + self.border, height)
        self.draw.set_clip_rect(clip_rect)
        p = lib.core.share_data.get_data('conf.editor.lineview')
        self.draw.set_color_gdk(p.background_color)
        self.draw.clear()
#        self.draw.set_color_gdk(p.text_color)
        self.draw.set_font_clip_rect(clip_rect)
        self.draw.set_font_color_gdk(p.text_color)
        self.draw.draw_text(self.width - self.border/2 - width, y, text)
        self.draw.set_clip_rect()
    def draw_all(self):
        p = lib.core.share_data.get_data('conf.editor.lineview')
        self.draw.set_color_gdk(p.background_color)
        self.draw.set_clip_rect()
        self.draw.clear()
        for i in range(self.line_count + 1):
            self.draw_line(i)
        self.drawed = True
    def configure_event(self, widget, event, data=None):
        width, height = self.draw.get_text_size('0')
        self.line_height = height
	self.line_count = (event.height - 2 * TEXT_BORDER) / self.line_height
        if self.line_count == 0:
            self.line_count = 1
        width = self.get_text_width(self.start_num) + self.border
        self.set_size_request(width, event.height)
    def expose_event(self, widget, event, data=None):
        if self.drawed:
            self.draw.set_clip_rect(event.area)
        else:
            self.draw_all()
        self.draw.flip(widget.window)
    def delete_event(self, widget, event, data=None):
        lib.core.events.disconnect(self)
    def set_font(self):
        p = lib.core.share_data.get_data('conf.editor.font')
	font_desc = pango.FontDescription(p.font_name)
        p = lib.core.share_data.get_data('conf.editor.lineview')
	font_desc.set_weight(p.font_weight)
	font_desc.set_style(p.font_style)
	self.modify_font(font_desc)        
        self.draw.set_font_description(font_desc)
    def create_renderer(self):
        self.draw = None
        gc.collect()
        self.draw = lib.draw.CairoDraw()
    def font_changed(self, share_data=None, param=None, data=None):
        self.set_font()
	self.queue_draw()
