# -*- coding: iso-8859-2 -*-

#	Programmer:	Piotr Borek
#	E-mail:		piotrborek@op.pl
#
#	Copyright 2006  Piotr Borek
#
#	Distributed under the terms of the GPL (GNU Public License)
#
#    pyX4 is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import widget.buttons

from lib.splash import SplashWindow
SplashWindow.pulse_text_import("widget.menu")

import gtk

import lib.GPL

class RadioGroupSubMenu(gtk.MenuItem):
    def __init__(self, label, clicked_callback = None):
        super(RadioGroupSubMenu, self).__init__(label)
        self.menu = gtk.Menu()
        self.menu.show()
        self.set_submenu(self.menu)
        self.group_item = None
        self.clicked_callback = clicked_callback
        self.radios = {}
    def create_radioitem(self, label, pos = -1):
        item = gtk.RadioMenuItem(self.group_item, label)
        if not self.group_item:
            self.group_item = item
        signal_id = None
        if self.clicked_callback:
            signal_id = item.connect('toggled', self.clicked_callback, label)
        item.show()
        if pos == -1:
            self.menu.append(item)
        else:
            self.menu.insert(item, pos)
        self.radios[label] = [item, signal_id]
    def set_active(self, label, active):
        self.handler_block()
        self.radios[label][0].set_active(active)
        self.handler_unblock()
    def handler_block(self):
        for item in self.radios.values():
            item[0].handler_block(item[1])
    def handler_unblock(self):
        for item in self.radios.values():
            item[0].handler_unblock(item[1])

class SubMenu(gtk.MenuItem):
    def __init__(self, label):
        super(SubMenu, self).__init__(label)
        self.menu = gtk.Menu()
        self.menu.show()
        self.set_submenu(self.menu)
    def append(self, item):
        self.menu.append(item)
    def create_item(self, name, label, tooltip, stock_id, clicked_callback=None):
	action = gtk.Action(name, label, tooltip, stock_id)
        item = action.create_menu_item()
        if clicked_callback:
            item.connect('activate', clicked_callback)
	self.menu.append(item)
        return item
    def create_radioitem(self, group, label, clicked_callback=None):
        item = gtk.RadioMenuItem(group, label)
        if clicked_callback:
            item.connect('toggled', clicked_callback)
        item.show()
        self.menu.append(item)
        return item
    def create_checkitem(self, label, toggled_callback=None):
        item = gtk.CheckMenuItem(label)
        if toggled_callback:
            item.connect('toggled', toggled_callback)
        item.show()
        self.menu.append(item)
        return item
    def create_submenu(self, name):
        submenu = SubMenu(name)
        submenu.show()
        self.menu.append(submenu)
        return submenu
    def create_separator(self):
        separator = gtk.SeparatorMenuItem()
        separator.show()
        self.menu.append(separator)
	
class ToolBar(gtk.Toolbar):
    def __init__(self):
	gtk.Toolbar.__init__(self)
    def create_item(self, stock_id, label, clicked_callback=None):
        item = widget.buttons.StockToolItem(stock_id, label)
        if clicked_callback:
            item.connect('clicked', clicked_callback)
        item.show()
        self.add(item)
        return item
    def create_xpmitem(self, xpm, label, clicked_callback=None):
        item = widget.buttons.XPMToolItem(xpm, label)
        if clicked_callback:
            item.connect('clicked', clicked_callback)
        item.show()
        self.add(item)
        return item
    def create_separator(self):
        separator = gtk.SeparatorToolItem()
        separator.show()
        self.add(separator)

class IndentMenu():
    def __init__(self):
        pass
