# -*- coding: iso-8859-2 -*-

#	Programmer:	Piotr Borek
#	E-mail:		piotrborek@op.pl
#
#	Copyright 2006  Piotr Borek
#
#	Distributed under the terms of the GPL (GNU Public License)
#
#    pyX4 is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

__all__ = ["NoteBook"]

import lib.core
import lib.config
import lib.tools
import gui.tooltips
import gui.main_gui
import widget.buttons
import widget.filedialog

from lib.splash import SplashWindow
SplashWindow.pulse_text_import("widget.notebook")

import gtk

class CloseLabel(gtk.HBox):
    """Prywatna klasa do uzytku dla klasy widget.NoteBook"""
    def __init__(self, label, label_tip = None):
        """Konstruktor"""
	gtk.HBox.__init__(self)
	self.label = gtk.Label(label)
	self.label.show()
        if label_tip:
            event_box = gtk.EventBox()
            event_box.show()
            event_box.add(self.label)
	    self.pack_start(event_box, True, True, 0)
            gui.tooltips.tips.set_tip(event_box, label_tip)
        else:
            self.pack_start(self.label, True, True, 0)
	self.button = widget.buttons.StockToolItem("gtk-close", _('Close area'))
	self.button.set_size_request(22, 20)
	self.button.show()
	self.pack_start(self.button, False, False, 0)

class NoteBook(gtk.Notebook):
    """Udoskonalony widget gtk.Notebook"""
    def __init__(self):
        """Konstruktor"""
	gtk.Notebook.__init__(self)
	self.set_scrollable(True)
	self.popup_enable()
        self.connect("key-press-event", self.keypress_event)
	self.connect("switch-page", self.switch_page_event)
        lib.core.events.connect(self, 'main.file.open', self.open_file_event)
    def append_page(self, child, tab_label=None):
        """
        Funkcja ta dodaje widget do panelu NoteBook'a.
        @param child: klasa dodawanego widget'u
        @param tab_label: nazwa pod jaka bedzie wyswietlany dodany widget
        @return: True jesli widget zostal poprawnie dodany do NoteBook'a, False jesli dodawany
        widget istnieje juz w Notebook'u (nie zostanie on ponownie dodany).
        """
        if self.page_num(child) > 0:
            return False
        try:
            label_tip = _("File path:\t\t%s\n"
                          "Name:\t\t%s\n"
                          "Type:\t\t%s\n"
                          "Size: \t\t%s\n"
                          "Modified:\t\t%s\n"
                          "Owner:\t\t%s\n"
                          "Permissions:\t%s") % lib.tools.get_file_info(tab_label)
	    close = CloseLabel(tab_label.split('/')[-1], label_tip)
        except:
            if tab_label==None:
                tab_label = _("No name")
                close = CloseLabel(tab_label, _("Name:\t\t%s") % tab_label)
            else:
                close = CloseLabel(tab_label)
	close.button.connect("clicked", self.close_current_page, child)
	close.show()
	gtk.Notebook.append_page(self, child, close)
	self.set_menu_label_text(child, tab_label)
        self.set_current_page(self.page_num(child))
        return True
    def get_current_widget(self):
        n = self.get_current_page()
        return self.get_nth_page(n)
    def save_file(self):
        n = self.get_current_page()
        if n >= 0:
            w = self.get_nth_page(n)
            try:
                buffer = w.get_text_controller().buffer
                if buffer.get_file_name():
                    buffer.write_file()
                else:
                    self.save_file_as()
            except:
                pass
    def save_file_as(self):
        fc = widget.filedialog.FileDialog(action=gtk.FILE_CHOOSER_ACTION_SAVE)
        response = fc.run()
        if response == gtk.RESPONSE_ACCEPT:
            n = self.get_current_page()
            if n >= 0:
                w = self.get_nth_page(n)
                try:
                    buffer = w.get_text_controller().buffer
                    buffer.write_file_as(fc.get_filename(), fc.get_encoding())
                except:
                    pass
        fc.destroy()
    def save_all_files(self):
        warning = False
        for i in range(0, self.get_n_pages()):
            buffer = self.get_nth_page(i).get_text_controller().buffer
            if buffer.get_file_name() == None:
                warning = True
            else:
                if buffer.is_modified():
                    buffer.write_file()
        if warning:
            dialog = gtk.MessageDialog(None, gtk.DIALOG_MODAL, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK,
                                       _("\nCannot save unnamed buffers"))
            dialog.set_title(_("Warning"))
            dialog.run()
            dialog.destroy()
    def close_current_page(self, close_button, page_widget=None):
        """Procedura obslugi sygnalu zamkniecia pojedynczej pozycji w NoteBook'u"""
	num = self.page_num(page_widget)
	self.remove_page(num)
        event = gtk.gdk.Event(gtk.gdk.DELETE)
        page_widget.emit("delete-event", event)
        textview = self.get_nth_page(0)
        if textview==None:
            gui.main_gui.menu.edit_menu.filetype_menu.set_sensitive(False)
    def keypress_event(self, widget, event, data=None):
        n = self.get_current_page()
        if n >= 0:
            widget = self.get_nth_page(n)
            widget.emit("key-press-event", event)
        return True
    def switch_page_event(self, notebook, page, page_num, data=None):
	textview = self.get_nth_page(page_num)
        try:
	    gui.main_gui.menu.edit_menu.filetype_menu.set_active(textview.syntax_name, True)
            gui.main_gui.menu.edit_menu.filetype_menu.set_sensitive(True)
        except:
            gui.main_gui.menu.edit_menu.filetype_menu.set_sensitive(False)
    def open_file_event(self, obj, param, data=None):
        textview = gui.main_gui.app.create_textview(param[1])
        self.append_page(textview, param[1])
