# -*- coding: iso-8859-2 -*-

#	Programmer:	Piotr Borek
#	E-mail:		piotrborek@op.pl
#
#	Copyright 2006  Piotr Borek
#
#	Distributed under the terms of the GPL (GNU Public License)
#
#    pyX4 is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

__all__ = ['TextView']

import lib.core
import lib.draw
import lib.env_data
import lib.syntax.dfa
import lib.syntax.parser
import widget.lineview
import lib.text

from lib.splash import SplashWindow
SplashWindow.pulse_text_import("widget.text")

import gtk
import gc

TEXT_BORDER = 5

ALLIGN_TOP = 0
ALLIGN_BOTTOM = 1

class TextController(object):
    def __init__(self, textview, width_function=None):
        self.textview = textview
        self.textview.drawable.connect('scroll-event', self.scroll_event)
        self.textview.drawable.add_events(gtk.gdk.SCROLL_MASK)
        self.cursor_real_x = 0
	self.allign = ALLIGN_TOP
	self.y_offset = 0
        self.syntax_name = 'text'
        self.syntax_dfa = lib.syntax.dfa.DFA()
	self.syntax_parser = lib.syntax.parser.SyntaxParser(self.syntax_dfa, width_function)
        self.buffer = None
        self.formatted_text_cache = []
        self.line_start = 0
    def set_text_buffer(self, buffer, start):
        self.buffer = buffer
        self.line_start = start
    def set_syntax(self, syntax):
        self.syntax_name = syntax
        self.syntax_dfa.set_syntax(syntax)
        self.syntax_dfa.set_state(None)
        self.delete_formatted_text_cache(start = 0)
        self.textview.set_color_attrs(syntax)
        self.textview.need_full_draw = True
        self.textview.queue_draw()
    def change_allign(self, allign):
	if self.allign != allign:
	    if allign == ALLIGN_TOP:
		self.y_offset = 0
	    else:
		self.y_offset = self.textview.drawable_height - 2 * TEXT_BORDER - self.textview.line_count * self.textview.line_height
	    self.allign = allign
	    return True
	return False
    def create_proper_formatted_text(self, i):
        line = self.buffer[i].get()
        if i > 0:
            cache = self.formatted_text_cache[i - 1]
            last_state = cache.text[-1].state
            last_attr = cache.text[-1].attr
            formatted_text = self.syntax_parser.create_formatted_text(line, last_state, last_attr)
        else:
            formatted_text = self.syntax_parser.create_formatted_text(line, None, None)
        return formatted_text
    def update_formatted_text_cache(self, i):
        formatted_text = self.create_proper_formatted_text(i)
        self.formatted_text_cache[i] = formatted_text
        return formatted_text
    def delete_formatted_text_cache(self, start=-1, end=-1):
        if end==-1 and start==-1:
            return
        if end==-1:
            while len(self.formatted_text_cache) > start:
                self.formatted_text_cache.pop()
        elif start==-1:
            while end > 0:
                self.formatted_text_cache.pop(0)
                end -= 1
        else:
            while start <= end:
                self.formatted_text_cache.pop(start)
                end -= 1            
    def insert_formatted_text_cache(self, formatted_text, i):
        self.formatted_text_cache.insert(i, formatted_text)
    def print_line(self, num):
	i = num + self.line_start
        try:
            formatted_text = self.formatted_text_cache[i]
            if i > 0:
                cache = self.formatted_text_cache[i - 1]
                if cache.text[-1].state != formatted_text.text[0].state:
                    formatted_text = self.update_formatted_text_cache(i)
        except:
            l = len(self.formatted_text_cache)
            if l==i:
                formatted_text = self.create_proper_formatted_text(i)
                self.formatted_text_cache.append(formatted_text)                    
            else:
                for x in range(l, i + 1):
                    formatted_text = self.create_proper_formatted_text(x)
                    self.formatted_text_cache.append(formatted_text)
        self.textview.draw_formatted_text(formatted_text, num, self.y_offset)
	return i
    def expose_event(self, start, end):
        if self.buffer:
            if start < 0:
                start = 0
            if end > self.textview.line_count:
                end = self.textview.line_count
            for counter in range(start, end + 1):
              	try:
       	            i = self.print_line(counter)
		except:
		    break
    def redraw_long_lines(self, width, draw):
        for counter in range(0, self.textview.line_count):
            try:
                formatted_text = self.formatted_text_cache[counter + self.line_start]
                text_width = 0
                for word in formatted_text.text:
                    text_width += word.width
                if text_width > width - 2 * TEXT_BORDER:
                    self.textview.draw_formatted_text(formatted_text, counter, self.y_offset)
            except:
                break
    def invalidate_rect(self, x, y, width, height):
        r = gtk.gdk.Rectangle(x, y, width, height)
        self.textview.drawable.window.invalidate_rect(r, False)
    def cursor_pos_correct_y(self):
        # correct cursor pos y
        line_count = self.textview.line_count - 1
        l = len(self.buffer)
        if line_count >= l:
            line_count = l - 1
        if self.textview.cursor_y < 0:
            self.textview.cursor_y = 0
        if self.textview.cursor_y > line_count:
            self.textview.cursor_y = line_count
    def cursor_pos_correct_x(self):
        # correct cursor pos x
        l = len(self.buffer[self.textview.cursor_y + self.line_start])
        if self.textview.cursor_x < 0:
            self.textview.cursor_x = 0
        if self.textview.cursor_x >= l:
            self.textview.cursor_x = l        
    def check_and_scroll_y(self):
        dy = self.textview.cursor_y
        self.cursor_pos_correct_y()
        dy -= self.textview.cursor_y
        l = len(self.buffer)
	if self.line_start + self.textview.cursor_y + dy + 1 > l:
	    dy = l - self.line_start - self.textview.cursor_y - 1
	    if dy < 0:
		self.textview.cursor_y += dy
		dy = 0
        if dy:
            self.line_start += dy
            if self.line_start < 0:
                dy -= self.line_start
                self.line_start = 0
            self.cursor_pos_correct_x()
            if dy != 0:   
		error = 0 
		if dy > 0:
		    if self.change_allign(ALLIGN_BOTTOM):
			error = self.y_offset
		else:
		    error = -self.y_offset
		    self.change_allign(ALLIGN_TOP)
            	self.textview.draw.scroll_dy(-dy * self.textview.line_height + error, 0, TEXT_BORDER, self.textview.drawable_width, self.textview.drawable_height - 2 * TEXT_BORDER)
                if dy > 0:
                    y = self.textview.line_count
                    self.expose_event(y - dy, y)
                else:
                    self.expose_event(0, -dy)
                self.invalidate_rect(0, 0, self.textview.drawable_width, self.textview.drawable_height)
                self.textview.lineview.set_start_number(self.line_start + 1)
        else:
            self.cursor_pos_correct_x()
        return dy
    def cursor_pos_y_changed(self):
        y = self.textview.cursor_old_y
        self.expose_event(y, y)
        scroll = self.check_and_scroll_y()
        if not scroll:
            y *= self.textview.line_height
            self.invalidate_rect(0, y + self.y_offset + TEXT_BORDER, self.textview.drawable_width, self.textview.line_height)
            y = self.textview.cursor_y
            self.expose_event(y, y)
            y *= self.textview.line_height
            self.invalidate_rect(0, y + self.y_offset + TEXT_BORDER, self.textview.drawable_width, self.textview.line_height)
        gtk.gdk.window_process_all_updates()
    def cursor_pos_x_changed(self):
        self.cursor_pos_correct_x()
        y = self.textview.cursor_y
        height = self.textview.line_height
        self.expose_event(y, y)
        y *= self.textview.line_height
        self.invalidate_rect(0, y + self.y_offset + TEXT_BORDER, self.textview.drawable_width, height)
        gtk.gdk.window_process_all_updates()
    def scroll_event(self, widget, event, data=None):
        if event.direction==gtk.gdk.SCROLL_UP:
            self.textview.cursor_y_rel(-1)
            self.textview.cursor_x = self.cursor_real_x
            self.cursor_pos_y_changed()
        elif event.direction==gtk.gdk.SCROLL_DOWN:
            self.textview.cursor_y_rel(1)
            self.textview.cursor_x = self.cursor_real_x
            self.cursor_pos_y_changed()
    def keypress_event(self, widget, event, data=None):
        key_name = gtk.gdk.keyval_name(event.keyval)
        if key_name == "Up":
            self.textview.cursor_y_rel(-1)
            self.textview.cursor_x = self.cursor_real_x
            self.cursor_pos_y_changed()
        elif key_name == "Down":
            self.textview.cursor_y_rel(1)
            self.textview.cursor_x = self.cursor_real_x
            self.cursor_pos_y_changed()
        elif key_name == "Page_Up":
            self.textview.cursor_y_rel(-self.textview.line_count / 2)
            self.textview.cursor_x = self.cursor_real_x
            self.cursor_pos_y_changed()
        elif key_name == "Page_Down":
            self.textview.cursor_y_rel(self.textview.line_count / 2)
            self.textview.cursor_x = self.cursor_real_x
            self.cursor_pos_y_changed()
        elif key_name == "Left":
            self.textview.cursor_x_rel(-1)
            change_y = self.textview.cursor_x
            self.cursor_pos_correct_x()
            change_y -= self.textview.cursor_x
            if change_y and (self.textview.cursor_y + self.line_start):
                self.textview.cursor_y_rel(-1)
                self.textview.cursor_x_set(0x7FFF)
                self.cursor_pos_y_changed()
                self.cursor_real_x = self.textview.cursor_x
            else:
                self.cursor_pos_x_changed()
                self.cursor_real_x = self.textview.cursor_x
        elif key_name == "Right":
            self.textview.cursor_x_rel(1)
            change_y = self.textview.cursor_x
            self.cursor_pos_correct_x()
            change_y -= self.textview.cursor_x
            if change_y:
                self.textview.cursor_x_set(0)
                self.cursor_real_x = 0
                self.textview.cursor_y_rel(1)
                self.cursor_pos_y_changed()
            else:
                self.cursor_real_x = self.textview.cursor_x
                self.cursor_pos_x_changed()
        elif key_name == "Home":
            self.textview.cursor_x_set(0)
            self.cursor_real_x = 0
            self.cursor_pos_x_changed()
        elif key_name == "End":
            self.textview.cursor_x_set(0x7FFF)
            self.cursor_real_x = 0x7FFF
            self.cursor_pos_x_changed()
        elif key_name == "BackSpace":
            pos = self.line_start + self.textview.cursor_y
            if self.textview.cursor_x==0:
                if pos > 0:
                    s = self.buffer.remove_string(pos)
                    self.cursor_real_x = len(self.buffer[pos - 1])
                    self.textview.cursor_x_set(self.cursor_real_x)
                    if len(s):
                        self.buffer[pos - 1].append_string(s)
                    self.delete_formatted_text_cache(start = pos - 1, end = pos - 1)
                    self.update_formatted_text_cache(pos - 1)
                    self.textview.cursor_y_rel(-1)
                    self.cursor_pos_correct_x()
                    y = self.textview.cursor_old_y * self.textview.line_height + self.y_offset + TEXT_BORDER
                    self.textview.draw.scroll_dy(-self.textview.line_height, 0, y, self.textview.drawable_width, self.textview.drawable_height - y)
                    self.expose_event(self.textview.line_count - 1, self.textview.line_count)
                    self.invalidate_rect(0, y, self.textview.drawable_width, self.textview.drawable_height - y)
                    self.cursor_pos_x_changed()
            else:
                self.buffer[pos].remove(self.textview.cursor_x - 1)
                self.update_formatted_text_cache(pos)
                self.textview.cursor_x_rel(-1)
                self.cursor_pos_x_changed()
		self.cursor_real_x = self.textview.cursor_x
        elif key_name == "Delete":
            pos = self.line_start + self.textview.cursor_y
            if len(self.buffer[pos]) == self.textview.cursor_x:
                if len(self.buffer) > pos + 1:
                    s = self.buffer.remove_string(pos + 1)
                    if len(s):
                        self.buffer[pos].append_string(s)
                    self.delete_formatted_text_cache(start = pos + 1, end = pos + 1)
                    self.update_formatted_text_cache(pos)
                    y = self.textview.cursor_y * self.textview.line_height + self.y_offset + TEXT_BORDER
                    self.textview.draw.scroll_dy(-self.textview.line_height, 0, y, self.textview.drawable_width, self.textview.drawable_height - y)
                    self.expose_event(self.textview.line_count - 1, self.textview.line_count)
                    self.invalidate_rect(0, y, self.textview.drawable_width, self.textview.drawable_height - y)
                    self.cursor_pos_x_changed()
            else:
                self.buffer[pos].remove(self.textview.cursor_x)
                self.update_formatted_text_cache(pos)
                self.cursor_pos_x_changed()
	elif key_name == "Return" or key_name == "KP_Enter":
	    pos = self.line_start + self.textview.cursor_y
            s = lib.text.String(u"")
            if self.textview.cursor_x < len(self.buffer[pos]):
                for i in range(self.textview.cursor_x, len(self.buffer[pos])):
                    s.append(self.buffer[pos].remove(self.textview.cursor_x))
            self.buffer.insert_string(pos + 1, s)
            self.update_formatted_text_cache(pos)
            self.insert_formatted_text_cache(self.create_proper_formatted_text(pos + 1), pos + 1)
            y = self.textview.cursor_y * self.textview.line_height + self.y_offset + TEXT_BORDER
            self.textview.draw.scroll_dy(self.textview.line_height, 0, y, self.textview.drawable_width, self.textview.drawable_height - y)
            self.invalidate_rect(0, y, self.textview.drawable_width, self.textview.drawable_height - y)
            self.textview.cursor_y_rel(1)
            self.textview.cursor_x_set(0)
            self.cursor_real_x = 0
            self.cursor_pos_y_changed()
        elif key_name == "Tab":
            p = lib.core.share_data.get_data('conf.editor.indentation')
            pos = self.line_start + self.textview.cursor_y
            align = p.indent_width - self.textview.cursor_x % p.indent_width
            for i in range(align):
                self.buffer[pos].insert(self.textview.cursor_x, u' ')
            self.update_formatted_text_cache(pos)
            self.textview.cursor_x_rel(align)
            self.cursor_real_x = self.textview.cursor_x
            self.cursor_pos_x_changed()
        elif key_name == "Escape":
            pass
        elif event.string:
            char = unicode(event.string, lib.core.share_data.get_data('env.encoding'))
            pos = self.line_start + self.textview.cursor_y
            self.buffer[pos].insert(self.textview.cursor_x, char)
            self.update_formatted_text_cache(pos)
            self.textview.cursor_x_rel(1)
            self.cursor_real_x = self.textview.cursor_x
            self.cursor_pos_x_changed()

class TextView(gtk.HBox):
    def __init__(self):
	gtk.HBox.__init__(self)
        self.gc = gtk.gdk.get_default_root_window().new_gc()
        self.gc.set_rgb_fg_color(gtk.gdk.Color(0, 0, 0))
        self.set_redraw_on_allocate(False)
	self.lineview = widget.lineview.LineView()
        self.connect('delete-event', self.delete_event)
        lib.core.events.connect(self, 'conf.editor.lineview', self.lineview_state_changed)
        lib.core.events.connect(self, 'conf.editor.font', self.font_changed)
        lib.core.events.connect(self, 'conf.editor.colors', self.colors_changed)
        lib.core.events.connect(self, 'conf.editor.general', self.cursor_shape_changed)
        self.lineview_state_changed()
	self.pack_start(self.lineview, False, False, 0)
        self.drawable = gtk.DrawingArea()
        self.drawable.set_double_buffered(False)
        self.drawable.set_app_paintable(True)
        self.drawable.set_redraw_on_allocate(True)
        self.drawable.connect('expose-event', self.drawable_expose_event)
        self.drawable.connect('configure-event', self.drawable_configure_event)
        self.drawable.connect('map', self.drawable_map_event)
        self.drawable.connect('unmap', self.drawable_unmap_event)
        self.connect("key-press-event", self.keypress_event)
        self.drawable.set_events(gtk.gdk.EXPOSURE_MASK)
        self.drawable.show()
        self.pack_start(self.drawable, True, True, 0)
        self.drawable_width = 0
        self.drawable_height = 0
        self.map = False
        self.need_full_draw = True
        self.draw==None
        self.font_name = ''
        self.line_height = 1
        self.line_count = 1
        self.create_renderer()
        self.set_font()
        self.attrs=None
        self.cursor_x = 0
        self.cursor_y = 0
        self.cursor_old_x = 0
        self.cursor_old_y = 0
        self.text_controller = TextController(self, self.draw.get_text_width)
    def cursor_x_set(self, new_x):
        self.cursor_old_x = self.cursor_x
        self.cursor_x = new_x
    def cursor_y_set(self, new_y):
        self.cursor_old_y = self.cursor_y
        self.cursor_y = new_y
    def cursor_x_rel(self, dx):
        self.cursor_old_x = self.cursor_x
        self.cursor_x += dx
    def cursor_y_rel(self, dy):
        self.cursor_old_y = self.cursor_y
        self.cursor_y += dy
    def get_text_controller(self):
        return self.text_controller
    def set_color_attrs(self, name):        
        self.syntax_name = name
        self.attrs = lib.core.share_data.get_data("conf.editor.colors.%s" % self.syntax_name, True)
        self.gc.set_rgb_fg_color(self.attrs.Background.color)
    def draw_text_line_raw(self, x, y, s, show_cursor, cursor_pos=-1):
        dx = 0
        l = len(s)
        if show_cursor and cursor_pos >= 0 and cursor_pos <= l:
            p = lib.core.share_data.get_data('conf.editor.general')
            if cursor_pos==l:
                dx = self.draw.draw_text(x, y, s)
                getattr(self, "draw_cursor_%i" % p.cursor_shape)(x + dx, y, ' ')
            else:    
                dx = self.draw.draw_text(x + dx, y, s[0:cursor_pos])
                dx += getattr(self, "draw_cursor_%i" % p.cursor_shape)(x + dx, y, s[cursor_pos])
                dx += self.draw.draw_text(x + dx, y, s[cursor_pos + 1:])
        else:
            dx = self.draw.draw_text(x, y, s)
        return dx
    def draw_formatted_text_raw(self, x, y, formatted_text, show_cursor, cursor_pos=-1):
        dx = 0
        text = ''
        font_style = None
        for word in formatted_text.text:
            if font_style==None:
                font_style = word.attr.font_style
                text = word.string
                dx = word.width
            else:
                if font_style!=word.attr.font_style:
                    self.draw.set_font_style(font_style)
                    self.draw_text_line_raw(x, y, text, show_cursor, cursor_pos)
                    cursor_pos -= len(text)
                    font_style = word.attr.font_style
                    text = word.string
                    x += dx
                    dx = word.width
                else:
                    text += word.string
                    dx += word.width
        if len(text) > 0:
            self.draw.set_font_style(font_style)
            self.draw_text_line_raw(x, y, text, show_cursor, cursor_pos)
    def draw_formatted_text(self, formatted_text, line_num, y_offset=0):
        x = TEXT_BORDER
        y = TEXT_BORDER + self.line_height * line_num + y_offset
        delta = (self.drawable_height - TEXT_BORDER) - y
        if delta > self.line_height:
            delta = self.line_height
        elif delta <= 0:
            return False
        self.draw.set_clip_rect(gtk.gdk.Rectangle(TEXT_BORDER, y, gtk.gdk.screen_width() - 2 * TEXT_BORDER, delta))
        self.draw.set_color_gdk(self.attrs.Background.color)
        self.draw.clear()
        x_offset = 0
        if self.cursor_y==line_num:
            # calculate x offset
            cursor_x = self.cursor_x
            text = None
            for word in formatted_text.text:
                s = word.string
                if len(s) < cursor_x:
                    cursor_x -= len(s)
                    x_offset += word.width
                else:
                    text = s
                    break
            if text:
                x_offset += self.draw.get_text_size(text[0:cursor_x + 1])[0] + 2 * TEXT_BORDER
            if x_offset > self.drawable_width:
                x_offset -= self.drawable_width
                text_len = -1
                for word in formatted_text.text:
                    text_len += len(word.string)
                if text_len == self.cursor_x:
                    x_offset += self.draw.get_text_size(' ')[0]
            else:
                x_offset = 0
        dx = -x_offset
        self.draw.push_group()
        for word in formatted_text.text:
                self.draw.set_color_gdk(word.attr.color)
                self.draw.draw_rectangle(True, x + dx, y, word.width, self.line_height)
                dx += word.width
        # draw cursor at the end of line
        self.draw.set_color_gdk(self.attrs.Text.color)
        width, height = self.draw.get_text_size(' ')
        self.draw.draw_rectangle(True, x + dx, y, width, self.line_height)
        self.draw.pop_group_to_source()
        self.draw_formatted_text_raw(x - x_offset, y, formatted_text, line_num==self.cursor_y, self.cursor_x)
        self.draw.set_clip_rect()
        return True
    def draw_cursor_0(self, x, y, letter):
        width, height = self.draw.get_text_size(letter)
        self.draw.draw_rectangle(True, x, y, width, height)
        pattern = self.draw.get_source_pattern()
        self.draw.set_font_color_gdk(self.attrs.Background.color)
        width = self.draw.draw_text(x, y, letter)
        self.draw.set_source_pattern(pattern)
        return width
    def draw_cursor_1(self, x, y, letter):
        width, height = self.draw.get_text_size(letter)
        self.draw.draw_rectangle(True, x, y + height - 2, width, 2)
        return self.draw.draw_text(x, y, letter)
    def draw_cursor_2(self, x, y, letter):
        width, height = self.draw.get_text_size(letter)
        self.draw.draw_rectangle(True, x, y, 2, height)
        return self.draw.draw_text(x, y, letter)
    def clear_drawable(self, x = -1, y = -1, width = -1, height = -1):
        if x == -1:
            x = 0
        if y == -1:
            y = 0
        if width == -1:
            width = gtk.gdk.screen_width()
        if height == -1:
            height = gtk.gdk.screen_height()
        self.draw.set_clip_rect(gtk.gdk.Rectangle(x, y, width, height))
        self.draw.set_color_gdk(self.attrs.Background.color)
#        self.draw.draw_rectangle(True, 0, 0, self.drawable_width, self.drawable_height)
        self.draw.clear() # - to jest lepsze
    def modify_line_count(self, drawable_height):
        self.line_count = (drawable_height - 2 * TEXT_BORDER) / self.line_height
        if self.line_count <= 0:
            self.line_count = 1
    def drawable_expose_event(self, widget, event):
        if self.drawable_height != 0 and self.map:
            if self.need_full_draw:
                self.need_full_draw = False
                self.clear_drawable()
                self.text_controller.expose_event(0, self.line_count)
            self.draw.set_clip_rect(event.area)
            self.draw.flip(widget.window)
            # Draw border
            widget.window.draw_rectangle(self.gc, True, 0, 0, self.drawable_width, TEXT_BORDER)
            widget.window.draw_rectangle(self.gc, True, self.drawable_width - TEXT_BORDER, 0, TEXT_BORDER, self.drawable_height)
            widget.window.draw_rectangle(self.gc, True, 0, self.drawable_height - TEXT_BORDER, self.drawable_width, TEXT_BORDER)
            widget.window.draw_rectangle(self.gc, True, 0, 0, TEXT_BORDER, self.drawable_height)
    def drawable_configure_event(self, widget, event):
        if event.width > self.drawable_width:
            self.clear_drawable(self.drawable_width, 0)
        tmp = self.drawable_width
        self.drawable_width = event.width        
        self.text_controller.redraw_long_lines(min(tmp, event.width), self.draw)
        if self.map and self.drawable_height != event.height:
            self.modify_line_count(event.height)
            if not self.need_full_draw and self.drawable_height < event.height:
                tmp = self.drawable_height
                self.clear_drawable(0, tmp, -1, event.height - tmp)
                self.drawable_height = event.height
                self.text_controller.expose_event(tmp / self.line_height - 1, event.height / self.line_height)
            else:
                self.drawable_height = event.height
                self.text_controller.cursor_pos_y_changed()
    def delete_event(self, widget, event, data=None):
        lib.core.events.disconnect(self)
        self.lineview.emit("delete-event", event) # emit this event further
    def keypress_event(self, widget, event, data=None):
	if self.map:
    	    self.text_controller.keypress_event(widget, event, data)
    def drawable_map_event(self, widget, data=None):
        self.map = True
    def drawable_unmap_event(self, widget, data=None):
        self.map = False
    def lineview_state_changed(self, share = None, sig_name = None, data = None):
        p = lib.core.share_data.get_data('conf.editor.lineview')
	if p.enabled:
	    self.lineview.show()
	else:
	    self.lineview.hide()
    def set_font(self, force=False):
        p = lib.core.share_data.get_data('conf.editor.font')
        if self.font_name != p.font_name or force:
            self.draw.set_font(p.font_name)
            width, height = self.draw.get_text_size('0')
            self.line_height = height
            self.modify_line_count(self.drawable_height)
            self.font_name = p.font_name
    def create_renderer(self):
        self.draw = None
        gc.collect()
	self.draw = lib.draw.CairoDraw()
        self.draw.create_size(gtk.gdk.screen_width(), gtk.gdk.screen_height())
    def font_changed(self, share_data=None, param=None, data=None):
        self.set_font()
        if self.map:    
            self.queue_draw()
        self.need_full_draw = True
        self.text_controller.delete_formatted_text_cache(start = 0)
    def colors_changed(self, share_data=None, param=None, data=None):
        if self.map:
            self.queue_draw()
        self.need_full_draw = True
        self.set_dfa_parser(self.syntax_name)
    def cursor_shape_changed(self, share_data=None, param=None, data=None):
        self.text_controller.expose_event(self.cursor_y, self.cursor_y)
