# -*- coding: iso-8859-2 -*-

#	Programmer:	Piotr Borek
#	E-mail:		piotrborek@op.pl
#
#	Copyright 2006  Piotr Borek
#
#	Distributed under the terms of the GPL (GNU Public License)
#
#    pyX4 is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import lib.core
import lib.syntax.dfa
import lib.syntax.parser
import widget.buttons

from lib.splash import SplashWindow
SplashWindow.pulse_text_import("widget.view")

import gtk;
import gobject

class SyntaxView(gtk.VBox):
    def __init__(self, syntax_name, colors_dict, text):
        gtk.VBox.__init__(self)
        p = lib.core.share_data.get_data('conf.editor.colors.%s' % syntax_name, True)
        self.background_color = p.Background.color
        self.text_color = p.Text.color
        self.colors_dict = colors_dict
        dfa = lib.syntax.dfa.DFA()
        dfa.set_syntax(syntax_name)
        self.syntax_parser = lib.syntax.parser.SyntaxParser(dfa)
        self.lines = text.split('\n')
        self.groups = {'Background' : []}
        self.init()
        self.status = gtk.Statusbar()
        self.status.show()
        self.pack_start(self.status, False, False)        
        self.callback = None
        self.active = ""
    def set_callback(self, callback):
        self.callback = callback
    def set_color(self, color_name, gdk_color):
        if color_name=='Background':
            list = self.groups[color_name]
            for widget in list:
                widget.set_background_color(gdk_color)
        else:
            list = self.groups[color_name]
            for widget in list:
                widget.set_text_color(gdk_color)
    def get_color(self, color_name):
        list = self.groups[color_name]
        if color_name=='Background':    
            return list[0].get_background_color()
        else:
            return list[0].get_text_color()
    def set_font_style(self, color_name, font_style):
        if color_name!='Background':
            list = self.groups[color_name]
            for widget in list:
                widget.set_font_style(font_style)    
    def get_font_style(self, color_name):
        list = self.groups[color_name]
        if color_name!='Backround':
            return list[0].get_font_style()
        else:
            return lib.draw.FONT_STYLE_NORMAL
    def init(self):
        last_state = None
        last_attr = None
        for x in self.lines:
            formatted_text = self.syntax_parser.create_formatted_text(x, last_state, last_attr)
            last_state = formatted_text.text[-1].state
            last_attr = formatted_text.text[-1].attr
            self.create_line(formatted_text)
    def add_to_group(self, color_name, button):
        try:
            list = self.groups[color_name]
        except:
            list = []
            self.groups[color_name] = list
        list.append(button)
        self.groups['Background'].append(button)
    def find_color_name(self, attr):
        for keys in self.colors_dict.keys():
            if attr==self.colors_dict[keys]:
                return keys
    def create_line(self, formatted_text):
        hbox = gtk.HBox()
        hbox.show()
        for x in formatted_text.text:
            if len(x.string) > 0:
                b = widget.buttons.SimpleButton(x.string)
                b.set_font("Monospace 9")
                b.set_font_style(x.attr.font_style)
                b.set_colors(x.attr.color, self.background_color)
                b.show()
                color_name = self.find_color_name(x.attr)
                self.add_to_group(color_name, b)
                b.connect('enter-notify-event', self.enter_notify_event, color_name)
                b.connect('leave-notify-event', self.leave_notify_event)
                b.connect('button-press-event', self.clicked_event)
                hbox.pack_start(b, False, False)
        b = widget.buttons.SimpleButton()
        b.set_font("Monospace 9")
        b.set_colors(self.text_color, self.background_color)
        self.add_to_group(color_name, b)
        b.connect('enter-notify-event', self.enter_notify_event, 'Background')
        b.connect('leave-notify-event', self.leave_notify_event)
        b.connect('button-press-event', self.clicked_event)
        b.show()
        hbox.pack_start(b, True, True)
        self.pack_start(hbox, False, False)
    def enter_notify_event(self, widget, event, data=None):
        self.active = data
        self.status.push(0, _("Color: ") + data)
    def leave_notify_event(self, widget, event, data=None):
        try:
            self.status.pop(0)
        except:
            pass
    def clicked_event(self, widget, data=None):
        if self.callback:
            self.callback(self.active)
